#include <fcntl.h>
#include <signal.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include "HPDI32DocSrcLib.h"

static int	_fd;

static void handle_sigio(int signo)
{
	hpdi32_interrupt_t	data;
	int					status;

	status	= ioctl(_fd, HPDI32_IOCTL_INT_STATUS, &data);

	if (status)
	{
		/* The request failed. */
	}
	else if (data.status.gsc & HPDI32_INT_GSC_TX_FIFO_AE)
	{
		/* Perform actions specific to this interrupt. */
	}
	else if (data.status.gsc & HPDI32_INT_GSC_RX_FIFO_AF)
	{
		/* Perform actions specific to this interrupt. */
	}
}

int hpdi32_async_setup(int fd)
{
	hpdi32_interrupt_t	data;
	int					flags;
	pid_t				pid;
	hpdi32_reg_t		reg;
	int					status;

	_fd	= fd;
	signal(SIGIO, handle_sigio);
	pid	= getpid();
	fcntl(fd, F_SETOWN, pid);
	flags	= fcntl(fd, F_GETFL);
	flags	|= FASYNC;
	fcntl(fd, F_SETFL, flags);
	data.notify.gsc		= HPDI32_INT_GSC_TX_FIFO_AE
						| HPDI32_INT_GSC_RX_FIFO_AF;
	data.notify.other	= 0;
	status		= ioctl(fd, HPDI32_IOCTL_INT_NOTIFY, &data);
	reg.value	= HPDI32_INT_GSC_TX_FIFO_AE
				| HPDI32_INT_GSC_RX_FIFO_AF;
	reg.mask	= HPDI32_INT_GSC_TX_FIFO_AE
				| HPDI32_INT_GSC_RX_FIFO_AF;

	if (status == 0)
	{
		reg.reg	= HPDI32_GSC_IELR;	/* Edge Triggered */
		status	= ioctl(fd, HPDI32_IOCTL_REG_MOD, &reg);
	}

	if (status == 0)
	{
		reg.reg		= HPDI32_GSC_IHLR;	/* Rising Edge */
		ioctl(fd, HPDI32_IOCTL_REG_MOD, &reg);
	}

	if (status == 0)
	{
		reg.reg		= HPDI32_GSC_ICR;	/* Enable */
		ioctl(fd, HPDI32_IOCTL_REG_MOD, &reg);
	}

	if (status)
	{
		data.notify.gsc		= 0;
		data.notify.other	= 0;
		ioctl(fd, HPDI32_IOCTL_INT_NOTIFY, &data);
	}

	return(status);
}
