/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 1 $ */
/* $Date: 1/14/05 8:04a $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32_Linux_Ver.1.09.0-GSC-DN/Linux 2.2, 2.4 kernels/driver/close.c $
 *
 * 1     1/14/05 8:04a Dnewbold
*/

#include "main.h"



/******************************************************************************
*
*	Function:	close
*
*	Purpose:
*
*		Implement the release() procedure.
*
*	Arguments:
*
*		inode	This is the device inode structure.
*
*		filp	This is the file structure.
*
*	Returned:
*
*		0	All went well.
*		< 0	There was a problem.
*
******************************************************************************/

int close(struct inode* inode, struct file* filp)
{
	dev_data_t*	dev;
	int		status;
	int		test;

	for (;;)	/* We'll use a loop for convenience. */
	{
		dev	= dev_locate(inode);

		if (dev == NULL)
		{
			/* The referenced device doesn't exist. */
			status	= -ENODEV;
			break;
		}

		test	= down_interruptible(&dev->sem);

		if (test)
		{
			/* We didn't get the semaphore. */
			status	= -ERESTARTSYS;
			break;
		}

		/* Close the device. */
		fasync_close_dev(dev, filp);
		mmap_close_dev(dev);
		dev_reset(dev);
		free_irq(dev->irq.number, dev);
		io_close_dev(dev);
		dev_data_t_init(dev);
		status		= 0;
		dev->in_use	= 0;
		MOD_DEC_USE_COUNT;
		up(&dev->sem);
		break;
	}

	return(status);
}


