/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 1 $ */
/* $Date: 1/14/05 8:04a $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32_Linux_Ver.1.09.0-GSC-DN/Linux 2.2, 2.4 kernels/driver/fasync.c $
 *
 * 1     1/14/05 8:04a Dnewbold
*/

#include "main.h"



/******************************************************************************
*
*	Function:	fasync
*
*	Purpose:
*
*		Implement the fasync() procedure.
*
*	Arguments:
*
*		fd	The file descriptor for which a change is being
*			applied.
*
*		filp	The kernel file structure for the open device.
*
*		mode	The change being made.
*
*	Returned:
*
*		0	All went well, though no data was transferred.
*		> 0	This is the number of bytes transferred.
*		< 0	There was an error.
*
******************************************************************************/

int fasync(int fd, struct file* filp, int mode)
{
	dev_data_t*	dev;
	int		status;
	int		test;

	for (;;)	/* We'll use a loop for convenience. */
	{
		/*
		*	Validate some of the parameters.
		*/

		dev	= (dev_data_t*) filp->private_data;

		if (dev == NULL)
		{
			/* The referenced device doesn't exist. */
			status	= -ENODEV;
			break;
		}

		/*
		*	Gain the neccessary access.
		*/

		test	= down_interruptible(&dev->sem);

		if (test)
		{
			/* We got a signal rather than the semaphore. */
			status	= -ERESTARTSYS;
			break;
		}

		test	= down_interruptible(&dev->irq.sem);

		if (test)
		{
			/* We got a signal rather than the IRQ semaphore. */
			up(&dev->sem);
			status	= -ERESTARTSYS;
			break;
		}

		/*
		*	Perform the work.
		*/

		status	= fasync_helper(fd, filp, mode, &dev->irq.async);
		up(&dev->irq.sem);
		up(&dev->sem);
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	fasync_close_dev
*
*	Purpose:
*
*		Perform any fasync work needed when closing a device.
*
*	Arguments:
*
*		dev	The device of interest.
*
*		filp	The file structure associated with the access.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void fasync_close_dev(dev_data_t* dev, struct file* filp)
{
	fasync_helper(-1, filp, 0, &dev->irq.async);
}


