/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 12 $ */
/* $Date: 9/28/06 5:42p $ */
/*
*	This header defines the application interface to the HPDI32 driver.
*/
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.17.0-GSC-DN/driver/hpdi32.h $
 *
 * 12    9/28/06 5:42p Dnewbold
 * Added I/O Data Size and I/O PIO Threshold parameters.
 *
 * 11    2/22/06 10:20a Dnewbold
 * Added DIPHASE2 support.
 *
 * 10    2/01/06 2:55p Dnewbold
 * Increased the value of the macro HPDI32_IO_SAMPLES_MAX.
 *
 * 9     3/29/04 10:24a Dnewbold
 * Made GP, plus added and updated some macros.
 *
 * 7     4/28/03 6:16p Dnewbold
 * Correct a few PCI9656 registers definitions and add FRR field #defines.
 *
 * 6     6/21/02 3:46p Dnewbold
 * Enhanced the HPDI32_ENCODE() macro and added #define for
 * HPDI32_GSC_BCR_TC_DMDMA0.
 *
 * 5     6/05/02 1:39p Dnewbold
 * Corrected a typo.
 *
 * 4     6/03/02 1:06p Dnewbold
 * Added support for the PCI64-HPDI32.
 *
 * 3     5/24/02 5:01p Dnewbold
 * Added mmap() support.
 *
 * 2     4/24/02 4:57p Dnewbold
 * Cleanup. Enhanced interrupt notification. Added buf pointer to
 * hpdi32_io_config_t.
*/

#ifndef __HPDI32_H__
#define __HPDI32_H__

#include <asm/types.h>
#include <linux/ioctl.h>
#include <linux/types.h>



/* #defines	**************************************************************/

#define	HPDI32_VENDOR_ID	0x10B5	/* PLX Technologies */
#define	HPDI32_SUBVENDOR_ID	0x10B5	/* PLX assigned subsystem device id */
/* The 32-bit board. */
#define	HPDI32_DEVICE_ID_32	0x9080	/* PCI9080 PCI interface chip */
#define	HPDI32_SUBDEVICE_ID_32	0x2400	/* HPDI32: 32-bit PCI */
/* The 64-bit board. */
#define	HPDI32_DEVICE_ID_64	0x9656	/* PCI9656 PCI interface chip */
#define	HPDI32_SUBDEVICE_ID_64	0x2705	/* HPDI32: 64-bit PCIX */



/* These are used for debug purposes. */

#define	HPDI32_DEBUG_DATA_VALUES	32



/* These are used to define the possible I/O read and write modes. */

#define	HPDI32_IO_MODE_PIO		0
#define	HPDI32_IO_MODE_DMA		1
#define	HPDI32_IO_MODE_DMDMA		2
#define	HPDI32_IO_MODE_DEFAULT		HPDI32_IO_MODE_PIO
#define	HPDI32_IO_MODE_NO_CHANGE	(-1)



/* These are used to define the PIO Threshold options. */

#define	HPDI32_IO_PIO_THRESHOLD_MIN		0
#define	HPDI32_IO_PIO_THRESHOLD_DEFAULT		64
#define	HPDI32_IO_PIO_THRESHOLD_NO_CHANGE	(-1)



/* These are used to define the possible I/O sample sizes. */

#define	HPDI32_IO_SAMPLE_BITS_8		8
#define	HPDI32_IO_SAMPLE_BITS_16	16
#define	HPDI32_IO_SAMPLE_BITS_32	32
#define	HPDI32_IO_SAMPLE_BITS_DEFAULT	HPDI32_IO_SAMPLE_BITS_32
#define	HPDI32_IO_SAMPLE_BITS_NO_CHANGE	(-1)



/* These are used to define the possible I/O read and write block sizes. */

#define	HPDI32_IO_SAMPLES_MIN		(128)
#define	HPDI32_IO_SAMPLES_DEFAULT	256		/* samples */
#define	HPDI32_IO_SAMPLES_MAX		(512L * 1024)	/* xfer block limit */
#define	HPDI32_IO_SAMPLES_NO_CHANGE	0



/* These are used to define the possible I/O read and write timeouts. */

#define	HPDI32_IO_TIMEOUT_MIN		0
#define	HPDI32_IO_TIMEOUT_DEFAULT	5
#define	HPDI32_IO_TIMEOUT_MAX		3600	/* One hour. */
#define	HPDI32_IO_TIMEOUT_NO_CHANGE	(-1)



/* IOCTL services. */
#define	HPDI32_IOCTL_NO_COMMAND		_IO('s', 0)
#define HPDI32_IOCTL_DEBUG_DATA_GET	_IOR('s', 1, hpdi32_debug_data_t)
#define	HPDI32_IOCTL_DRIVER_INFO_GET	_IOR('s', 2, hpdi32_driver_info_t)
#define	HPDI32_IOCTL_REG_READ		_IOWR('s', 3, hpdi32_reg_t)
#define	HPDI32_IOCTL_REG_WRITE		_IOW('s', 4, hpdi32_reg_t)
#define	HPDI32_IOCTL_REG_MOD		_IOW('s', 5, hpdi32_reg_t)
#define	HPDI32_IOCTL_IO_CONFIG		_IOWR('s', 6, hpdi32_io_config_t)
#define	HPDI32_IOCTL_INT_NOTIFY		_IOWR('s', 7, hpdi32_interrupt_t)
#define	HPDI32_IOCTL_INT_STATUS		_IOWR('s', 8, hpdi32_interrupt_t)
#define	HPDI32_IOCTL_MMAP_INFO		_IOR('s', 9, hpdi32_mmap_t)



/* Register definitions. */

#define	HPDI32_DECODE(r,b,e)	(((r)>>(e)) & (0xFFFFFFFFUL >> (31-((b)-(e)))))
#define	HPDI32_ENCODE(v,b,e)	(((v) << (e)) & \
				((~0UL << (e)) & (~0UL >> (31 - (b)))))

#define	HPDI32_REG_ENCODE(t,s,o)(((t) & 0xC00) | \
				HPDI32_ENCODE((s) - 1,9,8) | \
				HPDI32_ENCODE(o,7,0))

#define	HPDI32_REG_OFFSET(r)	HPDI32_DECODE(r,7,0)
#define	HPDI32_REG_SIZE(r)	(HPDI32_DECODE(r,9,8) + 1)
#define	HPDI32_REG_TYPE(r)	((r) & 0xC00)

#define	HPDI32_PCI_REGISTER	0x000
#define	HPDI32_PLX_REGISTER	0x400
#define	HPDI32_GSC_REGISTER	0x800



/* PCI Configuration Registers: PCI9080 and PCI9656 */
#define HPDI32_PCI_IDR		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 4, 0x00)
#define HPDI32_PCI_CR		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 2, 0x04)
#define HPDI32_PCI_SR		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 2, 0x06)
#define HPDI32_PCI_REV		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x08)
#define HPDI32_PCI_CCR		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 4, 0x08)
#define HPDI32_PCI_CLSR		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x0C)
#define HPDI32_PCI_LTR		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x0D)
#define HPDI32_PCI_HTR		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x0E)
#define HPDI32_PCI_BISTR	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x0F)
#define HPDI32_PCI_BAR0		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 4, 0x10)
#define HPDI32_PCI_BAR1		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 4, 0x14)
#define HPDI32_PCI_BAR2		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 4, 0x18)
#define HPDI32_PCI_BAR3		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 4, 0x1C)
#define HPDI32_PCI_BAR4		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 4, 0x20)
#define HPDI32_PCI_BAR5		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 4, 0x24)
#define HPDI32_PCI_CIS		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 4, 0x28)
#define HPDI32_PCI_SVID		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 2, 0x2C)
#define HPDI32_PCI_SID		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 2, 0x2E)
#define HPDI32_PCI_ERBAR	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 4, 0x30)
#define HPDI32_PCI_ILR		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x3C)
#define HPDI32_PCI_IPR		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x3D)
#define HPDI32_PCI_MGR		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x3E)
#define HPDI32_PCI_MLR		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x3F)

/* PCI Configuration Registers: PCI9656 */
#define HPDI32_PCI_CAP_PTR	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x34)
#define HPDI32_PCI_PMCAPID	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x40)
#define HPDI32_PCI_PMNEXT	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x41)
#define HPDI32_PCI_PMC		HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 2, 0x42)
#define HPDI32_PCI_PMCSR	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 2, 0x44)
#define HPDI32_PCI_PMCSR_BSE	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x46)
#define HPDI32_PCI_PMDATA	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x47)
#define HPDI32_PCI_HS_CNTL	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x48)
#define HPDI32_PCI_HS_NEXT	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x49)
#define HPDI32_PCI_HS_CSR	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x4A)
#define HPDI32_PCI_VPDCNTL	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x4C)
#define HPDI32_PCI_VPD_NEXT	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 1, 0x4D)
#define HPDI32_PCI_VPDAD	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 2, 0x4E)
#define HPDI32_PCI_VPDATA	HPDI32_REG_ENCODE(HPDI32_PCI_REGISTER, 4, 0x50)



/* PLX 9080 and 9656 feature set registers: Local Configuration Registers */
#define HPDI32_PLX_LAS0RR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x00)
#define HPDI32_PLX_LAS0BA	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x04)
#define HPDI32_PLX_MARBR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x08)
#define HPDI32_PLX_BIGEND	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 1, 0x0C)
#define HPDI32_PLX_EROMRR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x10)
#define HPDI32_PLX_EROMBA	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x14)
#define HPDI32_PLX_LBRD0	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x18)
#define HPDI32_PLX_DMRR		HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x1C)
#define HPDI32_PLX_DMLBAM	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x20)
#define HPDI32_PLX_DMLBAI	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x24)
#define HPDI32_PLX_DMPBAM	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x28)
#define HPDI32_PLX_DMCFGA	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x2C)
#define HPDI32_PLX_LAS1RR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xF0)
#define HPDI32_PLX_LAS1BA	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xF4)
#define HPDI32_PLX_LBRD1	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xF8)

/* PLX 9080 and 9656 feature set registers: Runtime Registers */
#define HPDI32_PLX_MBOX0	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x40)
#define HPDI32_PLX_MBOX1	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x44)
#define HPDI32_PLX_MBOX2	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x48)
#define HPDI32_PLX_MBOX3	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x4C)
#define HPDI32_PLX_MBOX4	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x50)
#define HPDI32_PLX_MBOX5	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x54)
#define HPDI32_PLX_MBOX6	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x58)
#define HPDI32_PLX_MBOX7	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x5C)
#define HPDI32_PLX_P2LDBELL	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x60)
#define HPDI32_PLX_L2PDBELL	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x64)
#define HPDI32_PLX_INTCSR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x68)
#define HPDI32_PLX_CNTRL	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x6C)
#define HPDI32_PLX_PCIHIDR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x70)
#define HPDI32_PLX_PCIHREV	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 1, 0x74)

/* PLX 9080 and 9656 feature set registers: DMA Registers */
#define HPDI32_PLX_DMAMODE0	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x80)
#define HPDI32_PLX_DMAPADR0	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x84)
#define HPDI32_PLX_DMALADR0	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x88)
#define HPDI32_PLX_DMASIZ0	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x8C)
#define HPDI32_PLX_DMADPR0	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x90)
#define HPDI32_PLX_DMAMODE1	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x94)
#define HPDI32_PLX_DMAPADR1	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x98)
#define HPDI32_PLX_DMALADR1	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x9C)
#define HPDI32_PLX_DMASIZ1	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xA0)
#define HPDI32_PLX_DMADPR1	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xA4)
#define HPDI32_PLX_DMACSR0	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 1, 0xA8)
#define HPDI32_PLX_DMACSR1	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 1, 0xA9)
#define HPDI32_PLX_DMAARB	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xAC)
#define HPDI32_PLX_DMATHR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xB0)

/* PLX 9080 and 9656 feature set registers: Messaging Queue Registers */
#define HPDI32_PLX_OPLFIS	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x30)
#define HPDI32_PLX_OPLFIM	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x34)
#define HPDI32_PLX_IQP		HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x40)
#define HPDI32_PLX_OQP		HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x44)
#define HPDI32_PLX_MQCR		HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xC0)
#define HPDI32_PLX_QBAR		HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x4C)
#define HPDI32_PLX_IFHPR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xC8)
#define HPDI32_PLX_IFTPR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xCC)
#define HPDI32_PLX_IPHPR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xD0)
#define HPDI32_PLX_IPTPR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xD4)
#define HPDI32_PLX_OFHPR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xD8)
#define HPDI32_PLX_OFTPR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xDC)
#define HPDI32_PLX_OPHPR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xE0)
#define HPDI32_PLX_OPTPR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xE4)
#define HPDI32_PLX_QSR		HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xE8)



/* PLX PCI9656 feature set registers: Local Configuration Registers */
#define HPDI32_PLX_LMISC1	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 1, 0x0D)
#define HPDI32_PLX_PROT_AREA	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 1, 0x0E)
#define HPDI32_PLX_LMISC2	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 1, 0x0F)
#define HPDI32_PLX_DMDAC	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xFC)
#define HPDI32_PLX_ARB		HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 1, 0x100)
#define HPDI32_PLX_ABTADR	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0x104)

/* PLX PCI9656 feature set registers: DMA Registers */
#define HPDI32_PLX_DMADAC0	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xB4)
#define HPDI32_PLX_DMADAC1	HPDI32_REG_ENCODE(HPDI32_PLX_REGISTER, 4, 0xB8)



#ifndef HPDI32_GSC_REGS_EXCLUDE

/* GSC specific registers. */
#define	HPDI32_GSC_FRR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x00)
#define	HPDI32_GSC_BCR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x04)
#define	HPDI32_GSC_BSR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x08)
#define	HPDI32_GSC_TAR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x0C)
#define	HPDI32_GSC_RAR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x10)
#define	HPDI32_GSC_FSR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x14)
#define	HPDI32_GSC_FDR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x18)
#define	HPDI32_GSC_TSVLCR	HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x1C)
#define	HPDI32_GSC_TLVLCR	HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x20)
#define	HPDI32_GSC_TLILCR	HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x24)
#define	HPDI32_GSC_RSCR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x28)
#define	HPDI32_GSC_RLCR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x2C)
#define	HPDI32_GSC_ICR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x30)
#define	HPDI32_GSC_ISR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x34)
#define	HPDI32_GSC_TCDR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x38)
#define	HPDI32_GSC_TFSR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x40)
#define	HPDI32_GSC_RFSR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x44)
#define	HPDI32_GSC_TFWR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x48)
#define	HPDI32_GSC_RFWR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x4C)
#define	HPDI32_GSC_IELR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x50)
#define	HPDI32_GSC_IHLR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x54)



/* Register field maskss. */

/* GSC: Firmware Revision Register */

#define	HPDI32_GSC_FRR_FIRMWARE		0x000000FF
#define	HPDI32_GSC_FRR_PCB		0x0000FF00
#define	HPDI32_GSC_FRR_SUB_ID		0x00FF0000
#define	HPDI32_GSC_FRR_RESERVED		0xFF000000
#define	HPDI32_GSC_FRR_UPDATED		0x40000000	/* Updated FRR format */
#define	HPDI32_GSC_FRR_FSR		0x80000000	/* Feature Set Reg. */

#define	HPDI32_GSC_FRR_SUB_ID_DIPHASE2	0x000E0000

/* This are from the original FRR format. */
#define	HPDI32_GSC_FRR_PMC		0x01000000	/* PMC (32-bit only)*/
#define	HPDI32_GSC_FRR_64		0x01000000	/* 64-bit firmware */

/* This are from the updated FRR format. */
#define	HPDI32_GSC_FRR_FF_PCI		0x01
#define	HPDI32_GSC_FRR_FF_PMC		0x02
#define	HPDI32_GSC_FRR_FF_CPCI		0x03
#define	HPDI32_GSC_FRR_FF_PC104P	0x04

/* GSC: Board Control Register */

#define	HPDI32_GSC_BCR_BOARD_RESET	0x00000001	/* Needs 10us pause. */
#define	HPDI32_GSC_BCR_TX_FIFO_RESET	0x00000002
#define	HPDI32_GSC_BCR_RX_FIFO_RESET	0x00000004
#define	HPDI32_GSC_BCR_TX_ENABLE	0x00000010
#define	HPDI32_GSC_BCR_RX_ENABLE	0x00000020
#define	HPDI32_GSC_BCR_TX_AUTO_STOP_NO	0x00000040
#define	HPDI32_GSC_BCR_STATUS_VALID	0x00000080
#define	HPDI32_GSC_BCR_TX_START		0x00000100
#define	HPDI32_GSC_BCR_TX_THROTTLE	0x00000200
#define	HPDI32_GSC_BCR_TX_DMDMA0	0x00000400
#define	HPDI32_GSC_BCR_SING_CYC_DIS	0x00000800

#define	HPDI32_GSC_BCR_D0_MASK		0x01010000	/* Frame Valid/GPIO6 */
#define	HPDI32_GSC_BCR_D0_FV		0x00000000
#define	HPDI32_GSC_BCR_D0_IN		0x01000000
#define	HPDI32_GSC_BCR_D0_OUT		0x00010000
#define	HPDI32_GSC_BCR_D0_OUT_H		0x01010000
#define	HPDI32_GSC_BCR_D0_OUT_L		0x00010000

#define	HPDI32_GSC_BCR_D1_MASK		0x02020000	/* Line Valid/GPIO0 */
#define	HPDI32_GSC_BCR_D1_LV		0x00000000
#define	HPDI32_GSC_BCR_D1_IN		0x02000000
#define	HPDI32_GSC_BCR_D1_OUT		0x00020000
#define	HPDI32_GSC_BCR_D1_OUT_H		0x02020000
#define	HPDI32_GSC_BCR_D1_OUT_L		0x00020000

#define	HPDI32_GSC_BCR_D2_MASK		0x04040000	/* Status Valid/GPIO1 */
#define	HPDI32_GSC_BCR_D2_SV		0x00000000
#define	HPDI32_GSC_BCR_D2_IN		0x04000000
#define	HPDI32_GSC_BCR_D2_OUT		0x00040000
#define	HPDI32_GSC_BCR_D2_OUT_H		0x04040000
#define	HPDI32_GSC_BCR_D2_OUT_L		0x00040000

#define	HPDI32_GSC_BCR_D3_MASK		0x08080000	/* Rx Ready/GPIO2 */
#define	HPDI32_GSC_BCR_D3_RR		0x00000000
#define	HPDI32_GSC_BCR_D3_IN		0x08000000
#define	HPDI32_GSC_BCR_D3_OUT		0x00080000
#define	HPDI32_GSC_BCR_D3_OUT_H		0x08080000
#define	HPDI32_GSC_BCR_D3_OUT_L		0x00080000

#define	HPDI32_GSC_BCR_D4_MASK		0x10100000	/* Tx Ready/GPIO3 */
#define	HPDI32_GSC_BCR_D4_TR		0x00000000
#define	HPDI32_GSC_BCR_D4_IN		0x10000000
#define	HPDI32_GSC_BCR_D4_OUT		0x00100000
#define	HPDI32_GSC_BCR_D4_OUT_H		0x10100000
#define	HPDI32_GSC_BCR_D4_OUT_L		0x00100000

#define	HPDI32_GSC_BCR_D5_MASK		0x20200000	/* Tx Enable/GPIO4 */
#define	HPDI32_GSC_BCR_D5_TE		0x00000000
#define	HPDI32_GSC_BCR_D5_IN		0x20000000
#define	HPDI32_GSC_BCR_D5_OUT		0x00200000
#define	HPDI32_GSC_BCR_D5_OUT_H		0x20200000
#define	HPDI32_GSC_BCR_D5_OUT_L		0x00200000

#define	HPDI32_GSC_BCR_D6_MASK		0x40400000	/* Rx Enable/GPIO5 */
#define	HPDI32_GSC_BCR_D6_RE		0x00000000
#define	HPDI32_GSC_BCR_D6_IN		0x40000000
#define	HPDI32_GSC_BCR_D6_OUT		0x00400000
#define	HPDI32_GSC_BCR_D6_OUT_H		0x40400000
#define	HPDI32_GSC_BCR_D6_OUT_L		0x00400000

#define	HPDI32_GSC_BCR_TEST_MODE	0x80000000

/* GSC: Board Status Register */

#define	HPDI32_GSC_BSR_D_MASK		0x0000007F
#define	HPDI32_GSC_BSR_D0		0x00000001
#define	HPDI32_GSC_BSR_D1		0x00000002
#define	HPDI32_GSC_BSR_D2		0x00000004
#define	HPDI32_GSC_BSR_D3		0x00000008
#define	HPDI32_GSC_BSR_D4		0x00000010
#define	HPDI32_GSC_BSR_D5		0x00000020
#define	HPDI32_GSC_BSR_D6		0x00000040

#define	HPDI32_GSC_BSR_FV_GPIO6		0x00000001
#define	HPDI32_GSC_BSR_LV_GPIO0		0x00000002
#define	HPDI32_GSC_BSR_SV_GPIO1		0x00000004
#define	HPDI32_GSC_BSR_RR_GPIO2		0x00000008
#define	HPDI32_GSC_BSR_TR_GPIO3		0x00000010
#define	HPDI32_GSC_BSR_TE_GPIO4		0x00000020
#define	HPDI32_GSC_BSR_RE_GPIO5		0x00000040

#define	HPDI32_GSC_BSR_FRAME_VALID	0x00000001
#define	HPDI32_GSC_BSR_LINE_VALID	0x00000002
#define	HPDI32_GSC_BSR_STATUS_VALID	0x00000004
#define	HPDI32_GSC_BSR_RX_READY		0x00000008
#define	HPDI32_GSC_BSR_TX_READY		0x00000010
#define	HPDI32_GSC_BSR_TX_ENABLED	0x00000020
#define	HPDI32_GSC_BSR_RX_ENABLED	0x00000040

#define	HPDI32_GSC_BSR_TX_ACTIVE	0x00000080

#define	HPDI32_GSC_BSR_TX_MASK		0x00000F00
#define	HPDI32_GSC_BSR_TX_NOT_EMPTY	0x00000100
#define	HPDI32_GSC_BSR_TX_NOT_AE	0x00000200
#define	HPDI32_GSC_BSR_TX_NOT_AF	0x00000400
#define	HPDI32_GSC_BSR_TX_NOT_FULL	0x00000800

#define	HPDI32_GSC_BSR_RX_MASK		0x0000F000
#define	HPDI32_GSC_BSR_RX_NOT_EMPTY	0x00001000
#define	HPDI32_GSC_BSR_RX_NOT_AE	0x00002000
#define	HPDI32_GSC_BSR_RX_NOT_AF	0x00004000
#define	HPDI32_GSC_BSR_RX_NOT_FULL	0x00008000

#define	HPDI32_GSC_BSR_BJ0		0x00010000	/* Not on PMC. */
#define	HPDI32_GSC_BSR_BJ1		0x00020000	/* Not on PMC. */
#define	HPDI32_GSC_BSR_TX_OVERRUN	0x00200000
#define	HPDI32_GSC_BSR_RX_UNDERRUN	0x00400000
#define	HPDI32_GSC_BSR_RX_OVERRUN	0x00800000

/* GSC: Tx Almost Register */

#define	HPDI32_GSC_TAR_ENCODE(f,e)	(HPDI32_ENCODE(f,31,16) | \
					HPDI32_ENCODE(e,15,0))

#define	HPDI32_GSC_TAR_DECODE_EMPTY(r)	HPDI32_DECODE(r,15,0)
#define	HPDI32_GSC_TAR_DECODE_FULL(r)	HPDI32_DECODE(r,31,16)

/* GSC: Rx Almost Register */

#define	HPDI32_GSC_RAR_ENCODE(f,e)	(HPDI32_ENCODE(f,31,16) | \
					HPDI32_ENCODE(e,15,0))

#define	HPDI32_GSC_RAR_DECODE_EMPTY(r)	HPDI32_DECODE(r,15,0)
#define	HPDI32_GSC_RAR_DECODE_FULL(r)	HPDI32_DECODE(r,31,16)

/* GSC: Feature Set Register */

#define	HPDI32_GSC_FSR_MASK		0x000000FF
#define	HPDI32_GSC_FSR_TFSR_RFSR	0x00000001
#define	HPDI32_GSC_FSR_TFWR_RFWR	0x00000002
#define	HPDI32_GSC_FSR_IELR_IHLR	0x00000004
#define	HPDI32_GSC_FSR_GPIO_0_5		0x00000008
#define	HPDI32_GSC_FSR_DMA_CH1		0x00000010
#define	HPDI32_GSC_FSR_TX_RX_RUN	0x00000020
#define	HPDI32_GSC_FSR_GPIO6_TXAS	0x00000040 /* GPIO6 + Tx Auto Stop */
#define	HPDI32_GSC_FSR_SCD		0x00000080 /* Single Cycle Disable */

/* Interrupt register bits: GSC specific */

#define	HPDI32_INT_GSC_FRAME_VALID_S	0x00000001
#define	HPDI32_INT_GSC_FRAME_VALID_E	0x00000002
#define	HPDI32_INT_GSC_LV_GPIO0		0x00000004
#define	HPDI32_INT_GSC_SV_GPIO1		0x00000008
#define	HPDI32_INT_GSC_RR_GPIO2		0x00000010
#define	HPDI32_INT_GSC_TR_GPIO3		0x00000020
#define	HPDI32_INT_GSC_TE_GPIO4		0x00000040
#define	HPDI32_INT_GSC_RE_GPIO5		0x00000080
#define	HPDI32_INT_GSC_TX_FIFO_MASK	0x00000F00
#define	HPDI32_INT_GSC_TX_FIFO_EMPTY	0x00000100
#define	HPDI32_INT_GSC_TX_FIFO_AE	0x00000200
#define	HPDI32_INT_GSC_TX_FIFO_AF	0x00000400
#define	HPDI32_INT_GSC_TX_FIFO_FULL	0x00000800
#define	HPDI32_INT_GSC_RX_FIFO_MASK	0x0000F000
#define	HPDI32_INT_GSC_RX_FIFO_EMPTY	0x00001000
#define	HPDI32_INT_GSC_RX_FIFO_AE	0x00002000
#define	HPDI32_INT_GSC_RX_FIFO_AF	0x00004000
#define	HPDI32_INT_GSC_RX_FIFO_FULL	0x00008000

#endif	// HPDI32_GSC_REGS_EXCLUDE

/* Interrupt register bits: Non-GSC specific */

#define	HPDI32_INT_PCI			0x00000001
#define	HPDI32_INT_ANY_OTHER		0x00000002
#define	HPDI32_INT_DMA_0_DONE		0x00000004
#define	HPDI32_INT_DMA_1_DONE		0x00000008



/* typedefs	**************************************************************/

/*
*	This structure is used to obtain debug information from the driver.
*	This is used primarily for driver development. The data is valid
*	only if the "valid" field id non-zero.
*/

typedef struct
{
	__u32	device[32];
	__u32	driver[32];
} hpdi32_debug_data_t;

/*
*	This structure is used to obtain version information from the driver.
*/

typedef struct
{
	__s8	version[8];
	__s8	built[32];
} hpdi32_driver_info_t;

/*
*	This structure is used for read and write I/O configuration.
*/

typedef struct
{
	struct
	{
		__s32	mode;
		__s32	timeout;
		__u32	samples;
		__s32	sample_bits;
		__s32	pio_threshold;	/* in samples */
	} read;

	struct
	{
		__s32	mode;
		__s32	timeout;
		__u32	samples;
		__s32	sample_bits;
		__s32	pio_threshold;	/* in samples */
	} write;
} hpdi32_io_config_t;

/*
*	This structure is used for interrupt notification and status.
*/

typedef struct
{
	struct
	{
		__u32	gsc;
		__u32	other;
	} notify;

	struct
	{
		__u32	gsc;
		__u32	other;
	} status;
} hpdi32_interrupt_t;

/*
*	These structures are used with the mmap() driver support services.
*/

typedef struct
{
	__u32	offset;
	__u32	size;
} hpdi32_mmap_mem_t;

typedef struct
{
	hpdi32_mmap_mem_t	regs;
	hpdi32_mmap_mem_t	read;
	hpdi32_mmap_mem_t	write;
} hpdi32_mmap_t;

/*
*	This structure is used for register read, write and modify requests.
*/

typedef struct
{
	__u32	reg;
	__u32	value;
	__u32	mask;	/* Only for MOD requests. */
} hpdi32_reg_t;



#endif

