/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 3 $ */
/* $Date: 2/22/06 10:22a $ */
/*
*	Provide special handling for things specific to the 2.2 kernel.
*/
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32_Linux/HPDI32_Linux_Ver.1.13.0-GSC-DN/driver/kernel_2_2.c $
 *
 * 3     2/22/06 10:22a Dnewbold
 * Removed an unused argument to the nopage routine.
 *
 * 2     1/25/05 3:46p Dnewbold
 * More modularization work, plus some housekeeping updates.
 *
 * 1     1/14/05 8:04a Dnewbold
*/

#include <linux/version.h>



#if	(LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)) && \
	(LINUX_VERSION_CODE <  KERNEL_VERSION(2,3,0))

#define	MODULE
#include "main.h"



/* Tags	**********************************************************************/

EXPORT_NO_SYMBOLS;
MODULE_AUTHOR("General Standards Corporation, www.generalstandards.com");
MODULE_DESCRIPTION("GSC HPDI32 driver " DRIVER_VERSION);
MODULE_SUPPORTED_DEVICE("GSC HPDI32");



/******************************************************************************
*
*	Function:	irq_isr
*
*	Purpose:
*
*		Service an interrupt.  For safety sake, each detected
*		interrupt is cleared then disabled.
*
*	Arguments:
*
*		irq	The interrupt number.
*
*		dev_id	The private data we've associated with the IRQ.
*
*		regs	Unused.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void irq_isr(int irq, void* dev_id, struct pt_regs* regs)
{
	irq_isr_common(irq, dev_id);
}



/******************************************************************************
*
*	Function:	mmap_nopage
*
*	Purpose:
*
*		Implement the mmap nopage() procedure.
*
*	Arguments:
*
*		vma		The virtual memory area of interest.
*
*		address		The virtual address of the page being accessed.
*
*		write_access	The "no-share" option.
*	Returned:
*
*		0		The page is invalid.
*		else		The virtual address of the page.
*
******************************************************************************/

unsigned long mmap_nopage(	struct vm_area_struct*	vma,
				unsigned long		address,
				int			write_access)
{
	unsigned long	ul;
	struct page*	page;

	mmap_nopage_common(vma, address, &page);

	if (page == NULL)
		ul	= 0;
	else
		ul	= (u32) __va((page - mem_map) << PAGE_SHIFT);

	return(ul);
}



/******************************************************************************
*
*	Function:	mmap_unmap
*
*	Purpose:
*
*		Implement the mmap unmap() procedure.
*
*	Arguments:
*
*		vma	The virtual memory area of interest.
*
*		addr	The address of the area being released.
*
*		len	The size of the area being released.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void mmap_unmap(struct vm_area_struct*	vma,
		unsigned long		addr,
		size_t			len)
{
	dev_data_t*	dev;
	int		test;

	for (;;)	/* We'll use a loop for convenience. */
	{
		/*
		*	Validate some of the parameters.
		*/

		dev	= dev_locate(vma->vm_file->f_dentry->d_inode);

		if (dev == NULL)
		{
			/* The referenced device doesn't exist. */
			break;
		}

		/*
		*	Gain the neccessary access.
		*/

		test	= down_interruptible(&dev->sem);

		if (test)
		{
			/* We got a signal rather than the semaphore. */
			break;
		}

		/*
		*	See which memory area this request applies to.
		*/

		if (	(VM_OFFSET_BYTES(vma) == dev->mmap.read.offset) &&
			(len == dev->mmap.read.pg_size))
		{
			/* This is the read() buffer. */
			up(&dev->sem);
			dev->mmap.read.usage--;
			break;
		}

		if (	(VM_OFFSET_BYTES(vma) == dev->mmap.write.offset) &&
			(len == dev->mmap.write.pg_size))
		{
			/* This is the write() buffer. */
			up(&dev->sem);
			dev->mmap.write.usage--;
			break;
		}

		/* The region is not one we can release. */
		up(&dev->sem);
		break;
	}
}



/******************************************************************************
*
*	Function:	proc_get_info
*
*	Purpose:
*
*		Implement the get_info() service for /proc file system support.
*
*	Arguments:
*
*		page	The data produced is put here.
*
*		start	Records pointer to where the data in "page" begins.
*
*		offset	The offset into the file where we're to begin.
*
*		count	The limit to the amount of data we can produce.
*
*		dummy	A parameter that is unused.
*
*	Returned:
*
*		int	The number of characters written.
*
******************************************************************************/

int proc_get_info(
	char*	page,
	char**	start,
	off_t	offset,
	int	count,
	int	dummy)
{
	int	eof;
	int	i;

	i	= proc_read(page, start, offset, count, &eof, NULL);
	return(i);
}



#endif
