/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 3 $ */
/* $Date: 2/22/06 10:22a $ */
/*
*	Provide special handling for things specific to the 2.4 kernel.
*/
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32_Linux/HPDI32_Linux_Ver.1.13.0-GSC-DN/driver/kernel_2_4.c $
 *
 * 3     2/22/06 10:22a Dnewbold
 * Removed an unused argument to the nopage routine.
 *
 * 2     1/25/05 3:46p Dnewbold
 * More modularization work, plus some housekeeping updates.
 *
 * 1     1/14/05 8:04a Dnewbold
*/

#include <linux/version.h>



#if	(LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)) && \
	(LINUX_VERSION_CODE <  KERNEL_VERSION(2,5,0))

#define	MODULE
#include "main.h"



/* #defines	**************************************************************/

#ifndef MODULE_LICENSE
	#define	MODULE_LICENSE(s)
#endif



/* Tags	**********************************************************************/

EXPORT_NO_SYMBOLS;
MODULE_AUTHOR("General Standards Corporation, www.generalstandards.com");
MODULE_DESCRIPTION("GSC HPDI32 driver " DRIVER_VERSION);
MODULE_LICENSE("GPL");
MODULE_SUPPORTED_DEVICE("GSC HPDI32");



/******************************************************************************
*
*	Function:	irq_isr
*
*	Purpose:
*
*		Service an interrupt.  For safety sake, each detected
*		interrupt is cleared then disabled.
*
*	Arguments:
*
*		irq	The interrupt number.
*
*		dev_id	The private data we've associated with the IRQ.
*
*		regs	Unused.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void irq_isr(int irq, void* dev_id, struct pt_regs* regs)
{
	irq_isr_common(irq, dev_id);
}



/******************************************************************************
*
*	Function:	mmap_nopage
*
*	Purpose:
*
*		Implement the mmap nopage() procedure.
*
*	Arguments:
*
*		vma	The virtual memory area of interest.
*
*		address	The virtual address of the page being accessed.
*
*		unused	This argument is unused.
*
*	Returned:
*
*		NULL	The request failed.
*		else	The request succeeded.
*
******************************************************************************/

struct page* mmap_nopage(
	struct vm_area_struct*	vma,
	unsigned long		address,
	int			unused)
{
	struct page*	page;

	mmap_nopage_common(vma, address, &page);
	return(page);
}



/******************************************************************************
*
*	Function:	proc_get_info
*
*	Purpose:
*
*		Implement the get_info() service for /proc file system support.
*
*	Arguments:
*
*		page	The data produced is put here.
*
*		start	Records pointer to where the data in "page" begins.
*
*		offset	The offset into the file where we're to begin.
*
*		count	The limit to the amount of data we can produce.
*
*	Returned:
*
*		int	The number of characters written.
*
******************************************************************************/

int proc_get_info(
	char*	page,
	char**	start,
	off_t	offset,
	int	count)
{
	int	eof;
	int	i;

	i	= proc_read(page, start, offset, count, &eof, NULL);
	return(i);
}



#endif
