/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 5 $ */
/* $Date: 7/25/07 9:36a $ */
/*
*	Provide special handling for things specific to the 2.6 kernel.
*/
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.18.0-GSC-DN/driver/kernel_2_6.c $
 *
 * 5     7/25/07 9:36a Dnewbold
 * Corrected the ISR prototype for 2.6.19 changes.
 *
 * 4     8/21/06 10:26a Dnewbold
 * Porting to the 2.6 and 64-bit kernels.
 *
 * 3     2/22/06 10:22a Dnewbold
 * Removed an unused argument to the nopage routine.
 *
 * 2     1/25/05 3:46p Dnewbold
 * More modularization work, plus some housekeeping updates.
 *
 * 1     1/14/05 8:04a Dnewbold
*/

#include <linux/module.h>
#include <linux/version.h>



#if	(LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)) && \
	(LINUX_VERSION_CODE <  KERNEL_VERSION(2,7,0))

#include "main.h"



/* Tags	**********************************************************************/

MODULE_AUTHOR("General Standards Corporation, www.generalstandards.com");
MODULE_DESCRIPTION("GSC HPDI32 driver " DRIVER_VERSION);
MODULE_LICENSE("GPL");
MODULE_SUPPORTED_DEVICE("GSC HPDI32");



/******************************************************************************
*
*	Function:	irq_isr
*
*	Purpose:
*
*		Service an interrupt.
*
*	Arguments:
*
*		irq	The interrupt number.
*
*		dev_id	The private data we've associated with the IRQ.
*
*		regs	Unused. Not present beginning at 2.6.19.
*
*	Returned:
*
*		The status as to whether we handled the interrupt or not.
*
******************************************************************************/

irqreturn_t irq_isr(int irq, void* dev_id  ISR_ARG3)
{
	int		handled;
	irqreturn_t	ret;

	handled	= irq_isr_common(irq, dev_id);
	ret	= handled ? IRQ_HANDLED : IRQ_NONE;
	return(ret);
}



/******************************************************************************
*
*	Function:	mmap_nopage
*
*	Purpose:
*
*		Implement the mmap nopage() procedure.
*
*	Arguments:
*
*		vma	The virtual memory area of interest.
*
*		address	The virtual address of the page being accessed.
*
*		type	We record the mapping status here.
*
*	Returned:
*
*		NULL	The request failed.
*		else	The request succeeded.
*
******************************************************************************/

struct page* mmap_nopage(
	struct vm_area_struct*	vma,
	unsigned long		address,
	int*			type)
{
	struct page*	page;

	mmap_nopage_common(vma, address, &page);

	if ((type) && (page == NULL))
		type[0]	= VM_FAULT_MINOR;

	return(page);
}



/******************************************************************************
*
*	Function:	proc_get_info
*
*	Purpose:
*
*		Implement the get_info() service for /proc file system support.
*
*	Arguments:
*
*		page	The data produced is put here.
*
*		start	Records pointer to where the data in "page" begins.
*
*		offset	The offset into the file where we're to begin.
*
*		count	The limit to the amount of data we can produce.
*
*	Returned:
*
*		int	The number of characters written.
*
******************************************************************************/

int proc_get_info(
	char*	page,
	char**	start,
	off_t	offset,
	int	count)
{
	int	eof;
	int	i;

	i	= proc_read(page, start, offset, count, &eof, NULL);
	return(i);
}



/******************************************************************************
*
*	Function:	event_wait_irq_to
*
*	Purpose:
*
*		Implement a wrapper around wait_event_interruptible_timeout().
*
*	Arguments:
*
*		queue	This is the queue being waited upon.
*
*		cond	This is the condition variable.
*
*		timeout	This is the timeout value.
*
*	Returned:
*
*		int	This is the return value from the called function.
*
******************************************************************************/

#if EVENT_WAIT_IRQ_TO_FUNCTION
int event_wait_irq_to(
	WAIT_QUEUE_HEAD_T*	queue,
	int*			cond,
	unsigned long		timeout)
{
	#define	USE_DELAY	1
	#define	DELAY_INC_US	1
	#define	DELAY_MAX_US	100

	int	ret;

	cond[0]	= 0;

#if USE_DELAY

	{
		int	i;

		for (i = 0; i < DELAY_MAX_US; i -= DELAY_INC_US)
		{
			ret	= wait_event_interruptible_timeout(queue[0], cond[0], timeout);

			if (ret >= 0)
				break;

			udelay(DELAY_INC_US);
		}
	}

#else

	ret	= wait_event_interruptible_timeout(queue[0], cond[0], timeout);

#endif

	return(ret);
}
#endif



#endif
