/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 9 $ */
/* $Date: 7/25/07 9:37a $ */
/*
*	Provide special handling for things specific to the 2.6 kernel.
*/
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.18.0-GSC-DN/driver/kernel_2_6.h $
 *
 * 9     7/25/07 9:37a Dnewbold
 * Corrected the ISR prototype for 2.6.19 changes.
 *
 * 8     8/23/06 11:20a Dnewbold
 * Reset the default for the EVENT_WAIT_IRQ_TO_FUNCTION macro.
 *
 * 7     8/21/06 10:26a Dnewbold
 * Porting to the 2.6 and 64-bit kernels.
 *
 * 6     8/16/06 11:59a Dnewbold
 * Additional kernel porting and added some work around for an older
 * kernel.
 *
 * 5     2/22/06 10:22a Dnewbold
 * Removed an unused argument to the nopage routine.
 *
 * 4     2/01/06 2:57p Dnewbold
 * Renamed KMALLOC_LIMIT to MEM_MALLOC_LIMIT.
 *
 * 3     9/01/05 1:29p Dnewbold
 * More 2.6 kernel porting.
 *
 * 2     1/25/05 3:46p Dnewbold
 * More modularization work, plus some housekeeping updates.
 *
 * 1     1/14/05 8:04a Dnewbold
*/

#ifndef __KERNEL_2_6_H__
#define __KERNEL_2_6_H__

#if	(LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)) && \
	(LINUX_VERSION_CODE <  KERNEL_VERSION(2,7,0))

#include <linux/moduleparam.h>
#include <linux/slab.h>



/* #defines	**************************************************************/


#define	EVENT_WAIT_IRQ_TO_FUNCTION	0   // Work around for some kernels.

#if EVENT_WAIT_IRQ_TO_FUNCTION
	#define	EVENT_WAIT_IRQ_TO(q,c,t)	event_wait_irq_to(q,&(c),t)
#else
	#define	EVENT_WAIT_IRQ_TO(q,c,t)	\
		c = 0; wait_event_interruptible_timeout(q[0],c,t)
#endif

#define	EVENT_RESUME_IRQ(q,c)		c = 1; wake_up_interruptible(q)
#define	GET_PAGE(page)			get_page((page))
#define	KILL_FASYNC(q,s,b)		kill_fasync(&(q),(s),(b))
#define	MEM_ALLOC_LIMIT			(2L * 1024L * 1024)
#undef	MOD_DEC_USE_COUNT
#define	MOD_DEC_USE_COUNT
#undef	MOD_INC_USE_COUNT
#define	MOD_INC_USE_COUNT
#define	PAGE_RESERVE(vpa)		SetPageReserved(virt_to_page((vpa)))
#define	PAGE_UNRESERVE(vpa)		ClearPageReserved(virt_to_page((vpa)))

#define	PCI_DEVICE_LOOP(p)	\
	for ((p) = NULL; ((p) = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, (p))); )

#define	PCI_ENABLE_DEVICE(pci)		pci_enable_device(pci)
#define	REGION_IO_CHECK(a,s)		0
#define	REGION_IO_RELEASE(a,s)		release_region(a,s)
#define	REGION_IO_REQUEST(a,s,n)	(request_region(a,s,n))
#define	REGION_MEM_CHECK(a,s)		check_mem_region(a,s)
#define	REGION_MEM_RELEASE(a,s)		release_mem_region(a,s)
#define	REGION_MEM_REQUEST(a,s,n)	(request_mem_region(a,s,n))
#define	SET_CURRENT_STATE(s)		current->state = (s)
#define	SET_MODULE_OWNER(p)
#define	VADDR_T				void*
#define	VIRT_TO_PAGE(p)			virt_to_page((p))
#define	VM_OFFSET_BYTES(vma)		((vma)->vm_pgoff << PAGE_SHIFT)
#define	VM_OPS_UNMAP(vmo,fun)
#define	WAIT_QUEUE_ENTRY_INIT(w,c)	init_waitqueue_entry((w),(c))
#define	WAIT_QUEUE_ENTRY_T		wait_queue_t
#define	WAIT_QUEUE_HEAD_INIT(q)		init_waitqueue_head((q))
#define	WAIT_QUEUE_HEAD_T		wait_queue_head_t

#if	(LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11))
	#define	REMAP_PAGE_RANGE(vma,vm_adrs,ph_adrs,size,prot)		\
			remap_pfn_range( vma,vm_adrs,ph_adrs >> PAGE_SHIFT,	\
								BYTES_TO_PAGE_BYTES((size)),prot)
#else
	#define	REMAP_PAGE_RANGE(v,f,t,s,p)	remap_page_range(v,f,t,s,p)
#endif

#if	(LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,19))
	#define	ISR_ARG3
#else
	#define	ISR_ARG3	, struct pt_regs* regs
#endif



/* prototypes	**************************************************************/

#if EVENT_WAIT_IRQ_TO_FUNCTION
int		event_wait_irq_to(	WAIT_QUEUE_HEAD_T*	queue,
					int*			cond,
					unsigned long		timeout);
#endif

irqreturn_t	irq_isr(int irq, void* dev_id ISR_ARG3);

struct page*	mmap_nopage(	struct vm_area_struct*	vma,
				unsigned long		address,
				int*			type);

int		proc_get_info(	char*	page,
				char**	start,
				off_t	offset,
				int	count);



#endif
#endif
