/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 6 $ */
/* $Date: 9/28/06 5:49p $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.17.0-GSC-DN/driver/main.c $
 *
 * 6     9/28/06 5:49p Dnewbold
 * Added code to specify the width of the board's PCI interface.
 *
 * 5     8/21/06 10:26a Dnewbold
 * Porting to the 2.6 and 64-bit kernels.
 *
 * 4     2/27/06 10:37a Dnewbold
 * Updated for new I/O memory allocation mechanism. Added DIPHASE2
 * support.
 *
 * 3     2/01/06 2:59p Dnewbold
 * initialized the field dev->vaddr.gsc_fsr_32.
 *
 * 2     1/25/05 3:46p Dnewbold
 * More modularization work, plus some housekeeping updates.
 *
 * 1     1/14/05 8:04a Dnewbold
*/

#include "main.h"



/* #defines	**************************************************************/

/* These are from dio24.h since the DIO24-GD1 looks like an HPDI32. */
#define	DIO24_GSC_FRR		HPDI32_REG_ENCODE(HPDI32_GSC_REGISTER, 4, 0x00)
#define	DIO24_FRR_TYPE_GD1	0x000B0000	/* DIO24 id in FRR */
#define	DIO24_FRR_TYPE_SBX	0x00130000	/* DIO24 id in FRR */
#define	DIO24_FRR_TYPE_MASK	0x00FF0000	/* FRR field for type id */

/* PLX EEPROM/Control register bits. */
#define	CNTRL_EEPROM_RELOAD		HPDI32_ENCODE(1,29,29)
#define	CNTRL_SOFTWARE_RESET		HPDI32_ENCODE(1,30,30)
#define	CNTRL_RESET_DONE		HPDI32_ENCODE(1,31,31)



/* data types	**************************************************************/

typedef struct
{
	unsigned long	offset;
	unsigned long	flags;	/* The flags must match. */
	long		size;	/* The size must be the same. */
} dev_bar_dat_t;

typedef struct
{
	unsigned long	reg;
	unsigned long	value;
} dev_reg_id_t;



/* variables	**************************************************************/

	dev_global_t	global;



/******************************************************************************
*
*	Function:	_bar_verify
*
*	Purpose:
*
*		Verify that the Base Address Registers are valid for the
*		the desired device.
*
*	Arguments:
*
*		pci	The structure referencing the PCI device of interest.
*
*		bar	This list give the BAR requirements.
*
*		region	The data describing the regions get put in this list.
*
*		size	The size of the above lists.
*
*	Returned:
*
*		0	All went well.
*		1	There was an error.
*
******************************************************************************/

static int _bar_verify(	struct pci_dev*		pci,
			const dev_bar_dat_t*	bar,
			dev_region_t**		region,
			size_t			size)
{
	u32	address;
	int	i;
	u32	r_size;

	for (i = 0; i < size; i++)
	{
		pci_read_config_dword(pci, bar[i].offset, &address);
		/* Use a PCI feature to get the address mask. */
		pci_write_config_dword(pci, bar[i].offset, ~0);
		pci_read_config_dword(pci, bar[i].offset, &r_size);
		/* Restore the register. */
		pci_write_config_dword(pci, bar[i].offset, address);
		r_size	= 1 + ~(r_size & ~0xF);
#if 0
		printk("<1>PCIBAR%d 0x%08lX: ", i, (unsigned long) address);
		printk(	"FLAGS want 0x%lX, got 0x%lX, ",
			(unsigned long) bar[i].flags,
			(unsigned long) address & 0xF);
		printk(	"SIZE want %lu, got %lu\n",
			(unsigned long) bar[i].size,
			(unsigned long) r_size);
#endif

		if ((address & 0xF) != bar[i].flags)
			break;

		if (r_size != bar[i].size)
			break;

		region[i]->phys_adrs	= address & 0xFFFFFFF0;
		region[i]->flags	= bar[i].flags;
		region[i]->size		= r_size;
	}

	i	= (i == size) ? 0 : 1;
	return(i);
}



/******************************************************************************
*
*	Function:	_dev_add
*
*	Purpose:
*
*		Add a device to our device list.
*
*	Arguments:
*
*		dev	The data for the device to add.
*
*	Returned:
*
*		0	All went well.
*		1	There was a problem.
*
******************************************************************************/

static int _dev_add(dev_data_t* dev)
{
	int	i;
	int	status;

	for (;;)	/* We'll use a loop for convenience. */
	{
		/* Find an available entry. */

		for (i = 0; i < ARRAY_ELEMENTS(global.dev_list); i++)
		{
			if (global.dev_list[i] == NULL)
				break;
		}

		if (i >= ARRAY_ELEMENTS(global.dev_list))
		{
			/* We don't have enough room to add the device. */
			status	= 1;
			printk(	"<1>%s: Too many devices were found (%d).\n"
				"%s: Rebuild the driver to increase support.\n",
				MODULE_NAME,
				(int) ARRAY_ELEMENTS(global.dev_list),
				MODULE_NAME);
			break;
		}

		global.dev_list[i]	= kmalloc(sizeof(dev_data_t), GFP_KERNEL);

		if (global.dev_list[i] == NULL)
		{
			/* We didn't get the needed memory. */
			status	= 1;
			printk("<1>%s: kmalloc() failure.\n", MODULE_NAME);
			break;
		}

		/* We got the needed memory. */
		memcpy(global.dev_list[i], dev, sizeof(dev_data_t));
		status	= 0;
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	_id_verify
*
*	Purpose:
*
*		Verify the identification of the given PCI device to see if it
*		complies with the requirements of the given list. is a 32-bit
*
*	Arguments:
*
*		pci	The structure referencing the PCI device of interest.
*
*		list	The id register list to validate.
*
*		size	The size of the above list.
*
*	Returned:
*
*		0	All went well.
*		1	There was an error.
*
******************************************************************************/

static int _id_verify(	struct pci_dev*		pci,
			const dev_reg_id_t*	list,
			size_t			size)
{
	int	i;
	u16	word;

	for (i = 0; i < (int) size; i++)
	{
		pci_read_config_word(pci, list[i].reg, &word);
#if 0
		printk("<1>ID %d. ", i);
		printk("want 0x%04lX, ", (long) list[i].value);
		printk("got 0x%04lX\n", (unsigned long) word);
#endif

		if (word != list[i].value)
			break;
	}

	i	= (i == size) ? 0 : 1;
	return(i);
}



/******************************************************************************
*
*	Function:	_regions_release
*
*	Purpose:
*
*		Release any of the given regions that were successfully
*		requested.
*
*	Arguments:
*
*		list	The list of resource regions is recorded here.
*
*		size	The size of the list.
*
*	Returned:
*
*		None.
*
******************************************************************************/

static void _regions_release(dev_region_t** list, size_t size)
{
	int	i;

	for (i = 0; i < size; i++)
	{
		if (list[i]->requested == 0)
		{
		}
		else if (list[i]->flags & PCI_BASE_ADDRESS_SPACE_IO)
		{
			REGION_IO_RELEASE(list[i]->phys_adrs, list[i]->size);
		}
		else
		{
			iounmap(list[i]->vaddr);
			REGION_MEM_RELEASE(list[i]->phys_adrs, list[i]->size);
		}
	}
}



/******************************************************************************
*
*	Function:	_regions_request
*
*	Purpose:
*
*		Request that the specified set of memory regions be reserved
*		for our use.
*
*	Arguments:
*
*		list	The list of resource regions is recorded here.
*
*		size	The size of the list.
*
*	Returned:
*
*		0	All went well.
*		1	There was an error.
*
******************************************************************************/

static int _regions_request(dev_region_t** list, size_t size)
{
	int	i;
	void*	test;

	for (i = 0; i < size; i++)
	{
		if (list[i]->size == 0)
			continue;

		if (list[i]->flags & PCI_BASE_ADDRESS_SPACE_IO)
		{
			test	= REGION_IO_REQUEST(	list[i]->phys_adrs,
							list[i]->size,
							MODULE_NAME);

			if (test == 0)
				break;

			list[i]->requested	= 1;
			list[i]->vaddr		= (void*) (long)
							list[i]->phys_adrs;
		}
		else
		{
			test	= REGION_MEM_REQUEST(	list[i]->phys_adrs,
							list[i]->size,
							MODULE_NAME);

			if (test == 0)
				break;

			list[i]->requested	= 1;
			list[i]->vaddr		= ioremap(
							list[i]->phys_adrs,
							list[i]->size);
		}
	}

	i	= (i == size) ? 0 : 1;
	return(i);
}



/******************************************************************************
*
*	Function:	_regions_verify
*
*	Purpose:
*
*		Verify that the resource regions are available for use.
*
*	Arguments:
*
*		list	The list of resource regions is recorded here.
*
*		size	The size of the list.
*
*	Returned:
*
*		0	All went well.
*		1	There was an error.
*
******************************************************************************/

static int _regions_verify(dev_region_t** list, size_t size)
{
	int	i;
	int	test;

	for (i = 0; i < size; i++)
	{
		if (list[i]->size == 0)
			continue;

		if (list[i]->flags & PCI_BASE_ADDRESS_SPACE_IO)
		{
			test	= REGION_IO_CHECK(	list[i]->phys_adrs,
							list[i]->size);
		}
		else
		{
			test	= REGION_MEM_CHECK(	list[i]->phys_adrs,
							list[i]->size);
		}

		if (test < 0)
			break;
	}

	i	= (i == size) ? 0 : 1;
	return(i);
}



/******************************************************************************
*
*	Function:	_id_hpdi32_32
*
*	Purpose:
*
*		See if the referenced board is a 32-bit PCI interface version.
*		This test is independent of the board's form factor.
*
*	Arguments:
*
*		dev	The device structure of interest.
*
*	Returned:
*
*		0	All went well.
*		1	There was an error.
*
******************************************************************************/

static int _id_hpdi32_32(dev_data_t* dev)
{
	static const dev_bar_dat_t	bar[6]	=
	{
		/* offset		flags	size			*/
		{ PCI_BASE_ADDRESS_0,	0,	256 /* PLX memory */	},
		{ PCI_BASE_ADDRESS_1,	0x1,	256 /* PLX I/O ports */	},
		{ PCI_BASE_ADDRESS_2,	0,	GSC_REG_BLOCK_SIZE	},
		{ PCI_BASE_ADDRESS_3,	0,	0 /* Unused. */		},
		{ PCI_BASE_ADDRESS_4,	0,	0 /* Unused. */		},
		{ PCI_BASE_ADDRESS_5,	0,	0 /* Unused. */		}
	};

	static const dev_reg_id_t	id[4]	=
	{
		/* reg				value			*/
		{ PCI_VENDOR_ID,		HPDI32_VENDOR_ID	},
		{ PCI_DEVICE_ID,		HPDI32_DEVICE_ID_32	},
		{ PCI_SUBSYSTEM_VENDOR_ID,	HPDI32_SUBVENDOR_ID	},
		{ PCI_SUBSYSTEM_ID,		HPDI32_SUBDEVICE_ID_32	}
	};

	dev_region_t	dummy;
	int		i;
	hpdi32_reg_t	reg;

	dev_region_t*		regions[6] =
	{
		&dev->plx_mem,
		&dev->plx_io,
		&dev->gsc_mem,
		&dummy,
		&dummy,
		&dummy
	};

	memset(&dummy, 0, sizeof(dummy));
	dev->type	= HPDI32_32_BIT;
	dev->bus_bits	= 32;
	i		= _id_verify(dev->pci, id, 4);

	if (i == 0)
		i	= _bar_verify(dev->pci, bar, regions, 6);

	/*
	*	Ordinarily this would be it, but some DIO24's ID registers
	*	appears like a 32-bit HPDI32. So, we've got to check the
	*	Firmware Revision Register to see if the board is a DIO24
	*	variation or an HPDI32.
	*/

	if (i == 0)
		i	= _regions_verify(regions, ARRAY_ELEMENTS(regions));

	if (i == 0)
		i	= _regions_request(regions, ARRAY_ELEMENTS(regions));

	if (i == 0)
	{
		reg.reg	= DIO24_GSC_FRR;
		reg_read_ioctl(dev, &reg);

		if ((reg.value & DIO24_FRR_TYPE_MASK) == DIO24_FRR_TYPE_GD1)
		{
			/* This is a DIO24-GD1. */
			i	= 1;
		}
		else
		if ((reg.value & DIO24_FRR_TYPE_MASK) == DIO24_FRR_TYPE_SBX)
		{
			/* This is a DIO24-SBX. */
			i	= 1;
		}
		else
		{
			/* This is an HPDI32. */
		}

		_regions_release(regions, ARRAY_ELEMENTS(regions));
	}

	return(i);
}



/******************************************************************************
*
*	Function:	_id_hpdi32_64
*
*	Purpose:
*
*		See if the referenced board is a 64-bit PCIX interface version.
*		This test is independent of the board's form factor.
*
*	Arguments:
*
*		dev	The device structure of interest.
*
*	Returned:
*
*		0	All went well.
*		1	There was an error.
*
******************************************************************************/

static int _id_hpdi32_64(dev_data_t* dev)
{
	static const dev_bar_dat_t	bar[6]	=
	{
		/* offset		flags	size			*/
		{ PCI_BASE_ADDRESS_0,	0,	512 /* PLX memory */	},
		{ PCI_BASE_ADDRESS_1,	0x1,	256 /* PLX I/O ports */	},
		{ PCI_BASE_ADDRESS_2,	0,	GSC_REG_BLOCK_SIZE	},
		{ PCI_BASE_ADDRESS_3,	0,	0 /* Unused. */		},
		{ PCI_BASE_ADDRESS_4,	0,	0 /* Unused. */		},
		{ PCI_BASE_ADDRESS_5,	0,	0 /* Unused. */		}
	};

	static const dev_reg_id_t	id[4]	=
	{
		/* reg				value			*/
		{ PCI_VENDOR_ID,		HPDI32_VENDOR_ID	},
		{ PCI_DEVICE_ID,		HPDI32_DEVICE_ID_64	},
		{ PCI_SUBSYSTEM_VENDOR_ID,	HPDI32_SUBVENDOR_ID	},
		{ PCI_SUBSYSTEM_ID,		HPDI32_SUBDEVICE_ID_64	}
	};

	dev_region_t	dummy;
	int		i;

	dev_region_t*		regions[6] =
	{
		&dev->plx_mem,
		&dev->plx_io,
		&dev->gsc_mem,
		&dummy,
		&dummy,
		&dummy
	};

	dev->type	= HPDI32_64_BIT;
	dev->bus_bits	= 64;
	i		= _id_verify(dev->pci, id, 4);

	if (i == 0)
		i	= _bar_verify(dev->pci, bar, regions, 6);

	return(i);
}



/******************************************************************************
*
*	Function:	cleanup_module
*
*	Purpose:
*
*		Clean things up when the kernel is about to unload the module.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void cleanup_module(void)
{
	int		i;
	dev_region_t*	list[3];

	for (i = 0; i < ARRAY_ELEMENTS(global.dev_list); i++)
	{
		if (global.dev_list[i])
		{
			dev_reset(global.dev_list[i]);
			list[0]	= &global.dev_list[i]->plx_mem;
			list[1]	= &global.dev_list[i]->plx_io;
			list[2]	= &global.dev_list[i]->gsc_mem;
			_regions_release(list, ARRAY_ELEMENTS(list));
			kfree(global.dev_list[i]);
			global.dev_list[i]	= NULL;
		}
	}

	if (global.major_number >= 0)
	{
		unregister_chrdev(global.major_number, MODULE_NAME);
		global.major_number	= -1;
	}

	proc_stop();

	if (global.driver_loaded)
	{
		global.driver_loaded	= 0;
		printk(	"HPDI32 driver version %s successfully removed.\n",
			global.driver_info.version);
	}

	global.dev_qty	= 0;
}



/******************************************************************************
*
*	Function:	dev_data_t_init
*
*	Purpose:
*
*		Initialize the idevice data structure. The initialization
*		done here must not assume that the device is open or closed.
*		Those related fields are untouched as we don't want to
*		interfere with the current state.
*
*	Arguments:
*
*		dev	The structure for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_data_t_init(dev_data_t* dev)
{
	dev_dma_t*	dma_ch;
	u32		id;
	VADDR_T		vaddr;

	sema_init(&dev->irq.sem, 1);

	memset(&dev->mmap, 0, sizeof(dev->mmap));

	dev->read.mode		= HPDI32_IO_MODE_DEFAULT;
	dev->read.timeout	= HPDI32_IO_TIMEOUT_DEFAULT;
	dev->read.buf		= NULL;
	dev->read.samples	= 0;
	sema_init(&dev->read.sem, 1);
	WAIT_QUEUE_HEAD_INIT(&dev->read.queue);

	dev->write.mode		= HPDI32_IO_MODE_DEFAULT;
	dev->write.timeout	= HPDI32_IO_TIMEOUT_DEFAULT;
	dev->write.buf		= NULL;
	dev->write.samples	= 0;
	sema_init(&dev->write.sem, 1);
	WAIT_QUEUE_HEAD_INIT(&dev->write.queue);

	dev->vaddr.plx_cntrl_32		= reg_vaddr(dev, HPDI32_PLX_CNTRL);
	dev->vaddr.plx_dmaarb_32	= reg_vaddr(dev, HPDI32_PLX_DMAARB);
	dev->vaddr.plx_dmacsr0_8	= reg_vaddr(dev, HPDI32_PLX_DMACSR0);
	dev->vaddr.plx_dmacsr1_8	= reg_vaddr(dev, HPDI32_PLX_DMACSR1);
	dev->vaddr.plx_dmathr_32	= reg_vaddr(dev, HPDI32_PLX_DMATHR);
	dev->vaddr.plx_intcsr_32	= reg_vaddr(dev, HPDI32_PLX_INTCSR);

	dev->vaddr.gsc_bcr_32		= reg_vaddr(dev, HPDI32_GSC_BCR);
	dev->vaddr.gsc_bsr_32		= reg_vaddr(dev, HPDI32_GSC_BSR);
	dev->vaddr.gsc_fdr_32		= reg_vaddr(dev, HPDI32_GSC_FDR);
	dev->vaddr.gsc_frr_32		= reg_vaddr(dev, HPDI32_GSC_FRR);
	dev->vaddr.gsc_fsr_32		= reg_vaddr(dev, HPDI32_GSC_FSR);
	dev->vaddr.gsc_icr_32		= reg_vaddr(dev, HPDI32_GSC_ICR);
	dev->vaddr.gsc_ielr_32		= reg_vaddr(dev, HPDI32_GSC_IELR);
	dev->vaddr.gsc_ihlr_32		= reg_vaddr(dev, HPDI32_GSC_IHLR);
	dev->vaddr.gsc_isr_32		= reg_vaddr(dev, HPDI32_GSC_ISR);
	dev->vaddr.gsc_rar_32		= reg_vaddr(dev, HPDI32_GSC_RAR);
	dev->vaddr.gsc_tar_32		= reg_vaddr(dev, HPDI32_GSC_TAR);

	dev->cache.gsc_frr	= readl(dev->vaddr.gsc_frr_32);
	vaddr			= reg_vaddr(dev, HPDI32_GSC_FSR);
	dev->cache.gsc_fsr	= readl(vaddr);
	vaddr			= reg_vaddr(dev, HPDI32_GSC_RFSR);
	dev->cache.gsc_rfsr	= readl(vaddr);
	vaddr			= reg_vaddr(dev, HPDI32_GSC_TFSR);
	dev->cache.gsc_tfsr	= readl(vaddr);

	if ((dev->cache.gsc_frr & HPDI32_GSC_FRR_FSR) == 0)
		dev->cache.gsc_fsr	= 0;

	if ((dev->cache.gsc_fsr & HPDI32_GSC_FSR_TFSR_RFSR) == 0)
	{
		dev->cache.gsc_rfsr	= 0;
		dev->cache.gsc_tfsr	= 0;
	}

	sema_init(&dev->dma.sem, 1);

	dma_ch			= &dev->dma.channel[0];
	dma_ch->in_use		= FALSE;
	dma_ch->flags		= _DMA_READ | _DMA_WRITE
				| _DMDMA_READ | _DMDMA_WRITE;
	dma_ch->irq_enable	= INTCSR_DMA_0_IRQ_ENABLE;
	dma_ch->vaddr.mode_32	= reg_vaddr(dev, HPDI32_PLX_DMAMODE0);
	dma_ch->vaddr.padr_32	= reg_vaddr(dev, HPDI32_PLX_DMAPADR0);
	dma_ch->vaddr.ladr_32	= reg_vaddr(dev, HPDI32_PLX_DMALADR0);
	dma_ch->vaddr.siz_32	= reg_vaddr(dev, HPDI32_PLX_DMASIZ0);
	dma_ch->vaddr.dpr_32	= reg_vaddr(dev, HPDI32_PLX_DMADPR0);
	dma_ch->vaddr.csr_8	= reg_vaddr(dev, HPDI32_PLX_DMACSR0);

	dma_ch			= &dev->dma.channel[1];
	dma_ch->in_use		= FALSE;
	dma_ch->irq_enable	= INTCSR_DMA_1_IRQ_ENABLE;
	dma_ch->vaddr.mode_32	= reg_vaddr(dev, HPDI32_PLX_DMAMODE1);
	dma_ch->vaddr.padr_32	= reg_vaddr(dev, HPDI32_PLX_DMAPADR1);
	dma_ch->vaddr.ladr_32	= reg_vaddr(dev, HPDI32_PLX_DMALADR1);
	dma_ch->vaddr.siz_32	= reg_vaddr(dev, HPDI32_PLX_DMASIZ1);
	dma_ch->vaddr.dpr_32	= reg_vaddr(dev, HPDI32_PLX_DMADPR1);
	dma_ch->vaddr.csr_8	= reg_vaddr(dev, HPDI32_PLX_DMACSR1);

	if (dev->cache.gsc_fsr & HPDI32_GSC_FSR_DMA_CH1)
		dma_ch->flags	= _DMA_READ | _DMA_WRITE | _DMDMA_WRITE;
	else
		dma_ch->flags	= 0;

	// Check for model specific characteristics.
	id	= dev->cache.gsc_frr & HPDI32_GSC_FRR_SUB_ID;

	if (id == HPDI32_GSC_FRR_SUB_ID_DIPHASE2)
	{
		// While the firmware refers to a BCR.Dual DMA bit, we don't
		// make use of it. Instead, it remains clear and we simply
		// use channel 0 for both Tx and Rx DMDMA operations.
		// Also, according to the hardware manual, the board doesn't
		// support simultaneous Tx and Rx, so clearing the BCR.Dual DMA
		// bit neither adds nor subtracts functionailty.
		strcpy(dev->model, "DIPHASE2");
		dev->dma.bcr_dir_bit		= 0;
		dev->dma.channel[0].flags	= _DMA_READ | _DMA_WRITE
						| _DMDMA_READ | _DMDMA_WRITE;
		dev->dma.channel[1].flags	= _DMA_READ | _DMA_WRITE;
	}
	else
	{
		strcpy(dev->model, "*");
		dev->dma.bcr_dir_bit	= 1;
	}
}



/******************************************************************************
*
*	Function:	dev_locate
*
*	Purpose:
*
*		Get a pointer to the device structure referenced by the given
*		i_node structure.
*
*	Arguments:
*
*		inode	This is the device inode structure.
*
*	Returned:
*
*		NULL	The respective device can't be accessed.
*		else	A pointer to the respective device.
*
******************************************************************************/

dev_data_t* dev_locate(struct inode* inode)
{
	dev_data_t*	dev;
	unsigned int	index	= MINOR(inode->i_rdev);

	if (index >= ARRAY_ELEMENTS(global.dev_list))
		dev	= NULL;
	else
		dev	= global.dev_list[index];

	return(dev);
}



/******************************************************************************
*
*	Function:	dev_reset
*
*	Purpose:
*
*		Reset the device to a default inactive state.
*
*	Arguments:
*
*		dev	The data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_reset(dev_data_t* dev)
{
	writel(0, dev->vaddr.plx_intcsr_32);		/* Interrupts off. */
	writel(HPDI32_GSC_BCR_BOARD_RESET, dev->vaddr.gsc_bcr_32);
	udelay(20);					/* Wait for reset. */
}



/******************************************************************************
*
*	Function:	init_module
*
*	Purpose:
*
*		Initialize the driver upon loading.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		0	All went well.
*		< 0	There was an error.
*
******************************************************************************/

int init_module(void)
{
	dev_data_t		dev;
	dev_region_t		dummy;

	dev_region_t*		regions[] =
	{
		&dev.plx_mem,
		&dev.plx_io,
		&dev.gsc_mem,
		&dummy,
		&dummy,
		&dummy
	};

	int		i;
	struct pci_dev*	pci;
	int		status	= -ENODEV;

	global.major_number	= -1;
	memset(&global.dev_list, 0, sizeof(global.dev_list));
	strcpy(global.driver_info.version, DRIVER_VERSION);
	sprintf(global.driver_info.built, "%s, %s", __DATE__, __TIME__);

	/*
	*	Locate any and all HPDI32 boards.
	*/

	PCI_DEVICE_LOOP(pci)
	{
		memset(&dummy, 0, sizeof(dummy));
		memset(&dev, 0, sizeof(dev_data_t));
		dev.pci	= pci;
		i	= _id_hpdi32_32(&dev);

		if (i)
			i	= _id_hpdi32_64(&dev);

		if (i)
			continue;

		i	= PCI_ENABLE_DEVICE(pci);

		if (i)
			continue;

		i	= _regions_verify(regions, ARRAY_ELEMENTS(regions));

		if (i)
			continue;

		/*
		*	We've got an HPDI32. Get the required resources.
		*/

		i	= _regions_request(regions, ARRAY_ELEMENTS(regions));

		if (i)
		{
			_regions_release(regions, ARRAY_ELEMENTS(regions));
			continue;
		}

		/* Initialize the structure.  */
		sema_init(&dev.sem, 1);
		dev_data_t_init(&dev);

		/* Reset the device and add it to our list. */
		dev_reset(&dev);
		i	= _dev_add(&dev);

		if (i)
			_regions_release(regions, ARRAY_ELEMENTS(regions));
		else
			global.dev_qty++;
	}

	/*
	*	Get the major number dynamicly.
	*/

	if (global.dev_qty)
	{
		global.fops.open	= open;
		global.fops.release	= close;
		global.fops.ioctl	= ioctl;
		global.fops.mmap	= mmap;
		global.fops.read	= read;
		global.fops.write	= write;
		global.fops.fasync	= fasync;
		SET_MODULE_OWNER(&global.fops);
		mmap_setup();
		global.major_number	= register_chrdev(	0,
								MODULE_NAME,
								&global.fops);

		if (global.major_number < 0)
			cleanup_module();
	}

	/*
	*	Other startup work.
	*/

	i	= ioctl_start();

	if (i)
		cleanup_module();

	i	= proc_start();

	if (i)
		cleanup_module();

	/*
	*	Finish up.
	*/

	if (global.dev_qty)
	{
		status			= 0;
		global.driver_loaded	= 1;
		printk(	"HPDI32 driver version %s successfully inserted.\n",
			DRIVER_VERSION);
	}

	return(status);
}



