/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 11 $ */
/* $Date: 7/30/07 8:25a $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.18.0-GSC-DN/driver/main.h $
 *
 * 11    7/30/07 8:25a Dnewbold
 * Updated the version number and comments and removed #include
 * <linux/config.h>.
 *
 * 10    9/28/06 5:48p Dnewbold
 * Updated the version number and comments. Added code for the Data Size
 * and PIO Threshold parameters.
 *
 * 9     8/21/06 2:29p Dnewbold
 * Updated the version number and comment.
 *
 * 8     5/30/06 1:17p Dnewbold
 * Updated the version number and comments.
 *
 * 7     4/17/06 5:09p Dnewbold
 *
 * 6     2/27/06 10:38a Dnewbold
 * Updatred for new I/O memory allocation mechanism. Added DIPHASE2
 * support.
 *
 * 5     2/01/06 3:00p Dnewbold
 * Updated the version number. Added the field
 * dev_data_t.vaddr.gsc_fsr_32. Changed I/O buffer allocation mechanism.
 *
 * 4     12/19/05 1:59p Dnewbold
 * Updated the version number.
 *
 * 3     9/01/05 1:31p Dnewbold
 * More 2.6 kernel porting.
 *
 * 2     1/25/05 3:46p Dnewbold
 * More modularization work, plus some housekeeping updates.
 *
 * 1     1/14/05 8:04a Dnewbold
*/

#ifndef __MAIN_H__
#define __MAIN_H__

#include "hpdi32.h"



#ifdef CONFIG_SMP
#define	__SMP__
#endif

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/page.h>
#include <asm/uaccess.h>

#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/proc_fs.h>
#include <linux/sched.h>
#include <linux/version.h>
#include <linux/wait.h>

#include "kernel_2_2.h"
#include "kernel_2_4.h"
#include "kernel_2_6.h"

#ifndef CONFIG_PCI
	#error This driver requires PCI support.
#endif



/* #defines	**************************************************************/

#define	DRIVER_VERSION			"1.19"
// 1.19	Updated the ISR prototype for the 2.6.19 kernel.
// 1.18	Added support for 8-bit and 16-bit I/O data samples.
// 1.17	Additional porting for newer 2.6 and 64-bit kernels.
// 1.16	Fixed a bug in that timed out DMA's reported that completed OK.
//	Mods to the PLX INTCSR are now done with restricted access.
// 1.15	Fixed a bug in the ISR - was disabling all but the serviced interrupts.
// 1.14	Modified how the I/O buffers are allocated, as well as their max size.
//	Modified cleanup code following a DMA operation.
//	Fixed a DMA timeout bug resulting in a negative timeout value.
//	Added support for the HPDI32-DIPHASE2.
//	Added a "model" field to the /proc/hpdi32 file.
//	Fixed bug in irq_gsc_acquire() - lock wasn't always released.
// 1.13	Fixed a bug in the HPDI32_IOCTL_REG_MOD service.
// 1.12	More porting to the 2.6 kernel.
// 1.11	More modularization work, plus some housekeeping updates.
// 1.10	Ported to the 2.6 kernel.
//	Modularized the sources.

#define	FALSE				0
#define	TRUE				1

#define	ARRAY_ELEMENTS(a)		(sizeof((a))/sizeof((a)[0]))
#define	BYTES_TO_PAGE_BYTES(b)		ROUND_UP_BOUNDARY((b), PAGE_SIZE)
#define	GSC_REG_BLOCK_SIZE		512	/* Register block size. */
#define	MAX_DEVICES			32
#define	MODULE_NAME			"hpdi32"
#define	ROUND_TO_PAGE_BOUNDARY(p)	ROUND_UP_BOUNDARY((u32)(p), PAGE_SIZE)
#define	ROUND_UP_BOUNDARY(by,bl)	((((by) + (bl) - 1) / (bl)) * (bl))

/* DMA items */
#define	_DMA_READ			0x1
#define	_DMA_WRITE			0x2
#define	_DMDMA_READ			0x4
#define	_DMDMA_WRITE			0x8

#define	DMA_CSR_ENABLE			HPDI32_ENCODE(1,0,0)
#define	DMA_CSR_START			HPDI32_ENCODE(1,1,1)
#define	DMA_CSR_ABORT			HPDI32_ENCODE(1,2,2)
#define	DMA_CSR_CLEAR			HPDI32_ENCODE(1,3,3)
#define	DMA_CSR_DONE			HPDI32_ENCODE(1,4,4)

#define	DMA_DPR_END_OF_CHAIN		HPDI32_ENCODE(1,1,1)
#define	DMA_DPR_TERMINAL_COUNT_IRQ	HPDI32_ENCODE(1,2,2)
#define	DMA_DPR_HOST_TO_BOARD		HPDI32_ENCODE(0,3,3)
#define	DMA_DPR_BOARD_TO_HOST		HPDI32_ENCODE(1,3,3)

#define	DMA_MODE_SIZE_8_BITS		HPDI32_ENCODE(0,1,0)
#define	DMA_MODE_SIZE_16_BITS		HPDI32_ENCODE(1,1,0)
#define	DMA_MODE_SIZE_32_BITS		HPDI32_ENCODE(2,1,0)
#define	DMA_MODE_INPUT_ENABLE		HPDI32_ENCODE(1,6,6)
#define	DMA_MODE_BURSTING_LOCAL		HPDI32_ENCODE(1,8,8)
#define	DMA_MODE_INTERRUPT_WHEN_DONE	HPDI32_ENCODE(1,10,10)
#define	DMA_MODE_LOCAL_ADRESS_CONSTANT	HPDI32_ENCODE(1,11,11)
#define	DMA_MODE_STANDARD_DMA		HPDI32_ENCODE(0,12,12)
#define	DMA_MODE_DEMAND_MODE_DMA	HPDI32_ENCODE(1,12,12)
#define	DMA_MODE_PCI_INTERRUPT_ENABLE	HPDI32_ENCODE(1,17,17)

/* Interrupt configuration macros. */
#define	_INT_INDEX_FRAME_VALID_S	0
#define	_INT_INDEX_FRAME_VALID_E	1
#define	_INT_INDEX_D1			2
#define	_INT_INDEX_D2			3
#define	_INT_INDEX_D3			4
#define	_INT_INDEX_D4			5
#define	_INT_INDEX_D5			6
#define	_INT_INDEX_D6			7
#define	_INT_INDEX_TX_FIFO_EMPTY	8
#define	_INT_INDEX_TX_FIFO_AE		9
#define	_INT_INDEX_TX_FIFO_AF		10
#define	_INT_INDEX_TX_FIFO_FULL		11
#define	_INT_INDEX_RX_FIFO_EMPTY	12
#define	_INT_INDEX_RX_FIFO_AE		13
#define	_INT_INDEX_RX_FIFO_AF		14
#define	_INT_INDEX_RX_FIFO_FULL		15

#define	_INT_EL_EDGE			0x0
#define	_INT_EL_LEVEL			0x1

#define	_INT_HL_HIGH			0x0
#define	_INT_HL_LOW			0x1

/* GSC IRQ items. */
#define	IRQ_GSC_FAIL			(-1)	/* Request failed. */
#define	IRQ_GSC_TIME_SIG		0	/* Got a signal or timeout. */
#define	IRQ_GSC_IRQ			1	/* Got the interrupt. */

/* PLX Interrupt Control/Status Register bits. */
#define	INTCSR_PCI_INT_ENABLE		HPDI32_ENCODE(1,8,8)
#define	INTCSR_LOCAL_INT_ENABLE		HPDI32_ENCODE(1,11,11)
#define	INTCSR_LOCAL_INT_OUTPUT_ENABLE	HPDI32_ENCODE(1,16,16)
#define	INTCSR_DMA_0_IRQ_ENABLE		HPDI32_ENCODE(1,18,18)
#define	INTCSR_DMA_1_IRQ_ENABLE		HPDI32_ENCODE(1,19,19)
#define	INTCSR_DMA_0_IRQ_ACTIVE		HPDI32_ENCODE(1,21,21)
#define	INTCSR_DMA_1_IRQ_ACTIVE		HPDI32_ENCODE(1,22,22)



/* typedefs	**************************************************************/

typedef struct
{
	int			in_use;
	unsigned int		flags;
	WAIT_QUEUE_HEAD_T*	queue;		/* Queue to resume on IRQ. */
	int*			condition;	/* To resume on IRQ. 2.6 */
	int			irq_enable;

	struct
	{
		VADDR_T	mode_32;	/* DMAMODEx */
		VADDR_T	padr_32;	/* DMAPADRx */
		VADDR_T	ladr_32;	/* DMALADRx */
		VADDR_T	siz_32;		/* DMASIZx */
		VADDR_T	dpr_32;		/* DMADPRx */
		VADDR_T	csr_8;		/* DMACSRx */
	} vaddr;

} dev_dma_t;

typedef struct
{
	struct semaphore	sem;		/* one at a time */
	s32			mode;
	s32			timeout;	/* In seconds. */

	// Memory allocations.
	unsigned long		adrs;		/* From get free pages. */
	u32*			buf;		/* Page aligned. */
	s32			bytes;		/* Rounded to page boundary. */
	u8			order;		// Order of the page request.
	s32			pio_threshold;	// Use PIO if qty <= this.
	u32			samples;	/* User requested size. */
	u32			sample_bits;	// 8, 16, 32

	// Waiting for interrupts.
	WAIT_QUEUE_HEAD_T	queue;
	int			condition;	/* 2.6 kernel */
} dev_io_t;

typedef struct
{
	int	usage;		/* For read()/write() restrictions. */
	u32	offset;		/* For mmap() call. */
	u32	size;		/* For mmap() call. */
	u32	pg_size;	/* "size" rounded up to page end. */
} dev_mmap_t;

typedef struct
{
	u32	phys_adrs;	/* The base physical address from the BAR. */
	void*	vaddr;		/* The virtual address (memory only). */
	u32	flags;		/* The BAR address flags (i.e. TYPE, ...). */
	u32	size;		/* The size of the region in bytes. */
	u32	requested;	/* Have we requested the region. */
} dev_region_t;

typedef struct
{
	unsigned long		reg;
	unsigned long		type;
	unsigned long		size;	/* 1, 2 or 4 bytes */
	unsigned long		offset;	/* Allign per size. */
	VADDR_T			vaddr;	/* Virtual address. */
	const dev_region_t*	region;
} dev_reg_t;

typedef enum
{
	HPDI32_32_BIT,
	HPDI32_64_BIT
} dev_type_t;

typedef struct
{
	struct semaphore	sem;		/* control process access */
	struct pci_dev*		pci;		/* For PCI access. */
	dev_type_t		type;		/* HPDI32 board type. */
	u32			bus_bits;	// PCI bus width in bits
	char			model[16];
	dev_region_t		plx_mem;	/* PLX features I/O memory. */
	dev_region_t		plx_io;		/* PLX features I/O ports. */
	dev_region_t		gsc_mem;	/* GSC features I/O memory. */
	int			in_use;		/* Only one open() at a time. */
	u32			debug[HPDI32_DEBUG_DATA_VALUES];
	dev_io_t		read;
	dev_io_t		write;

	struct
	{
		struct semaphore	sem;		/* control access */
		dev_dma_t		channel[2];
		int			bcr_dir_bit;	// Set BCR Dir bit?
	} dma;

	struct
	{
		struct semaphore	sem;		/* control access */
		unsigned char		number;		/* IRQ number. */
		u32			driver_gsc;	/* Driver used. */
		u32			user_gsc;	/* User used. */
		u32			user_other;	/* User used. */
		u32			status_gsc;	/* Logged IRQs. */
		u32			status_other;	/* Logged IRQs. */
		struct fasync_struct*	async;		/* Notify apps. */
	} irq;

	struct
	{
		int		usage;
		dev_mmap_t	regs;
		dev_mmap_t	read;
		dev_mmap_t	write;
	} mmap;

	struct
	{
		VADDR_T	plx_cntrl_32;	/* Serial EEPROM Control */
		VADDR_T	plx_dmaarb_32;	/* DMA Mode Arbitration */
		VADDR_T	plx_dmacsr0_8;	/* DMA 0 Command/Status */
		VADDR_T	plx_dmacsr1_8;	/* DMA 1 Command/Status */
		VADDR_T	plx_dmathr_32;	/* DMA Threshold */
		VADDR_T	plx_intcsr_32;	/* Interrupt Control/Status */

		VADDR_T	gsc_bcr_32;	/* Board Control */
		VADDR_T	gsc_bsr_32;	/* Board Status */
		VADDR_T	gsc_fdr_32;	/* FIFO Data */
		VADDR_T	gsc_frr_32;	/* Firmware Revision */
		VADDR_T	gsc_fsr_32;	/* Feature Set */
		VADDR_T	gsc_icr_32;	/* Interrupt Control */
		VADDR_T	gsc_ielr_32;	/* Interrupt Edge/Level */
		VADDR_T	gsc_ihlr_32;	/* Interrupt Hi/Low */
		VADDR_T	gsc_isr_32;	/* Interrupt Status */
		VADDR_T	gsc_rar_32;	/* Rx Almost Register */
		VADDR_T	gsc_tar_32;	/* Tx Almost Register */
	} vaddr;

	struct
	{
		u32	gsc_frr;	/* Firmware Revision */
		u32	gsc_fsr;	/* Feature Set */
		u32	gsc_rfsr;	/* Rx FIFO Size */
		u32	gsc_tfsr;	/* Tx FIFO Size */
	} cache;
} dev_data_t;

typedef struct
{
	u32				debug[HPDI32_DEBUG_DATA_VALUES];
	dev_data_t*			dev_list[MAX_DEVICES];/* minor index */
	int				dev_qty;
	hpdi32_driver_info_t		driver_info;
	int				driver_loaded;
	struct file_operations		fops;
	int				major_number;
	int				proc_enabled;
	struct vm_operations_struct	vmops;
} dev_global_t;



/* variables	**************************************************************/

extern	dev_global_t	global;



/* prototypes	**************************************************************/

int		close(struct inode* inode, struct file* filp);
void		dev_data_t_init(dev_data_t* dev);
dev_data_t*	dev_locate(struct inode* inode);
void		dev_reset(dev_data_t* dev);
ssize_t		dma_read_available(dev_data_t* dev, ssize_t samples);
ssize_t		dma_write_available(dev_data_t* dev, ssize_t samples);
ssize_t		dmdma_read_available(dev_data_t* dev, ssize_t samples);
ssize_t		dmdma_write_available(dev_data_t* dev, ssize_t samples);
int		fasync(int fd, struct file* filp, int mode);
void		fasync_close_dev(dev_data_t* dev, struct file* filp);
void		io_close_dev(dev_data_t* dev);
int		io_config_ioctl(dev_data_t* dev, hpdi32_io_config_t* arg);
int		ioctl_start(void);
int		irq_access_lock(dev_data_t* dev, int isr);
void		irq_access_unlock(dev_data_t* dev, int isr);
void		irq_intcsr_mod(dev_data_t* dev, u32 value, u32 mask);
int		irq_isr_common(int irq, void* dev_id);
int		irq_request(dev_data_t* dev);
int		irq_notify_ioctl(dev_data_t* dev, hpdi32_interrupt_t* arg);
int		irq_status_ioctl(dev_data_t* dev, hpdi32_interrupt_t* arg);
int		mmap(struct file* filp, struct vm_area_struct* vma);
void		mmap_close_dev(dev_data_t* dev);
void		mmap_info_compute(dev_data_t* dev);
int		mmap_info_ioctl(dev_data_t* dev, hpdi32_mmap_t* arg);
void		mmap_setup(void);
int		open(struct inode* inode, struct file* filp);
ssize_t		pio_read_available(dev_data_t* dev, ssize_t samples);
ssize_t		pio_write_available(dev_data_t* dev, ssize_t samples);
int		proc_start(void);
void		proc_stop(void);
int		reg_mod_ioctl(dev_data_t* dev, hpdi32_reg_t* arg);
int		reg_read_ioctl(dev_data_t* dev, hpdi32_reg_t* arg);
VADDR_T		reg_vaddr(dev_data_t* dev, unsigned long reg);
int		reg_write_ioctl(dev_data_t* dev, hpdi32_reg_t* arg);

ssize_t	dma_perform(		dev_data_t*		dev,
				WAIT_QUEUE_HEAD_T*	queue,
				int*			cond,
				unsigned long		jit_end,
				unsigned int		ability,
				unsigned int		direction,
				unsigned int		type,
				int			sample_bits,
				void*			buff,
				ssize_t			samples);

ssize_t	dma_read_work(		dev_data_t*		dev,
				u8*			buff,
				ssize_t			samples,
				unsigned long		jit_end);

ssize_t	dma_write_work(		dev_data_t*		dev,
				const u8*		buff,
				ssize_t			samples,
				unsigned long		jit_end);

ssize_t	dmdma_read_work(	dev_data_t*		dev,
				u8*			buff,
				ssize_t			samples,
				unsigned long		jit_end);

ssize_t	dmdma_write_work(	dev_data_t*		dev,
				const u8*		buff,
				ssize_t			samples,
				unsigned long		jit_end);

int	ioctl(			struct inode*		inode,
				struct file*		filp,
				unsigned int		cmd,
				unsigned long		arg);

int	irq_gsc_acquire(	dev_data_t* 		dev,
				WAIT_QUEUE_HEAD_T*	queue,
				int*			cond,
				unsigned long		jit_end,
				unsigned int		index,
				int			el,
				int			hl);

void	mmap_nopage_common(	struct vm_area_struct*	vma,
				unsigned long		address,
				struct page**		page);

ssize_t	pio_read_work(		dev_data_t*		dev,
				u8*			buff,
				ssize_t			samples,
				unsigned long		jit_end);

ssize_t	pio_write_work(		dev_data_t*		dev,
				const u8*		buff,
				ssize_t			samples,
				unsigned long		jit_end);

int	proc_read(		char*			page,
				char**			start,
				off_t			offset,
				int			count,
				int*			eof,
				void*			data);

ssize_t	read(			struct file*		filp,
				char*			buff,
				size_t			count,
				loff_t*			offp);

void	reg_decode(		const dev_region_t*	region,
				unsigned long		reg,
				dev_reg_t*		dreg);

ssize_t	write(			struct file*		filp,
				const char*		buff,
				size_t			count,
				loff_t*			offp);



#endif
