/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 2 $ */
/* $Date: 9/28/06 5:47p $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.17.0-GSC-DN/driver/open.c $
 *
 * 2     9/28/06 5:47p Dnewbold
 * Added code to support the Data Size and PIO Threshold parameters.
 *
 * 1     1/14/05 8:04a Dnewbold
*/

#include "main.h"



/******************************************************************************
*
*	Function:	open
*
*	Purpose:
*
*		Implement the open() procedure.
*
*	Arguments:
*
*		inode	This is the device inode structure.
*
*		filp	This is the file structure.
*
*	Returned:
*
*		0	All went well.
*		< 0	There was a problem.
*
******************************************************************************/

int open(struct inode* inode, struct file* filp)
{
	#define	_RW_	(FMODE_READ | FMODE_WRITE)

	dev_data_t*		dev;
	hpdi32_io_config_t	io;
	int			status;
	int			test;
	unsigned long		ul;

	for (;;)	/* We'll use a loop for convenience. */
	{
		if ((filp->f_mode & _RW_) != _RW_)
		{
			/* Read and write access are both required. */
			status	= -EINVAL;
			break;
		}

		dev	= dev_locate(inode);

		if (dev == NULL)
		{
			/* The referenced device doesn't exist. */
			status	= -ENODEV;
			break;
		}

		test	= down_interruptible(&dev->sem);

		if (test)
		{
			/* We didn't get the semaphore. */
			status	= -ERESTARTSYS;
			break;
		}

		if (dev->in_use)
		{
			/* The device is already being used. */
			status	= -EBUSY;
			up(&dev->sem);
			break;
		}

		dev->in_use	= TRUE;
		status		= irq_request(dev);

		if (status)
		{
			/* We can't get the interrupt. */
			status		= -EBUSY;
			dev->in_use	= FALSE;
			up(&dev->sem);
			break;
		}

		dev_data_t_init(dev);
		io.read.mode		= HPDI32_IO_MODE_DEFAULT;
		io.read.timeout		= HPDI32_IO_TIMEOUT_DEFAULT;
		io.read.samples		= HPDI32_IO_SAMPLES_DEFAULT;
		io.read.sample_bits	= HPDI32_IO_SAMPLE_BITS_DEFAULT;
		io.read.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_DEFAULT;
		io.write.mode		= HPDI32_IO_MODE_DEFAULT;
		io.write.timeout	= HPDI32_IO_TIMEOUT_DEFAULT;
		io.write.samples	= HPDI32_IO_SAMPLES_DEFAULT;
		io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_DEFAULT;
		io.write.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_DEFAULT;
		io_config_ioctl(dev, &io);

		if ((dev->read.buf == NULL) || (dev->write.buf == NULL))
		{
			io_close_dev(dev);
			status		= -ENOMEM;
			dev->in_use	= FALSE;
			free_irq(dev->irq.number, dev);
			up(&dev->sem);
			break;
		}

		/* Open the device. */
		MOD_INC_USE_COUNT;
		dev_reset(dev);
		ul	= INTCSR_PCI_INT_ENABLE
			| INTCSR_LOCAL_INT_ENABLE
			| INTCSR_LOCAL_INT_OUTPUT_ENABLE;
		writel(ul, dev->vaddr.plx_intcsr_32);/* Interrupts on. */
		filp->private_data	= dev;
		status			= 0;
		up(&dev->sem);
		break;
	}

	return(status);
}



