/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 2 $ */
/* $Date: 2/22/06 10:20a $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32_Linux/HPDI32_Linux_Ver.1.13.0-GSC-DN/driver/proc.c $
 *
 * 2     2/22/06 10:20a Dnewbold
 * Added DIPHASE2 support.
 *
 * 1     1/14/05 8:04a Dnewbold
*/

#include "main.h"



/******************************************************************************
*
*	Function:	proc_read
*
*	Purpose:
*
*		Implement the read service for the module's /proc file system
*		entry. Read the documentation on this service for details, as
*		we ignore some arguments according to our needs and the
*		documentation.
*
*	Arguments:
*
*		page	The data produced is put here.
*
*		start	Records pointer to where the data in "page" begins.
*
*		offset	The offset into the file where we're to begin.
*
*		count	The limit to the amount of data we can produce.
*
*		eof	Set this flag when we hit EOF.
*
*		data	A private data item that we may use, but don't.
*
*	Returned:
*
*		int	The number of characters written.
*
******************************************************************************/

int proc_read(
	char*	page,
	char**	start,
	off_t	offset,
	int	count,
	int*	eof,
	void*	data)
{
#define	_PROC_MSG_SIZE	256

#if PAGE_SIZE < _PROC_MSG_SIZE
#error	"/proc file size may be too big."
#endif
	char*	comma;
	int	i;
	char*	str;

	MOD_INC_USE_COUNT;
	i	= sprintf(page,
			"version: %s\n"
			"built: %s\n"
			"boards: %d\n",
			global.driver_info.version,
			global.driver_info.built,
			global.dev_qty);
	page[i]	= 0;
	comma	= "";
	strcat(page, "types: ");

	for (i = 0; i < MAX_DEVICES; i++)
	{
		if (global.dev_list[i] == NULL)
			continue;

		switch (global.dev_list[i]->type)
		{
			default:

				printk(	"<1>ERROR: unsupported type: %d\n",
					(int) global.dev_list[i]->type);
				str	= "ERROR";
				break;

			case HPDI32_32_BIT:

				str	= "32";
				break;

			case HPDI32_64_BIT:

				str	= "64";
				break;
		}

		strcat(page, comma);
		strcat(page, str);
		comma	= ",";
	}

	strcat(page, "\n");
	strcat(page, "models: ");
	comma	= "";

	for (i = 0; i < MAX_DEVICES; i++)
	{
		if (global.dev_list[i] == NULL)
			continue;

		strcat(page, comma);
		strcat(page, global.dev_list[i]->model);
		comma	= ",";
	}

	strcat(page, "\n");
	i	= strlen(page) + 1;

	if (i >= PAGE_SIZE)
		printk("<1>/proc/%s is larger than PAGE_SIZE\n", MODULE_NAME);

	i--;
	eof[0]	= 1;
	MOD_DEC_USE_COUNT;
	return(i);
}



/******************************************************************************
*
*	Function:	proc_start
*
*	Purpose:
*
*		initialize use of the /proc file system.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		0	All went well.
*		! 0	We failed.
*
******************************************************************************/

int proc_start(void)
{
	struct proc_dir_entry*	proc;
	int			ret;

	if (global.dev_qty)
	{
		remove_proc_entry(MODULE_NAME, NULL);	/* Just in case. */
		proc	= create_proc_entry(MODULE_NAME, S_IRUGO, NULL);

		if (proc)
		{
			ret			= 0;
			global.proc_enabled	= 1;
			proc->read_proc		= proc_read;
			proc->get_info		= proc_get_info;
		}
		else
		{
			ret	= 1;
			printk(	"<1>%s: create_proc_entry() failure.\n",
				MODULE_NAME);
		}
	}
	else
	{
		ret	= 0;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	proc_stop
*
*	Purpose:
*
*		Stop use of the /proc file system.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void proc_stop(void)
{
	if (global.proc_enabled)
	{
		remove_proc_entry(MODULE_NAME, NULL);
		global.proc_enabled	= 0;
	}
}



