/* $Revision: 2 $ */
/* $Date: 5/30/06 1:11p $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.15.0-GSC-DN/irq/irq1.c $
 *
 * 2     5/30/06 1:11p Dnewbold
 * Added ability to test specific interrupts.
 *
 * 1     4/17/06 5:06p Dnewbold
*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>

#include "HPDI32DocSrcLib.h"
#include "main.h"
#include "util_label.h"



/******************************************************************************
*
*	Function:	_io_test
*
*	Purpose:
*
*		Verify the interrupt using an I/O Transfer.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int _io_test(int fd)
{
	__u32				bit		= HPDI32_INT_GSC_FRAME_VALID_E;
	int					errs	= 0;
	hpdi32_interrupt_t	irq;
	__u32				reg;

	label("Frame Valid End");
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_BCR, 1, 0);

	// Perform initialization.
	errs	+= hpdi32_reg_write(fd,
								HPDI32_GSC_BCR,
								HPDI32_GSC_BCR_TX_ENABLE,
								0);

	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_FDR, 0, 0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_FDR, 0, 0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_FDR, 0, 0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_FDR, 0, 0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_TLVLCR, 1, 0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_TLILCR, 0x0000FFFF, 0);

	memset(&irq, 0, sizeof(irq));
	irq.notify.gsc	= bit;
	errs	+= hpdi32_int_notify(fd, &irq, 0);

	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_IELR, bit, 0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_IHLR, bit, 0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_ICR, bit, 0);
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_ICR, &reg, 0);
	errs	+= (reg & bit) ? 0 : 1;

	// Cause the interrupt - via a transfer.
	errs	+= hpdi32_reg_write(fd,
								HPDI32_GSC_BCR,
								HPDI32_GSC_BCR_TX_ENABLE |
								HPDI32_GSC_BCR_TX_START,
								0);
	usleep(10000);	// Wait for the transfer to complete.

	// Verify the results.
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_ICR, &reg, 0);
	errs	+= (reg & bit) ? 1 : 0;

	errs	+= hpdi32_int_status(fd, &irq, 0);
	errs	+= (irq.status.gsc == bit) ? 0 : 1;

	// Done.
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_BCR, 1, 0);
	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/******************************************************************************
*
*	Function:	_gpio_test
*
*	Purpose:
*
*		Verify the interrupt using the GPIO lines.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _gpio_test(int fd)
{
	__u32				bit		= HPDI32_INT_GSC_FRAME_VALID_E;
	int					errs	= 0;
	__u32				frr;
	__u32				fsr;
	hpdi32_interrupt_t	irq;
	__u32				reg;

	label("GPIO 6 (H -> L)");
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_BCR, 1, 0);

	// Perform initialization.
	memset(&irq, 0, sizeof(irq));
	irq.notify.gsc	= bit;
	errs	+= hpdi32_int_notify(fd, &irq, 0);

	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_IELR, bit, 0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_IHLR, 0, 0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_ICR, bit, 0);
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_ICR, &reg, 0);
	errs	+= (reg & bit) ? 0 : 1;

	// Cause the interrupt - via a GPIO low to high transition.
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_D0_OUT_H, 0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_D0_OUT_L, 0);
	usleep(1);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_D0_FV, 0);

	// Verify the results.
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_ICR, &reg, 0);
	errs	+= (reg & bit) ? 1 : 0;

	errs	+= hpdi32_int_status(fd, &irq, 0);
	errs	+= (irq.status.gsc == bit) ? 0 : 1;

	// Done.
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_BCR, 1, 0);
	errs	= errs ? 1 : 0;

	// Verify that the feature is supported.
	hpdi32_reg_read(fd, HPDI32_GSC_FRR, &frr, 0);
	hpdi32_reg_read(fd, HPDI32_GSC_FSR, &fsr, 0);

	if (((frr & 0x80000000) == 0) || ((fsr & 0x40) == 0))
	{
		errs	= 0;
		printf("SKIPPED  (GPIO 6 not supported)\n");
	}
	else
	{
		printf("%s\n", errs ? "FAIL <---" : "PASS");
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	irq_1_test
*
*	Purpose:
*
*		Perform an IRQ 1 test.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		irq		The mask of interrupts to test.
*
*		fc		The mask of Flow Control interrupts to test.
*
*		gpio	The mask of GPIO interrupts to test.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int irq_1_test(int fd, int irq, int fc, int gpio)
{
	int	errs	= 0;

	if (irq & 0x2 & (fc | gpio))
	{
		label("IRQ 1");
		printf("\n");
		label_level_inc();

		if (fc & 0x2)
			errs	+= _io_test(fd);

		if (gpio & 0x2)
			errs	+= _gpio_test(fd);

		label_level_dec();
	}

	return(errs);
}



