/* $Revision: 2 $ */
/* $Date: 5/30/06 1:11p $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.15.0-GSC-DN/irq/irq11.c $
 *
 * 2     5/30/06 1:11p Dnewbold
 * Added ability to test specific interrupts.
 *
 * 1     4/17/06 5:06p Dnewbold
*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>

#include "HPDI32DocSrcLib.h"
#include "main.h"
#include "util_label.h"



/******************************************************************************
*
*	Function:	_fifo_test
*
*	Purpose:
*
*		Verify the interrupt using the FIFO fill level.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _fifo_test(int fd)
{
	__u32				bit		= HPDI32_INT_GSC_TX_FIFO_FULL;
	int					errs	= 0;
	hpdi32_interrupt_t	irq;
	long				l;
	__u32				reg;

	label("Tx FIFO Full");
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_BCR, 1, 0);

	// Perform initialization.
	memset(&irq, 0, sizeof(irq));
	irq.notify.gsc	= bit;
	errs	+= hpdi32_int_notify(fd, &irq, 0);

	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_IELR, bit, 0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_IHLR, bit, 0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_ICR, bit, 0);
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_ICR, &reg, 0);
	errs	+= (reg & bit) ? 0 : 1;

	// Cause the interrupt - via a transfer.

	for (l = 0; l <= (128L * 1024); l++)
		errs	+= hpdi32_reg_write(fd, HPDI32_GSC_FDR, 0, 0);

	usleep(1);

	// Verify the results.
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_ICR, &reg, 0);
	errs	+= (reg & bit) ? 1 : 0;

	errs	+= hpdi32_int_status(fd, &irq, 0);
	errs	+= (irq.status.gsc == bit) ? 0 : 1;

	// Done.
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_BCR, 1, 0);
	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/******************************************************************************
*
*	Function:	irq_11_test
*
*	Purpose:
*
*		Perform an IRQ 11 test.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		irq		The mask of interrupts to test.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int irq_11_test(int fd, int irq)
{
	int	errs	= 0;

	if (irq & 0x800)
	{
		label("IRQ 11");
		printf("\n");
		label_level_inc();

		errs	+= _fifo_test(fd);

		label_level_dec();
	}

	return(errs);
}



