/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 3 $ */
/* $Date: 9/28/06 5:38p $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.17.0-GSC-DN/test/io.c $
 *
 * 3     9/28/06 5:38p Dnewbold
 * Updated per changes to the 1.18 driver and added data size tests.
 *
 * 2     4/17/06 5:10p Dnewbold
 * Corrected use of macros.
 *
 * 1     2/27/06 10:40a Dnewbold
 * Modularized the test code.
*/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/timeb.h>

#include "main.h"
#include "HPDI32DocSrcLib.h"
#include "util_label.h"



/* #defines	***************************************************************/

#define	_1K				(1024L)
#define	_100K			(100L * _1K)

#define	IO_TIMEOUT		1	/* don't block */

#define	FALSE			0
#define	TRUE			1



/* variables *****************************************************************/

static	__u32	_big_tx_buffer[128L * 1024];
static	__u32	_big_rx_buffer[128L * 1024];



/*****************************************************************************
*
*	Function:	_value_format
*
*	Purpose:
*
*		Format a decimal number using K, M and G multipliers.
*
*	Arguments:
*
*		value	The value to format.
*
*		dest	The formatted value goes here.
*
*		size	This is the size of the above buffer.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _value_format(unsigned long value, char* dest, size_t size)
{
	char	buf[32];
	char*	mult	= "";

	if ((value) && ((value % _1K) == 0))
	{
		mult	= "K";
		value	/= _1K;
	}

	if ((value) && ((value % _1K) == 0))
	{
		mult	= "M";
		value	/= _1K;
	}

	if ((value) && ((value % _1K) == 0))
	{
		mult	= "G";
		value	/= _1K;
	}

	sprintf(buf, "%lu%s", value, mult);
	strncpy(dest, buf, size);
	dest[size - 1]	= 0;
}



/*****************************************************************************
*
*	Function:	_write_read_mode_tests
*
*	Purpose:
*
*		Verify that the specified data transfer read and write mode
*		operates properly.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		mode		The data transfer mode to use.
*
*		tx_values	The number of values to write.
*
*		rx_values	The number of values to read.
*
*		inactivity	Is an inactivity timeout expected?
*
*		function	Call this function to perform the actual tests.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _write_read_mode_tests(
			int		fd,
			__s32		mode,
			unsigned long	tx_values,
			unsigned long	rx_values,
			int		inactivity,
			int		(*function)(
						int		fd,
						unsigned long*	tx_values,
						unsigned long*	rx_values,
						int		inactivity))
{
	char			buf[64];
	hpdi32_io_config_t	io;
	char			rxv[8];
	int			status;
	char			txv[8];

	_value_format(tx_values, txv, sizeof(txv));
	_value_format(rx_values, rxv, sizeof(rxv));
	sprintf(buf, "Write %s, Read %s", txv, rxv);
	label(buf);
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	io.read.mode		= mode;
	io.read.timeout		= IO_TIMEOUT;
	io.read.samples		= HPDI32_IO_SAMPLES_NO_CHANGE;
	io.read.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.read.pio_threshold	= 0;
	io.write.mode		= mode;
	io.write.timeout	= IO_TIMEOUT;
	io.write.samples	= HPDI32_IO_SAMPLES_NO_CHANGE;
	io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.write.pio_threshold	= 0;
	status	= hpdi32_io_config(fd, &io, 0);

	if (status == 0)
	{
		status	= (function)(	fd,
					&tx_values,
					&rx_values,
					inactivity);
	}

	printf("%s", status ? "FAIL" : "PASS");
	_value_format(tx_values, txv, sizeof(txv));
	_value_format(rx_values, rxv, sizeof(rxv));
	printf(" (WT %s, RD %s)\n", txv, rxv);
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	return(status);
}



/*****************************************************************************
*
*	Function:	_write_read_test_basic
*
*	Purpose:
*
*		Perform a single threaded read-write test using the basic
*		read/write services. We perform this test by writing data of a
*		given pattern while reading and comparing what we got to what
*		we sent. If it is the same, then the test passes.
*
*	Arguments:
*
*		fd		The handle of the board of interest.
*
*		tx_values	Gives the number of values to write.
*				Return number actually written.
*
*		rx_values	Gives the number of values to read.
*				Return number actually read.
*
*		inactivity	Is an inactivity timeout expected?
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem.
*
*****************************************************************************/

static int _write_read_test_basic(	int		fd,
					unsigned long*	tx_values,
					unsigned long*	rx_values,
					int		inactivity)
{
	#define	XFER_BLOCK_SAMPLES	1024
	#define	XFER_INACTIVITY_SEC	1

	unsigned long	bcr;
	long		get_bytes;
	__u32*		get_dest	= 0;
	int		get_values	= 0;
	long		got_bytes;
	long		got_values;
	unsigned long	got_total	= 0;
	int		i;
	size_t		memcmp_bytes	= 0;
	size_t		memcmp_values	= 0;
	__u32		rx_buf[XFER_BLOCK_SAMPLES];
	long		send_bytes;
	__u32*		send_dest	= NULL;
	int		send_values	= 0;
	long		sent_bytes;
	int		sent_values;
	unsigned long	sent_total	= 0;
	int		status		= 0;
	time_t		timeout		= time(NULL) + XFER_INACTIVITY_SEC;
	__u32		tx_buf[XFER_BLOCK_SAMPLES];

	for (i = 0; i < ARRAY_ELEMENTS(tx_buf); i++)
		tx_buf[i]	= (unsigned long) i;

	bcr	= 0;

	if (tx_values[0])
	{
		/* Enable the transmit before receive. */
		bcr	|= HPDI32_GSC_BCR_TX_ENABLE;
		hpdi32_reg_write(fd, HPDI32_GSC_BCR, bcr, 0);
	}

	if (rx_values[0])
	{
		bcr	|= HPDI32_GSC_BCR_RX_ENABLE;
		hpdi32_reg_write(fd, HPDI32_GSC_BCR, bcr, 0);
	}

	if (tx_values[0])
		bcr	|= HPDI32_GSC_BCR_TX_START;

	for (; (sent_total < tx_values[0]) || (got_total < rx_values[0]);)
	{
		if (sent_total < tx_values[0])
		{
			if (send_values <= 0)
			{
				send_values	= tx_values[0] - sent_total;

				if (send_values > XFER_BLOCK_SAMPLES)
					send_values	= XFER_BLOCK_SAMPLES;

				send_dest	= tx_buf;
			}

			send_bytes	= send_values * sizeof(tx_buf[0]);
			sent_bytes	= write(fd, send_dest, send_bytes);
			hpdi32_reg_write(fd, HPDI32_GSC_BCR, bcr, 0);
			usleep(20000);

			if (sent_bytes < 0)
			{
				printf(	"write(fd, dest, %ld) => %ld ",
					send_bytes,
					sent_bytes);
				status	= -1;
				break;
			}
			else if (sent_bytes)
			{
				timeout	= time(NULL) + XFER_INACTIVITY_SEC;
			}

			sent_values	= sent_bytes / sizeof(tx_buf[0]);
			send_values	-= sent_values;
			sent_total	+= sent_values;
			send_dest	+= sent_values;
		}

		if (got_total < rx_values[0])
		{
			if (get_values <= 0)
			{
				get_values	= rx_values[0] - got_total;

				if (get_values > XFER_BLOCK_SAMPLES)
					get_values	= XFER_BLOCK_SAMPLES;

				get_dest	= rx_buf;
				memset(rx_buf, 0, sizeof(rx_buf));
				memcmp_values	= get_values;
				memcmp_bytes	= get_values * 4;
			}

			get_bytes	= get_values * sizeof(rx_buf[0]);
			got_bytes	= read(fd, get_dest, get_bytes);

			if ((got_bytes < 0) && (tx_values[0] >= rx_values[0]))
			{
				printf(	"read(fd, dest, %lu) => %ld ",
					get_bytes,
					got_bytes);
				status	= -1;
				break;
			}
			else if (got_bytes < 0)
			{
				/*
				*	This is expected for Demand Mode DMA.
				*	We'll assume that we timed out without
				*	getting any data.
				*/

				got_bytes	= 0;
			}
			else if (got_bytes)
			{
				timeout	= time(NULL) + XFER_INACTIVITY_SEC;
			}

			got_values	= got_bytes / sizeof(rx_buf[0]);
			get_values	-= got_values;
			got_total	+= got_values;
			get_dest	+= got_values;

			if (get_values <= 0)
			{
				/* Locate and report any failure. */

				if (memcmp(tx_buf, rx_buf, memcmp_bytes))
				{
					for (i = 0; i < memcmp_values; i++)
					{
						if (tx_buf[i] != rx_buf[i])
							break;
					}

					printf("(%4d. %4lu 0x%08lX) ",
						i,
						(unsigned long) rx_buf[i],
						(unsigned long) rx_buf[i]);

					printf(	"(at %ld, w[%ld], r[%ld]) ",
						got_total - (memcmp_values - i),
						sent_total,
						got_total);
					status	= -1;
					break;
				}
			}
		}

		if (time(NULL) > timeout)
		{
			if (inactivity)
			{
				status	= 0;
			}
			else
			{
				status	= -1;
				printf(	"(inactivity: "
					"%d seconds, w[%ld], r[%ld]) ",
					XFER_INACTIVITY_SEC,
					sent_total,
					got_total);
			}

			inactivity	= 0;
			break;
		}
	}

	if ((status == 0) && (inactivity))
	{
		printf("(inactivity expected) ");
		status	= -1;
	}

	if ((tx_values[0]) && (sent_total <= 0))
	{
		printf("(write failure) ");
		status	= -1;
	}

	if ((tx_values[0]) && (rx_values[0]) && (got_total <= 0))
	{
		printf("(read failure) ");
		status	= -1;
	}

	tx_values[0]	= sent_total;
	rx_values[0]	= got_total;
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0, 0);
	return(status);
}



/*****************************************************************************
*
*	Function:	io_read_write_tests
*
*	Purpose:
*
*		Perform the appropriate testing.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		name		The name of the I/O mode.
*
*		mode		The I/O mode to use.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

int io_read_write_tests(int fd, const char* name, int mode)
{
	static const struct
	{
		__u32	tx;
		__u32	rx;
		int	inactivity;
	} tests[]	=
	{
		/* tx		rx	inact	*/
		{ 0,		0,	FALSE	},
		{ 1,		0,	FALSE	},
		{ 10,		0,	FALSE	},
		{ 100,		0,	FALSE	},
		{ _1K,		0,	FALSE	},
		{ _100K,	0,	FALSE	},
		{ 1,		1,	FALSE	},
		{ 10,		10,	FALSE	},
		{ 100,		100,	FALSE	},
		{ _1K,		_1K,	FALSE	},
		{ _100K,	_100K,	FALSE	}
	};

	__u32	fsr;
	__u32	frr;
	int	inactivity;
	char	message[128];
	int	status	= 0;
	int	t;
	__u32	tfsr;

	hpdi32_reg_read(fd, HPDI32_GSC_FRR, &frr, 0);

	if (frr & HPDI32_GSC_FRR_FSR)
		hpdi32_reg_read(fd, HPDI32_GSC_FSR, &fsr, 0);
	else
		fsr	= 0;

	if (fsr & HPDI32_GSC_FSR_TFSR_RFSR)
		hpdi32_reg_read(fd, HPDI32_GSC_TFSR, &tfsr, 0);
	else
		tfsr	= 0;

	sprintf(message, "%s%s", name, " (basic)");
	label(message);
	printf("\n");
	label_level_inc();

	for (t = 0; t < ARRAY_ELEMENTS(tests); t++)
	{
		if (tests[t].tx == tests[t].rx)
			inactivity	= FALSE;
		else if (tests[t].tx < tests[t].rx)
			inactivity	= TRUE;
		else if (tfsr == 0)
			inactivity	= tests[t].inactivity;
		else
			inactivity	= FALSE;

		status	|= _write_read_mode_tests(
				fd,
				mode,
				tests[t].tx,
				tests[t].rx,
				inactivity,
				_write_read_test_basic);
	}

	label_level_dec();
	status	= status ? -1 : 0;
	return(status);
}



/*****************************************************************************
*
*	Function:	io_throughput_test_dma
*
*	Purpose:
*
*		Perform a throughput test for DMA mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		seconds		The length of time to run the test.
*
*		mem_map		Use the mmap() service?
*
*	Returned:
*
*		>= 0	The rate observed.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

long io_throughput_test_dma(int fd, int seconds, int mem_map)
{
	unsigned long		almost;
	long			begin;
	unsigned long*		bsr;
	__u32*			data;
	long			bytes;
	long			end;
	unsigned long		flags;
	__u32			frr;
	__u32			fsr;
	__u8*			gsc;
	__u32			junk[HPDI32_IO_SAMPLES_MAX];
	hpdi32_mmap_t		map;
	long			ms;
	__u32			reg;
	long			sec_ms	= 1000L * seconds;
	__u32			send;
	int			sent;
	long			status	= 0;
	__u32			tar;
	__u32			tfsr;
	struct timeb		tim;
	__u32			tx_auto_stop;
	__u8*			tx_buf	= NULL;

	/* Reset the board to a known state. */
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);

	// Get some info from the board.
	hpdi32_reg_read(fd, HPDI32_GSC_FRR, &frr, 0);
	hpdi32_reg_read(fd, HPDI32_GSC_FSR, &fsr, 0);
	hpdi32_reg_read(fd, HPDI32_GSC_TFSR, &tfsr, 0);

	if (!(frr & HPDI32_GSC_FRR_FSR) && !(fsr & HPDI32_GSC_FSR_TFSR_RFSR))
	{
		tfsr	= 0;
		printf("(FIFO size not known) ");
	}

	if ((frr & HPDI32_GSC_FRR_FSR) && (fsr & HPDI32_GSC_FSR_GPIO6_TXAS))
		tx_auto_stop	= HPDI32_GSC_BCR_TX_AUTO_STOP_NO;
	else
		tx_auto_stop	= 0;

	/* Set the Almost Empty level. */
	almost	= (long) tfsr * 0.5;
	almost	= (almost <= 0) ? 1 : (almost > 0xFFFF) ? 0xFFFF : almost;
	tar	= HPDI32_GSC_TAR_ENCODE(1, almost);
	hpdi32_reg_write(fd, HPDI32_GSC_TAR, tar, 0);
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_TX_FIFO_RESET, 0);

	/* See how much data we send at a time. */
	send	= (tfsr - almost) * 4;
	send	= (send <= 0) ? 1 : send;
	send	= (send > sizeof(junk)) ? sizeof(junk) : send;

	/* Gain fast access to the GSC register block. */
	hpdi32_mmap_info(fd, &map, 0);

	if (map.regs.size)
	{
		gsc	= (__u8*) mmap(	NULL,
					map.regs.size,
					PROT_READ | PROT_WRITE,
					MAP_SHARED,
					fd,
					map.regs.offset);
		bsr	= (unsigned long*)
			(gsc + HPDI32_REG_OFFSET(HPDI32_GSC_BSR));

		if (gsc == NULL)
		{
			printf("(mmap() returned NULL) ");
			status	= -1;
		}
	}
	else
	{
		gsc	= NULL;
		bsr	= NULL;
	}

	/* Get the data pointers. */

	if (mem_map)
	{
		data	= NULL;
		tx_buf	= (__u8*) mmap(	NULL,
					map.write.size,
					PROT_READ | PROT_WRITE,
					MAP_SHARED,
					fd,
					map.write.offset);

		if (tx_buf == NULL)
		{
			printf("(mmap() returned NULL) ");
			status	= -1;
		}
	}
	else
	{
		data	= junk;
	}

	/*
	*	Fill the FIFO with data before we begin. We do this so that
	*	the transfer rate isn't artifically high due to the presence
	*	of the Tx FIFO.
	*/

	for (bytes = 0; bytes < (128L * 1024); bytes += send)
		hpdi32_reg_write(fd, HPDI32_GSC_FDR, 0, 0);

	/* Begin. */
	flags	= HPDI32_GSC_BCR_TX_ENABLE | tx_auto_stop;
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, flags, 0);
	flags	|= HPDI32_GSC_BCR_TX_START;
	ftime(&tim);
	begin	= (1000L * tim.time) + tim.millitm;

	if ((status == 0) && (tfsr))
	{
		hpdi32_reg_write(fd, HPDI32_GSC_BCR, flags, 0);

		for (bytes = 0;;)
		{
			if (bsr)
				reg	= bsr[0];
			else
				hpdi32_reg_read(fd, HPDI32_GSC_BSR, &reg, 0);

			if (reg & HPDI32_GSC_BSR_TX_NOT_AE)
				continue;

			sent	= write(fd, data, send);
			hpdi32_reg_write(fd, HPDI32_GSC_BCR, flags, 0);

			if (sent < 0)
			{
				status	= sent;
				break;
			}

			bytes	+= sent;
			ftime(&tim);
			end	= (1000L * tim.time) + tim.millitm;
			ms	= end - begin;

			if (ms >= sec_ms)
			{
				status	= (long) (1000.0 * bytes / ms);
				break;
			}
		}
	}

	if (bsr)
		reg	= bsr[0];
	else
		hpdi32_reg_read(fd, HPDI32_GSC_BSR, &reg, 0);

	if (reg & HPDI32_GSC_BSR_TX_OVERRUN)
	{
		status	= -1;
		printf("(Tx Overrun) ");
	}

	if (gsc)
		munmap(gsc, map.regs.size);

	if (tx_buf)
		munmap(tx_buf, map.write.size);

	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	return(status);
}



/*****************************************************************************
*
*	Function:	io_throughput_test_dmdma
*
*	Purpose:
*
*		Perform a throughput test for Demand Mode DMA mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		seconds		The length of time to run the test.
*
*		mem_map		Use the mmap() service?
*
*	Returned:
*
*		>= 0	The rate observed.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

long io_throughput_test_dmdma(int fd, int seconds, int mem_map)
{
	long			begin;
	long			bytes;
	__u32*			data;
	long			end;
	unsigned long		flags;
	__u32			frr;
	__u32			fsr;
	__u32			junk[1024L * 1024];
	hpdi32_mmap_t		map;
	long			ms;
	__u32			scd;	// Single Cycle Disable
	long			sec_ms	= 1000L * seconds;
	__u32			send;
	int			sent;
	long			status	= 0;
	struct timeb		tim;
	__u32			tx_auto_stop;
	__u8*			tx_buf	= NULL;
	__u32			tfsr;

	/* Reset the board to a known state. */
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);

	// Get some info from the board.
	hpdi32_reg_read(fd, HPDI32_GSC_FRR, &frr, 0);
	hpdi32_reg_read(fd, HPDI32_GSC_FSR, &fsr, 0);
	hpdi32_reg_read(fd, HPDI32_GSC_TFSR, &tfsr, 0);

	if ((frr & HPDI32_GSC_FRR_FSR) && (fsr & HPDI32_GSC_FSR_TFSR_RFSR))
	{
		send	= (long) ((((long) ((tfsr * 4) * 0.2)) / 4) * 4);
	}
	else
	{
		send	= 0;
		printf("(FIFO size not known) ");
	}

	if ((frr & HPDI32_GSC_FRR_FSR) && (fsr & HPDI32_GSC_FSR_SCD))
		scd	= HPDI32_GSC_BCR_SING_CYC_DIS;
	else
		scd	= 0;

	if ((frr & HPDI32_GSC_FRR_FSR) && (fsr & HPDI32_GSC_FSR_GPIO6_TXAS))
		tx_auto_stop	= HPDI32_GSC_BCR_TX_AUTO_STOP_NO;
	else
		tx_auto_stop	= 0;

	/* Get the data pointers. */

	if (mem_map)
	{
		data	= NULL;
		hpdi32_mmap_info(fd, &map, 0);
		tx_buf	= (__u8*) mmap(	NULL,
					map.write.size,
					PROT_READ | PROT_WRITE,
					MAP_SHARED,
					fd,
					map.write.offset);

		if (tx_buf == NULL)
		{
			printf("(mmap() returned NULL) ");
			status	= -1;
		}
	}
	else
	{
		data	= junk;
	}

	/*
	*	Fill the FIFO with data before we begin. We do this so that
	*	the transfer rate isn't artifically high due to the presence
	*	of the Tx FIFO.
	*/

	for (bytes = 0; bytes < (128L * 1024); bytes += send)
		hpdi32_reg_write(fd, HPDI32_GSC_FDR, 0, 0);

	flags	= HPDI32_GSC_BCR_TX_ENABLE | tx_auto_stop | scd;
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, flags, 0);

	/* Begin. */
	flags	|= HPDI32_GSC_BCR_TX_START;
	ftime(&tim);
	begin	= (1000L * tim.time) + tim.millitm;

	if ((status == 0) && (send))
	{
		hpdi32_reg_write(fd, HPDI32_GSC_BCR, flags, 0);

		for (bytes = 0;;)
		{
			hpdi32_reg_write(fd, HPDI32_GSC_BCR, flags, 0);
			sent	= write(fd, data, send);
			hpdi32_reg_write(fd, HPDI32_GSC_BCR, flags, 0);

			if (sent < 0)
			{
				status	= sent;
				break;
			}

			bytes	+= sent;
			ftime(&tim);
			end	= (1000L * tim.time) + tim.millitm;
			ms	= end - begin;

			if (ms >= sec_ms)
			{
				status	= (long) (1000.0 * bytes / ms);
				break;
			}
		}
	}

	if (tx_buf)
		munmap(tx_buf, map.write.size);

	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	return(status);
}



/*****************************************************************************
*
*	Function:	io_throughput_test_pio
*
*	Purpose:
*
*		Perform a throughput test for PIO mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		seconds		The length of time to run the test.
*
*		mem_map		Use the mmap() service?
*
*	Returned:
*
*		>= 0	The rate observed.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

long io_throughput_test_pio(int fd, int seconds, int mem_map)
{
	long			begin;
	__u32			bsr;
	__u32*			data;
	__u32			junk[HPDI32_IO_SAMPLES_MAX];
	long			bytes;
	long			end;
	unsigned long		flags;
	__u32			frr;
	__u32			fsr;
	__u32*			init;
	hpdi32_io_config_t	io;
	hpdi32_mmap_t		map;
	long			ms;
	long			sec_ms	= 1000L * seconds;
	__u32			send;
	int			sent;
	long			status	= 0;
	struct timeb		tim;
	__u32			tx_auto_stop;

	/* Determine the I/O transfer size that is in use. */
	io.read.mode		= HPDI32_IO_MODE_NO_CHANGE;
	io.read.timeout		= HPDI32_IO_TIMEOUT_NO_CHANGE;
	io.read.samples		= HPDI32_IO_SAMPLES_NO_CHANGE;
	io.read.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.read.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
	io.write.mode		= HPDI32_IO_MODE_NO_CHANGE;
	io.write.timeout	= 0;
	io.write.samples	= HPDI32_IO_SAMPLES_NO_CHANGE;
	io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.write.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
	hpdi32_io_config(fd, &io, 0);
	send	= io.write.samples * 4;
	send	= (send > sizeof(junk)) ? sizeof(junk) : send;
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	hpdi32_reg_read(fd, HPDI32_GSC_FRR, &frr, 0);
	hpdi32_reg_read(fd, HPDI32_GSC_FSR, &fsr, 0);

	if ((frr & HPDI32_GSC_FRR_FSR) && (fsr & HPDI32_GSC_FSR_GPIO6_TXAS))
		tx_auto_stop	= HPDI32_GSC_BCR_TX_AUTO_STOP_NO;
	else
		tx_auto_stop	= 0;

	/* Get the data pointers. */

	if (mem_map)
	{
		data	= NULL;
		hpdi32_mmap_info(fd, &map, 0);
		init	= (__u32*) mmap(NULL,
					map.write.size,
					PROT_READ | PROT_WRITE,
					MAP_SHARED,
					fd,
					map.write.offset);

		if (init == NULL)
		{
			printf("(mmap() returned NULL) ");
			status	= -1;
		}
	}
	else
	{
		data	= junk;
		init	= junk;
	}

	/*
	*	Fill the FIFO with data before we begin. We do this so that
	*	the transfer rate isn't artifically high due to the FIFO
	*	initially being empty.
	*/

	for (bytes = 0; bytes <= (128L * 1024); bytes++)
	{
		hpdi32_reg_write(fd, HPDI32_GSC_FDR, 0, 0);
		hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

		if ((bsr & HPDI32_GSC_BSR_TX_NOT_FULL) == 0)
			break;
	}

	/* Begin. */
	flags	= HPDI32_GSC_BCR_TX_ENABLE | tx_auto_stop;
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, flags, 0);
	flags	|= HPDI32_GSC_BCR_TX_START;
	ftime(&tim);
	begin	= (1000L * tim.time) + tim.millitm;

	if (status == 0)
	{
		for (bytes = 0;;)
		{
			sent	= write(fd, data, send);
			hpdi32_reg_write(fd, HPDI32_GSC_BCR, flags, 0);

			if (sent < 0)
			{
				status	= sent;
				break;
			}

			bytes	+= sent;
			ftime(&tim);
			end	= (1000L * tim.time) + tim.millitm;
			ms	= end - begin;

			if (ms >= sec_ms)
			{
				status	= (long) (1000.0 * bytes / ms);
				break;
			}
		}
	}

	if ((mem_map) && (init))
		munmap(init, map.write.size);

	hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

	if (bsr & HPDI32_GSC_BSR_TX_OVERRUN)
		printf("(Tx Overrun) ");

	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	return(status);
}



/*****************************************************************************
*
*	Function:	io_throughput_tests
*
*	Purpose:
*
*		Perform a series of throughput tests.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		mode		The I/O mode to test.
*
*		text		The text to appear after the test title.
*
*		function	The function to call that implements the test.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

int io_throughput_tests(int		fd,
			int		mode,
			const char*	text,
			long		(*function)(int, int, int))
{
	static const struct
	{
		unsigned long	limit;
		const char*	format;
		long		div;
	} rates[]	=
	{
		/* limit	format		div	*/
		{     1000,	"%.0f B/S",	1	},
		{    99900,	"%.1f KB/S",	1000	},
		{  1000000,	"%.0f KB/S",	1000	},
		{ 99900000,	"%.1f MB/S",	1000000	},
		{       -1,	"%.0f MB/S",	1000000	}
	};

	static const	struct
	{
		const char*	name;
		int		mem_map;
	} args[]	=
	{
		/* name			mem_map	*/
		{ "Standard",		FALSE	},
		{ "mmap()",		TRUE	}
	};

	__u32			bsr;
	int			i;
	hpdi32_io_config_t	io;
	char			message[128];
	long			rate;
	int			seconds	= 5;
	int			status	= 0;
	int			type;

	io.read.mode		= mode;
	io.read.timeout		= 0;
	io.read.samples		= HPDI32_IO_SAMPLES_MIN;
	io.read.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.read.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
	io.write.mode		= mode;
	io.write.timeout	= 5;
	io.write.samples	= HPDI32_IO_SAMPLES_MIN;
	io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.write.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
	status			= hpdi32_io_config(fd, &io, 0);
	io.write.samples	= HPDI32_IO_SAMPLES_MAX;
	hpdi32_io_config(fd, &io, 0);
	io.write.samples	= HPDI32_IO_SAMPLES_NO_CHANGE;
	hpdi32_io_config(fd, &io, 0);

	sprintf(message, "Throughput (%s)", text);
	label(message);
	printf(	"(1K = 1000, write buffer = %ld samples)\n",
		(long) io.write.samples);
	label_level_inc();

	for (type = 0; type < ARRAY_ELEMENTS(args); type++)
	{
		sprintf(message, "%s (%d seconds)", args[type].name, seconds);
		label(message);
		rate	= (function)(fd, seconds, args[type].mem_map);

		if (rate < 0)
		{
			status	= -1;
			printf("FAIL\n");
			continue;
		}

		for (i = 0; i < ARRAY_ELEMENTS(rates); i++)
		{
			if (	(rate < rates[i].limit) ||
				(rates[i].limit == -1))
			{
				printf(	rates[i].format,
					(double) rate / rates[i].div);
				break;
			}
		}

		hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

		if (bsr & HPDI32_GSC_BSR_TX_OVERRUN)
		{
			status	= -1;
			printf("(Tx Overrun)\n");
			continue;
		}

		printf("\n");
	}

	label_level_dec();
	io.read.samples		= HPDI32_IO_SAMPLES_MIN;
	io.write.samples	= HPDI32_IO_SAMPLES_MIN;
	hpdi32_io_config(fd, &io, 0);
	io.read.samples		= HPDI32_IO_SAMPLES_DEFAULT;
	io.write.samples	= HPDI32_IO_SAMPLES_MIN;
	hpdi32_io_config(fd, &io, 0);
	return(status);
}



static int _data_size_get_fifo_size(int fd, long* fifo)
{
	__u32	frr;
	__u32	fsr;
	__u32	rfsr;
	int		status	= 0;
	__u32	tfsr;

	fifo[0]	= 0;

	status	|= hpdi32_reg_read(fd, HPDI32_GSC_FRR, &frr, 0);

	if (frr & HPDI32_GSC_FRR_FSR)
		hpdi32_reg_read(fd, HPDI32_GSC_FSR, &fsr, 0);
	else
		fsr	= 0;

	if (fsr & HPDI32_GSC_FSR_TFSR_RFSR)
	{
		hpdi32_reg_read(fd, HPDI32_GSC_TFSR, &tfsr, 0);
		hpdi32_reg_read(fd, HPDI32_GSC_RFSR, &rfsr, 0);
	}
	else
	{
		tfsr	= 0;
		rfsr	= 0;
	}

	if (tfsr > rfsr)
		fifo[0]	= (long) rfsr;
	else
		fifo[0]	= (long) tfsr;

	return(status);
}



/*****************************************************************************
*
*	Function:	_tx_data_test_8_io
*
*	Purpose:
*
*		Verify I/O writes with 8-bit data and the given I/O mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		mode	The I/O mode to use.
*
*		size	The number of samples to transmit and receive.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

static int _tx_data_test_8_io(int fd, __u32 mode, long size)
{
	__u32				bsr;
	long				bytes	= size;
	__u32				fdr;
	hpdi32_io_config_t	io;
	long				l;
	const char*			msg		= NULL;
	__u8*				rx		= (__u8*) _big_rx_buffer;
	__u8*				tx		= (__u8*) _big_tx_buffer;
	int					status	= 0;

	// Initialize the buffer.

	for (l = 0; l < size; l++)
		tx[l]	= (__u8) ((unsigned long) (l & 0xFF));

	// Initialize the board.
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	io.read.mode			= HPDI32_IO_MODE_NO_CHANGE;
	io.read.timeout			= HPDI32_IO_TIMEOUT_NO_CHANGE;
	io.read.samples			= HPDI32_IO_SAMPLES_MIN;
	io.read.sample_bits		= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.read.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
	io.write.mode			= mode;
	io.write.timeout		= HPDI32_IO_TIMEOUT_NO_CHANGE;
	io.write.samples		= size;
	io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_8;
	io.write.pio_threshold	= 0;
	status	= hpdi32_io_config(fd, &io, 0);
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x36, 0);// Tx + Rx FIFO Reset

	// Write the data to the board.
	l	= write(fd, tx, bytes);

	if (l != bytes)
	{
		if (status == 0)
			msg	= "write() failure";

		status	|= -1;
	}

	// Transmit the data and wait for the Tx FIFO to empty.
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x030, 0);// Tx Enable + Rx Enable
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x130, 0);// Tx Start

	for (l = 0; l < 15; l++)
	{
		usleep(100000);
		hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

		if ((bsr & 0x100) == 0)	// The Tx FIFO is now empty;
			break;
	}

	hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

	if ((status == 0) && (bsr & 0x100))
	{
		status	|= -1;
		msg		= "Tx FIFO didn't empty";
	}

	// Retrieve the data and verify that it is correct.

	if (status == 0)
	{
		for (l = 0; l < size; l++)
		{
			hpdi32_reg_read(fd, HPDI32_GSC_FDR, &fdr, 0);
			rx[l]	= (__u8) (fdr & 0xFF);
		}

		if (memcmp(tx, rx, bytes))
		{
			status	|= -1;
			msg		= "data validation failure";
		}
	}

	if (status == 0)
		printf("PASS   (%ld samples)\n", size);
	else if (msg)
		printf("FAIL <---  (%s)\n", msg);
	else
		printf("FAIL\n");

	return(status);
}



/*****************************************************************************
*
*	Function:	_tx_data_test_8
*
*	Purpose:
*
*		Verify I/O writes with 8-bit data and the given I/O mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

static int _tx_data_test_8(int fd, __u32 mode)
{
	char	buf[256];
	char*	psz;
	long	size;
	int		status	= 0;

	switch (mode)
	{
		default:	status	= -1;	psz	= "ERROR";	break;
		case HPDI32_IO_MODE_PIO:	psz	= "PIO";	break;
		case HPDI32_IO_MODE_DMA:	psz	= "DMA";	break;
		case HPDI32_IO_MODE_DMDMA:	psz	= "DMDMA";	break;
	}

	sprintf(buf, "Tx  8-bit %s", psz);
	label(buf);

	status	|= _data_size_get_fifo_size(fd, &size);

	if (status)
		printf("FAIL <---  (setup)\n");
	else if (size == 0)
		printf("SKIPPED   (FIFO size not known)\n");
	else
		status	|= _tx_data_test_8_io(fd, mode, size);

	return(status);
}



/*****************************************************************************
*
*	Function:	_tx_data_test_16_io
*
*	Purpose:
*
*		Verify I/O writes with 16-bit data and the given I/O mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		mode	The I/O mode to use.
*
*		size	The number of samples to transmit and receive.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

static int _tx_data_test_16_io(int fd, __u32 mode, long size)
{
	__u32				bsr;
	long				bytes	= size * 2;
	__u32				fdr;
	hpdi32_io_config_t	io;
	long				l;
	const char*			msg		= NULL;
	__u16*				rx		= (__u16*) _big_rx_buffer;
	__u16*				tx		= (__u16*) _big_tx_buffer;
	int					status	= 0;

	// Initialize the buffer.

	for (l = 0; l < size; l++)
		tx[l]	= (__u16) ((unsigned long) (l & 0xFFFF));

	// Initialize the board.
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	io.read.mode			= HPDI32_IO_MODE_NO_CHANGE;
	io.read.timeout			= HPDI32_IO_TIMEOUT_NO_CHANGE;
	io.read.samples			= HPDI32_IO_SAMPLES_MIN;
	io.read.sample_bits		= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.read.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
	io.write.mode			= mode;
	io.write.timeout		= HPDI32_IO_TIMEOUT_NO_CHANGE;
	io.write.samples		= size;
	io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_16;
	io.write.pio_threshold	= 0;
	status	= hpdi32_io_config(fd, &io, 0);
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x36, 0);// Tx + Rx FIFO Reset

	// Write the data to the board.
	l	= write(fd, tx, bytes);

	if (l != bytes)
	{
		if (status == 0)
			msg	= "write() failure";

		status	|= -1;
	}

	// Transmit the data and wait for the Tx FIFO to empty.
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x030, 0);// Tx Enable + Rx Enable
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x130, 0);// Tx Start

	for (l = 0; l < 15; l++)
	{
		usleep(100000);
		hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

		if ((bsr & 0x100) == 0)	// The Tx FIFO is now empty;
			break;
	}

	hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

	if ((status == 0) && (bsr & 0x100))
	{
		status	|= -1;
		msg		= "Tx FIFO didn't empty";
	}

	// Retrieve the data and verify that it is correct.

	if (status == 0)
	{
		for (l = 0; l < size; l++)
		{
			hpdi32_reg_read(fd, HPDI32_GSC_FDR, &fdr, 0);
			rx[l]	= (__u16) (fdr & 0xFFFF);
		}

		if (memcmp(tx, rx, bytes))
		{
			status	|= -1;
			msg		= "data validation failure";
		}
	}

	if (status == 0)
		printf("PASS   (%ld samples)\n", size);
	else if (msg)
		printf("FAIL <---  (%s)\n", msg);
	else
		printf("FAIL\n");

	return(status);
}



/*****************************************************************************
*
*	Function:	_tx_data_test_16
*
*	Purpose:
*
*		Verify I/O writes with 16-bit data and the given I/O mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

static int _tx_data_test_16(int fd, __u32 mode)
{
	char	buf[256];
	char*	psz;
	long	size;
	int		status	= 0;

	switch (mode)
	{
		default:	status	= -1;	psz	= "ERROR";	break;
		case HPDI32_IO_MODE_PIO:	psz	= "PIO";	break;
		case HPDI32_IO_MODE_DMA:	psz	= "DMA";	break;
		case HPDI32_IO_MODE_DMDMA:	psz	= "DMDMA";	break;
	}

	sprintf(buf, "Tx 16-bit %s", psz);
	label(buf);

	status	|= _data_size_get_fifo_size(fd, &size);

	if (status)
		printf("FAIL <---  (setup)\n");
	else if (size == 0)
		printf("SKIPPED   (FIFO size not known)\n");
	else
		status	|= _tx_data_test_16_io(fd, mode, size);

	return(status);
}



/*****************************************************************************
*
*	Function:	_tx_data_test_32_io
*
*	Purpose:
*
*		Verify I/O writes with 32-bit data and the given I/O mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		mode	The I/O mode to use.
*
*		size	The number of samples to transmit and receive.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

static int _tx_data_test_32_io(int fd, __u32 mode, long size)
{
	long				bytes	= size * 4;
	__u32				bsr;
	__u32				fdr;
	hpdi32_io_config_t	io;
	long				l;
	const char*			msg		= NULL;
	__u32*				rx		= (__u32*) _big_rx_buffer;
	__u32*				tx		= (__u32*) _big_tx_buffer;
	int					status	= 0;

	// Initialize the buffer.

	for (l = 0; l < size; l++)
		tx[l]	= (__u32) ((unsigned long) l);

	// Initialize the board.
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	io.read.mode			= HPDI32_IO_MODE_NO_CHANGE;
	io.read.timeout			= HPDI32_IO_TIMEOUT_NO_CHANGE;
	io.read.samples			= HPDI32_IO_SAMPLES_MIN;
	io.read.sample_bits		= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.read.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
	io.write.mode			= mode;
	io.write.timeout		= HPDI32_IO_TIMEOUT_NO_CHANGE;
	io.write.samples		= size;
	io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_32;
	io.write.pio_threshold	= 0;
	status	= hpdi32_io_config(fd, &io, 0);
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x36, 0);// Tx + Rx FIFO Reset

	// Write the data to the board.
	l	= write(fd, tx, bytes);

	if (l != bytes)
	{
		if (status == 0)
			msg	= "write() failure";

		status	|= -1;
	}

	// Transmit the data and wait for the Tx FIFO to empty.
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x030, 0);// Tx Enable + Rx Enable
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x130, 0);// Tx Start

	for (l = 0; l < 15; l++)
	{
		usleep(100000);
		hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

		if ((bsr & 0x100) == 0)	// The Tx FIFO is now empty;
			break;
	}

	hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

	if ((status == 0) && (bsr & 0x100))
	{
		status	|= -1;
		msg		= "Tx FIFO didn't empty";
	}

	// Retrieve the data and verify that it is correct.

	if (status == 0)
	{
		for (l = 0; l < size; l++)
		{
			hpdi32_reg_read(fd, HPDI32_GSC_FDR, &fdr, 0);
			rx[l]	= fdr;
		}

		if (memcmp(tx, rx, bytes))
		{
			status	|= -1;
			msg		= "data validation failure";
		}
	}

	if (status == 0)
		printf("PASS   (%ld samples)\n", size);
	else if (msg)
		printf("FAIL <---  (%s)\n", msg);
	else
		printf("FAIL\n");

	return(status);
}



/*****************************************************************************
*
*	Function:	_tx_data_test_32
*
*	Purpose:
*
*		Verify I/O writes with 32-bit data and the given I/O mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

static int _tx_data_test_32(int fd, __u32 mode)
{
	char	buf[256];
	char*	psz;
	long	size;
	int		status	= 0;

	switch (mode)
	{
		default:	status	= -1;	psz	= "ERROR";	break;
		case HPDI32_IO_MODE_PIO:	psz	= "PIO";	break;
		case HPDI32_IO_MODE_DMA:	psz	= "DMA";	break;
		case HPDI32_IO_MODE_DMDMA:	psz	= "DMDMA";	break;
	}

	sprintf(buf, "Tx 32-bit %s", psz);
	label(buf);

	status	|= _data_size_get_fifo_size(fd, &size);

	if (status)
		printf("FAIL <---  (setup)\n");
	else if (size == 0)
		printf("SKIPPED   (FIFO size not known)\n");
	else
		status	|= _tx_data_test_32_io(fd, mode, size);

	return(status);
}



/*****************************************************************************
*
*	Function:	_rx_data_test_8_io
*
*	Purpose:
*
*		Verify I/O read with 8-bit data and the given I/O mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		mode	The I/O mode to use.
*
*		size	The number of samples to transmit and receive.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

static int _rx_data_test_8_io(int fd, __u32 mode, long size)
{
	__u32				bsr;
	long				bytes	= size;
	hpdi32_io_config_t	io;
	long				l;
	const char*			msg		= NULL;
	__u8*				rx		= (__u8*) _big_rx_buffer;
	__u8*				tx		= (__u8*) _big_tx_buffer;
	int					status	= 0;

	// Initialize the buffer.

	for (l = 0; l < size; l++)
		tx[l]	= (__u8) ((unsigned long) (l & 0xFF));

	// Initialize the board.
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	io.read.mode			= mode;
	io.read.timeout			= HPDI32_IO_TIMEOUT_NO_CHANGE;
	io.read.samples			= size;
	io.read.sample_bits		= HPDI32_IO_SAMPLE_BITS_8;
	io.read.pio_threshold	= 0;
	io.write.mode			= HPDI32_IO_MODE_NO_CHANGE;
	io.write.timeout		= HPDI32_IO_TIMEOUT_NO_CHANGE;
	io.write.samples		= HPDI32_IO_SAMPLES_MIN;
	io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.write.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
	status	= hpdi32_io_config(fd, &io, 0);
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x36, 0);// Tx + Rx FIFO Reset

	// Write the data to the board.

	for (l = 0; l < size; l++)
		hpdi32_reg_write(fd, HPDI32_GSC_FDR, tx[l], 0);

	// Transmit the data and wait for the Tx FIFO to empty.
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x030, 0);// Tx Enable + Rx Enable
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x130, 0);// Tx Start

	for (l = 0; l < 15; l++)
	{
		usleep(100000);
		hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

		if ((bsr & 0x100) == 0)	// The Tx FIFO is now empty;
			break;
	}

	hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

	if ((status == 0) && (bsr & 0x100))
	{
		status	|= -1;
		msg		= "Tx FIFO didn't empty";
	}

	// Retrieve the data.

	if (status == 0)
	{
		l	= read(fd, rx, bytes);

		if (l != bytes)
		{
			if (status == 0)
				msg	= "write() failure";

			status	|= -1;
		}
	}

	// Verify that the data is correct.

	if ((status == 0) && (memcmp(tx, rx, bytes)))
	{
		status	|= -1;
		msg		= "data validation failure";
	}

	if (status == 0)
		printf("PASS   (%ld samples)\n", size);
	else if (msg)
		printf("FAIL <---  (%s)\n", msg);
	else
		printf("FAIL\n");

	return(status);
}



/*****************************************************************************
*
*	Function:	_rx_data_test_8
*
*	Purpose:
*
*		Verify I/O read with 8-bit data and the given I/O mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

static int _rx_data_test_8(int fd, __u32 mode)
{
	char	buf[256];
	char*	psz;
	long	size;
	int		status	= 0;

	switch (mode)
	{
		default:	status	= -1;	psz	= "ERROR";	break;
		case HPDI32_IO_MODE_PIO:	psz	= "PIO";	break;
		case HPDI32_IO_MODE_DMA:	psz	= "DMA";	break;
		case HPDI32_IO_MODE_DMDMA:	psz	= "DMDMA";	break;
	}

	sprintf(buf, "Rx  8-bit %s", psz);
	label(buf);

	status	|= _data_size_get_fifo_size(fd, &size);

	if (status)
		printf("FAIL <---  (setup)\n");
	else if (size == 0)
		printf("SKIPPED   (FIFO size not known)\n");
	else
		status	|= _rx_data_test_8_io(fd, mode, size);

	return(status);
}



/*****************************************************************************
*
*	Function:	_rx_data_test_16_io
*
*	Purpose:
*
*		Verify I/O read with 16-bit data and the given I/O mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		mode	The I/O mode to use.
*
*		size	The number of samples to transmit and receive.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

static int _rx_data_test_16_io(int fd, __u32 mode, long size)
{
	__u32				bsr;
	long				bytes	= size * 2;
	hpdi32_io_config_t	io;
	long				l;
	const char*			msg		= NULL;
	__u16*				rx		= (__u16*) _big_rx_buffer;
	__u16*				tx		= (__u16*) _big_tx_buffer;
	int					status	= 0;

	// Initialize the buffer.

	for (l = 0; l < size; l++)
		tx[l]	= (__u16) ((unsigned long) (l & 0xFFFF));

	// Initialize the board.
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	io.read.mode			= mode;
	io.read.timeout			= HPDI32_IO_TIMEOUT_NO_CHANGE;
	io.read.samples			= size;
	io.read.sample_bits		= HPDI32_IO_SAMPLE_BITS_16;
	io.read.pio_threshold	= 0;
	io.write.mode			= HPDI32_IO_MODE_NO_CHANGE;
	io.write.timeout		= HPDI32_IO_TIMEOUT_NO_CHANGE;
	io.write.samples		= HPDI32_IO_SAMPLES_MIN;
	io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.write.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
	status	= hpdi32_io_config(fd, &io, 0);
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x36, 0);// Tx + Rx FIFO Reset

	// Write the data to the board.

	for (l = 0; l < size; l++)
		hpdi32_reg_write(fd, HPDI32_GSC_FDR, tx[l], 0);

	// Transmit the data and wait for the Tx FIFO to empty.
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x030, 0);// Tx Enable + Rx Enable
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x130, 0);// Tx Start

	for (l = 0; l < 15; l++)
	{
		usleep(100000);
		hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

		if ((bsr & 0x100) == 0)	// The Tx FIFO is now empty;
			break;
	}

	hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

	if ((status == 0) && (bsr & 0x100))
	{
		status	|= -1;
		msg		= "Tx FIFO didn't empty";
	}

	// Retrieve the data.

	if (status == 0)
	{
		l	= read(fd, rx, bytes);

		if (l != bytes)
		{
			if (status == 0)
				msg	= "write() failure";

			status	|= -1;
		}
	}

	// Verify that the data is correct.

	if ((status == 0) && (memcmp(tx, rx, bytes)))
	{
		status	|= -1;
		msg		= "data validation failure";
	}

	if (status == 0)
		printf("PASS   (%ld samples)\n", size);
	else if (msg)
		printf("FAIL <---  (%s)\n", msg);
	else
		printf("FAIL\n");

	return(status);
}



/*****************************************************************************
*
*	Function:	_rx_data_test_16
*
*	Purpose:
*
*		Verify I/O read with 16-bit data and the given I/O mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

static int _rx_data_test_16(int fd, __u32 mode)
{
	char	buf[256];
	char*	psz;
	long	size;
	int		status	= 0;

	switch (mode)
	{
		default:	status	= -1;	psz	= "ERROR";	break;
		case HPDI32_IO_MODE_PIO:	psz	= "PIO";	break;
		case HPDI32_IO_MODE_DMA:	psz	= "DMA";	break;
		case HPDI32_IO_MODE_DMDMA:	psz	= "DMDMA";	break;
	}

	sprintf(buf, "Rx 16-bit %s", psz);
	label(buf);

	status	|= _data_size_get_fifo_size(fd, &size);

	if (status)
		printf("FAIL <---  (setup)\n");
	else if (size == 0)
		printf("SKIPPED   (FIFO size not known)\n");
	else
		status	|= _rx_data_test_16_io(fd, mode, size);

	return(status);
}



/*****************************************************************************
*
*	Function:	_rx_data_test_32_io
*
*	Purpose:
*
*		Verify I/O read with 32-bit data and the given I/O mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		mode	The I/O mode to use.
*
*		size	The number of samples to transmit and receive.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

static int _rx_data_test_32_io(int fd, __u32 mode, long size)
{
	__u32				bsr;
	long				bytes	= size * 2;
	hpdi32_io_config_t	io;
	long				l;
	const char*			msg		= NULL;
	__u32*				rx		= (__u32*) _big_rx_buffer;
	__u32*				tx		= (__u32*) _big_tx_buffer;
	int					status	= 0;

	// Initialize the buffer.

	for (l = 0; l < size; l++)
		tx[l]	= ((__u32) l) | (((__u32) l) << 24);

	// Initialize the board.
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	io.read.mode			= mode;
	io.read.timeout			= HPDI32_IO_TIMEOUT_NO_CHANGE;
	io.read.samples			= size;
	io.read.sample_bits		= HPDI32_IO_SAMPLE_BITS_32;
	io.read.pio_threshold	= 0;
	io.write.mode			= HPDI32_IO_MODE_NO_CHANGE;
	io.write.timeout		= HPDI32_IO_TIMEOUT_NO_CHANGE;
	io.write.samples		= HPDI32_IO_SAMPLES_MIN;
	io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.write.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
	status	= hpdi32_io_config(fd, &io, 0);
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x36, 0);// Tx + Rx FIFO Reset

	// Write the data to the board.

	for (l = 0; l < size; l++)
		hpdi32_reg_write(fd, HPDI32_GSC_FDR, tx[l], 0);

	// Transmit the data and wait for the Tx FIFO to empty.
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x030, 0);// Tx Enable + Rx Enable
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x130, 0);// Tx Start

	for (l = 0; l < 15; l++)
	{
		usleep(100000);
		hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

		if ((bsr & 0x100) == 0)	// The Tx FIFO is now empty;
			break;
	}

	hpdi32_reg_read(fd, HPDI32_GSC_BSR, &bsr, 0);

	if ((status == 0) && (bsr & 0x100))
	{
		status	|= -1;
		msg		= "Tx FIFO didn't empty";
	}

	// Retrieve the data.

	if (status == 0)
	{
		l	= read(fd, rx, bytes);

		if (l != bytes)
		{
			if (status == 0)
				msg	= "write() failure";

			status	|= -1;
		}
	}

	// Verify that the data is correct.

	if ((status == 0) && (memcmp(tx, rx, bytes)))
	{
		status	|= -1;
		msg		= "data validation failure";
	}

	if (status == 0)
		printf("PASS   (%ld samples)\n", size);
	else if (msg)
		printf("FAIL <---  (%s)\n", msg);
	else
		printf("FAIL\n");

	return(status);
}



/*****************************************************************************
*
*	Function:	_rx_data_test_32
*
*	Purpose:
*
*		Verify I/O read with 32-bit data and the given I/O mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

static int _rx_data_test_32(int fd, __u32 mode)
{
	char	buf[256];
	char*	psz;
	long	size;
	int		status	= 0;

	switch (mode)
	{
		default:	status	= -1;	psz	= "ERROR";	break;
		case HPDI32_IO_MODE_PIO:	psz	= "PIO";	break;
		case HPDI32_IO_MODE_DMA:	psz	= "DMA";	break;
		case HPDI32_IO_MODE_DMDMA:	psz	= "DMDMA";	break;
	}

	sprintf(buf, "Rx 32-bit %s", psz);
	label(buf);

	status	|= _data_size_get_fifo_size(fd, &size);

	if (status)
		printf("FAIL <---  (setup)\n");
	else if (size == 0)
		printf("SKIPPED   (FIFO size not known)\n");
	else
		status	|= _rx_data_test_32_io(fd, mode, size);

	return(status);
}



/*****************************************************************************
*
*	Function:	io_data_size_tests
*
*	Purpose:
*
*		Verify I/O reads and write with the each data size and I/O mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

int io_data_size_tests(int fd)
{
	int	status	= 0;

	label("I/O Data Size Tests");
	printf("\n");
	label_level_inc();

	status	|= _tx_data_test_8(fd, HPDI32_IO_MODE_PIO);
	status	|= _tx_data_test_8(fd, HPDI32_IO_MODE_DMA);
	status	|= _tx_data_test_8(fd, HPDI32_IO_MODE_DMDMA);
	status	|= _tx_data_test_16(fd, HPDI32_IO_MODE_PIO);
	status	|= _tx_data_test_16(fd, HPDI32_IO_MODE_DMA);
	status	|= _tx_data_test_16(fd, HPDI32_IO_MODE_DMDMA);
	status	|= _tx_data_test_32(fd, HPDI32_IO_MODE_PIO);
	status	|= _tx_data_test_32(fd, HPDI32_IO_MODE_DMA);
	status	|= _tx_data_test_32(fd, HPDI32_IO_MODE_DMDMA);

	status	|= _rx_data_test_8(fd, HPDI32_IO_MODE_PIO);
	status	|= _rx_data_test_8(fd, HPDI32_IO_MODE_DMA);
	status	|= _rx_data_test_8(fd, HPDI32_IO_MODE_DMDMA);
	status	|= _rx_data_test_16(fd, HPDI32_IO_MODE_PIO);
	status	|= _rx_data_test_16(fd, HPDI32_IO_MODE_DMA);
	status	|= _rx_data_test_16(fd, HPDI32_IO_MODE_DMDMA);
	status	|= _rx_data_test_32(fd, HPDI32_IO_MODE_PIO);
	status	|= _rx_data_test_32(fd, HPDI32_IO_MODE_DMA);
	status	|= _rx_data_test_32(fd, HPDI32_IO_MODE_DMDMA);

	label_level_dec();
	return(status);
}



