/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 2 $ */
/* $Date: 9/28/06 5:37p $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.17.0-GSC-DN/test/ioctl.c $
 *
 * 2     9/28/06 5:37p Dnewbold
 * Rewrote the IOCTL test routines.
 *
 * 1     2/27/06 10:40a Dnewbold
 * Modularized the test code.
*/

#include <stdio.h>

#include "main.h"
#include "HPDI32DocSrcLib.h"
#include "util_label.h"



/* data types	*************************************************************/

typedef struct
{
	__u32	set;
	__u32	get;
} u32_2_t;



/*****************************************************************************
*
*	Function:	_no_command_test
*
*	Purpose:
*
*		Verify the No Command IOCTL services.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _no_command_test(int fd)
{
	int	status;

	label("HPDI32_IOCTL_NO_COMMAND");
	status	= hpdi32_ioctl(fd, HPDI32_IOCTL_NO_COMMAND, NULL, 0);
	printf("%s\n", status ? "FAIL <---" : "PASS");
	return(status);
}



/*****************************************************************************
*
*	Function:	_mmap_info_test
*
*	Purpose:
*
*		Verify the mmap Information IOCTL services.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _mmap_info_test(int fd)
{
	hpdi32_mmap_t	mm;
	int				status;

	label("HPDI32_IOCTL_MMAP_INFO");
	status	= hpdi32_ioctl(fd, HPDI32_IOCTL_MMAP_INFO, &mm, 0);
	printf("%s\n", status ? "FAIL <---" : "PASS");
	return(status);
}



/*****************************************************************************
*
*	Function:	_debug_data_get_test
*
*	Purpose:
*
*		Verify the Debug Data Get IOCTL services.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _debug_data_get_test(int fd)
{
	hpdi32_debug_data_t	data;
	int			status;

	label("HPDI32_IOCTL_DEBUG_DATA_GET");
	status	= hpdi32_ioctl(fd, HPDI32_IOCTL_DEBUG_DATA_GET, &data, 0);
	printf("%s\n", status ? "FAIL <---" : "PASS");
	return(status);
}



/*****************************************************************************
*
*	Function:	_driver_info_get_test
*
*	Purpose:
*
*		Verify the Driver Info Get IOCTL services.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _driver_info_get_test(int fd)
{
	hpdi32_driver_info_t	data;
	int			status;

	label("HPDI32_IOCTL_DRIVER_INFO_GET");
	status	= hpdi32_ioctl(fd, HPDI32_IOCTL_DRIVER_INFO_GET, &data, 0);
	printf("%s\n", status ? "FAIL <---" : "PASS");
	return(status);
}



/*****************************************************************************
*
*	Function:	_reg_mod_test
*
*	Purpose:
*
*		Verify the Register Modify IOCTL services.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _reg_mod_test(int fd)
{
	__u32		before;
	hpdi32_reg_t	reg;
	int		status	= 0;

	label("HPDI32_IOCTL_REG_MOD");
	reg.reg		= HPDI32_GSC_TAR;
	reg.value	= 0x00100010;
	status		|= hpdi32_ioctl(fd, HPDI32_IOCTL_REG_WRITE, &reg, 0);
	status		|= hpdi32_ioctl(fd, HPDI32_IOCTL_REG_READ, &reg, 0);
	before		= reg.value;
	reg.value	= 0x00000001;
	reg.mask	= 0x00000001;
	status		|= hpdi32_ioctl(fd, HPDI32_IOCTL_REG_MOD, &reg, 0);
	status		|= hpdi32_ioctl(fd, HPDI32_IOCTL_REG_READ, &reg, 0);

	if ((status == 0) && (reg.value != 0x00100011))
		status	= -1;

	printf("%s\n", status ? "FAIL <---" : "PASS");
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_config_parm_test
*
*	Purpose:
*
*		Verify the Rx Mode parameter to the I/O Configuration IOCTL
*		services.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _io_config_parm_test(
	int			fd,
	const char*		name,
	hpdi32_io_config_t*	io,
	__u32*			parm,
	const u32_2_t*		list,
	size_t			size)
{
	int	i;
	int	status	= 0;

	label(name);

	for (i = 0; i < (int) size; i++)
	{
		io->read.mode		= HPDI32_IO_MODE_NO_CHANGE;
		io->read.timeout	= HPDI32_IO_TIMEOUT_NO_CHANGE;
		io->read.samples	= HPDI32_IO_SAMPLES_NO_CHANGE;
		io->read.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
		io->read.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
		io->write.mode		= HPDI32_IO_MODE_NO_CHANGE;
		io->write.timeout	= HPDI32_IO_TIMEOUT_NO_CHANGE;
		io->write.samples	= HPDI32_IO_SAMPLES_NO_CHANGE;
		io->write.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
		io->write.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
		parm[0]			= list[i].set;
		status	|= hpdi32_ioctl(fd, HPDI32_IOCTL_IO_CONFIG, io, 0);

		if (	(parm == &io->read.samples) ||
			(parm == &io->write.samples))
		{
			// It is OK if they don't match exactly.
			// But we're asking for so little that it
			// shouldn't be less.

			if (parm[0] > list[i].get)
				status	= -1;
		}
		else if (parm[0] != list[i].get)
		{
			// In all other case it must match exactly.
			status	= -1;
		}
	}

	printf("%s\n", status ? "FAIL <---" : "PASS");
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_mode_test
*
*	Purpose:
*
*		Verify the Tx and Rx Mode parameters to the I/O Configuration
*		IOCTL services.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _io_mode_test(int fd)
{
	static const u32_2_t	list[]	=
	{
		// set				get
		{ HPDI32_IO_MODE_PIO,		HPDI32_IO_MODE_PIO	},
		{ HPDI32_IO_MODE_NO_CHANGE,	HPDI32_IO_MODE_PIO	},
		{ HPDI32_IO_MODE_DMA,		HPDI32_IO_MODE_DMA	},
		{ HPDI32_IO_MODE_NO_CHANGE,	HPDI32_IO_MODE_DMA	},
		{ HPDI32_IO_MODE_DMDMA,		HPDI32_IO_MODE_DMDMA	},
		{ HPDI32_IO_MODE_NO_CHANGE,	HPDI32_IO_MODE_DMDMA	},
		{ HPDI32_IO_MODE_DEFAULT,	HPDI32_IO_MODE_DEFAULT	},
		{ HPDI32_IO_MODE_NO_CHANGE,	HPDI32_IO_MODE_DEFAULT	}
	};

	hpdi32_io_config_t	io;
	int			status;

	status	= _io_config_parm_test(	fd,
					"Rx Mode",
					&io,
					(__u32*) &io.read.mode,
					list,
					ARRAY_ELEMENTS(list));
	status	= _io_config_parm_test(	fd,
					"Tx Mode",
					&io,
					(__u32*) &io.write.mode,
					list,
					ARRAY_ELEMENTS(list));
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_timeout_test
*
*	Purpose:
*
*		Verify the Tx and Rx Timeout parameters to the I/O
*		Configuration IOCTL services.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _io_timeout_test(int fd)
{
	static const u32_2_t	list[]	=
	{
	// set				get
	{ HPDI32_IO_TIMEOUT_MIN,	HPDI32_IO_TIMEOUT_MIN		},
	{ HPDI32_IO_TIMEOUT_NO_CHANGE,	HPDI32_IO_TIMEOUT_MIN		},
	{ HPDI32_IO_TIMEOUT_MAX,	HPDI32_IO_TIMEOUT_MAX		},
	{ HPDI32_IO_TIMEOUT_NO_CHANGE,	HPDI32_IO_TIMEOUT_MAX		},
	{ HPDI32_IO_TIMEOUT_DEFAULT,	HPDI32_IO_TIMEOUT_DEFAULT	},
	{ HPDI32_IO_TIMEOUT_NO_CHANGE,	HPDI32_IO_TIMEOUT_DEFAULT	}
	};

	hpdi32_io_config_t	io;
	int			status;

	status	= _io_config_parm_test(	fd,
					"Rx Timeout",
					&io,
					(__u32*) &io.read.timeout,
					list,
					ARRAY_ELEMENTS(list));
	status	= _io_config_parm_test(	fd,
					"Tx Timeout",
					&io,
					(__u32*) &io.write.timeout,
					list,
					ARRAY_ELEMENTS(list));
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_samples_test
*
*	Purpose:
*
*		Verify the Tx and Rx Samples parameters to the I/O
*		Configuration IOCTL services.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _io_samples_test(int fd)
{
	static const u32_2_t	list[]	=
	{
		// set				get
		{ 4096,				4096	},
		{ HPDI32_IO_SAMPLES_NO_CHANGE,	4096	},
		{ 8192,				8192	},
		{ HPDI32_IO_SAMPLES_NO_CHANGE,	8192	},
		{ 16384,			16384	},
		{ HPDI32_IO_SAMPLES_NO_CHANGE,	16384	}
	};

	hpdi32_io_config_t	io;
	int			status;

	status	= _io_config_parm_test(	fd,
					"Rx Samples",
					&io,
					&io.read.samples,
					list,
					ARRAY_ELEMENTS(list));
	status	= _io_config_parm_test(	fd,
					"Tx Samples",
					&io,
					&io.write.samples,
					list,
					ARRAY_ELEMENTS(list));
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_sample_bits_test
*
*	Purpose:
*
*		Verify the Tx and Rx Sample Bits parameters to the I/O
*		Configuration IOCTL services.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _io_sample_bits_test(int fd)
{
	static const u32_2_t	list[]	=
	{
	// set					get
	{ HPDI32_IO_SAMPLE_BITS_8,		HPDI32_IO_SAMPLE_BITS_8	},
	{ HPDI32_IO_SAMPLE_BITS_NO_CHANGE,	HPDI32_IO_SAMPLE_BITS_8	},
	{ HPDI32_IO_SAMPLE_BITS_16,		HPDI32_IO_SAMPLE_BITS_16},
	{ HPDI32_IO_SAMPLE_BITS_NO_CHANGE,	HPDI32_IO_SAMPLE_BITS_16},
	{ HPDI32_IO_SAMPLE_BITS_32,		HPDI32_IO_SAMPLE_BITS_32},
	{ HPDI32_IO_SAMPLE_BITS_NO_CHANGE,	HPDI32_IO_SAMPLE_BITS_32},
	{ HPDI32_IO_SAMPLE_BITS_DEFAULT,	HPDI32_IO_SAMPLE_BITS_DEFAULT},
	{ HPDI32_IO_SAMPLE_BITS_NO_CHANGE,	HPDI32_IO_SAMPLE_BITS_DEFAULT}
	};

	hpdi32_io_config_t	io;
	int			status;

	status	= _io_config_parm_test(	fd,
					"Rx Sample Bits",
					&io,
					(__u32*) &io.read.sample_bits,
					list,
					ARRAY_ELEMENTS(list));
	status	= _io_config_parm_test(	fd,
					"Tx Sample Bits",
					&io,
					(__u32*) &io.write.sample_bits,
					list,
					ARRAY_ELEMENTS(list));
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_pio_threshold_test
*
*	Purpose:
*
*		Verify the Tx and Rx Pio Threshold parameters to the I/O
*		Configuration IOCTL services.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _io_pio_threshold_test(int fd)
{
	static const u32_2_t	list[]	=
	{
	// set					get
	{ HPDI32_IO_PIO_THRESHOLD_MIN,		HPDI32_IO_PIO_THRESHOLD_MIN},
	{ HPDI32_IO_PIO_THRESHOLD_NO_CHANGE,	HPDI32_IO_PIO_THRESHOLD_MIN},
	{ 1024,					1024			},
	{ HPDI32_IO_PIO_THRESHOLD_NO_CHANGE,	1024			},
	{ HPDI32_IO_PIO_THRESHOLD_DEFAULT,	HPDI32_IO_PIO_THRESHOLD_DEFAULT},
	{ HPDI32_IO_PIO_THRESHOLD_NO_CHANGE,	HPDI32_IO_PIO_THRESHOLD_DEFAULT}
	};

	hpdi32_io_config_t	io;
	int			status;

	status	= _io_config_parm_test(	fd,
					"Rx PIO Threshold",
					&io,
					(__u32*) &io.read.pio_threshold,
					list,
					ARRAY_ELEMENTS(list));
	status	= _io_config_parm_test(	fd,
					"Tx PIO Threshold",
					&io,
					(__u32*) &io.write.pio_threshold,
					list,
					ARRAY_ELEMENTS(list));
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_config_test
*
*	Purpose:
*
*		Verify the I/O Configuration IOCTL services.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _io_config_test(int fd)
{
	int	status	= 0;

	label("HPDI32_IOCTL_IO_CONFIG");
	printf("\n");
	label_level_inc();

	status	|= _io_mode_test(fd);
	status	|= _io_timeout_test(fd);
	status	|= _io_samples_test(fd);
	status	|= _io_sample_bits_test(fd);
	status	|= _io_pio_threshold_test(fd);

	label_level_dec();
	return(status);
}



/*****************************************************************************
*
*	Function:	ioctl_test
*
*	Purpose:
*
*		Verify that all IOCTL services function properly.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

int ioctl_test(int fd)
{
#if 0
	union
	{
		hpdi32_mmap_t			mmap_info;
	} u;

	hpdi32_io_config_t	io_config	=
	{
		{
			HPDI32_IO_MODE_NO_CHANGE,
			HPDI32_IO_TIMEOUT_NO_CHANGE,
			HPDI32_IO_SAMPLES_NO_CHANGE,
			HPDI32_IO_SAMPLE_BITS_NO_CHANGE,
			HPDI32_IO_PIO_THRESHOLD_NO_CHANGE
		},
		{
			HPDI32_IO_MODE_NO_CHANGE,
			HPDI32_IO_TIMEOUT_NO_CHANGE,
			HPDI32_IO_SAMPLES_NO_CHANGE,
			HPDI32_IO_SAMPLE_BITS_NO_CHANGE,
			HPDI32_IO_PIO_THRESHOLD_NO_CHANGE
		}
	};

	hpdi32_interrupt_t	int_test	= { { 0, 0 }, { 0, 0 } };

	const struct
	{
		unsigned long	cmd;
		void*		arg;
	} list[]	=
	{
		/* cmd							arg		*/
		{ HPDI32_IOCTL_IO_CONFIG,		&io_config	},
		{ HPDI32_IOCTL_INT_NOTIFY,		&int_test	},
		{ HPDI32_IOCTL_INT_STATUS,		&int_test	},
		{ HPDI32_IOCTL_MMAP_INFO,		&u.mmap_info	}
	};
#endif

	int	status	= 0;

	label("IOCTL Tests");
	printf("\n");
	label_level_inc();

	status	|= _no_command_test(fd);
	status	|= _debug_data_get_test(fd);
	status	|= _driver_info_get_test(fd);
	status	|= register_read_test(fd);
	status	|= register_write_test(fd);
	status	|= _reg_mod_test(fd);
	status	|= _io_config_test(fd);
	status	|= int_notify_ioctl_test(fd);
	status	|= int_status_ioctl_test(fd);
	status	|= _mmap_info_test(fd);

	label_level_dec();
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	return(status);
}



