/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 2 $ */
/* $Date: 9/28/06 5:36p $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.17.0-GSC-DN/test/irq.c $
 *
 * 2     9/28/06 5:36p Dnewbold
 * Added more test functions and renamed an existing one.
 *
 * 1     2/27/06 10:40a Dnewbold
 * Modularized the test code.
*/

#include <fcntl.h>
#include <signal.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>

#include "main.h"
#include "HPDI32DocSrcLib.h"
#include "util_label.h"



/* variables	*************************************************************/

static	int	_sigio		= 0;



/*****************************************************************************
*
*	Function:	_handle_sigio
*
*	Purpose:
*
*		Handle an asynchronous signal.
*
*	Arguments:
*
*		signo	The signal number to process.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _handle_sigio(int signo)
{
	_sigio++;
}



/*****************************************************************************
*
*	Function:	int_notify_callback_test
*
*	Purpose:
*
*		Verify that the interrupt notification feature is
*		functional.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

int int_notify_callback_test(int fd)
{
	#define	_INT_RE		HPDI32_INT_GSC_RE_GPIO5
	#define	_INT_TE		HPDI32_INT_GSC_TE_GPIO4
	#define	_INT_TE_RE	(_INT_TE | _INT_RE)

	static const struct
	{
		const char*		name;
		int				sigio;
		unsigned long	icr;
	} tests[]	=
	{
		/* name				sigio	icr			*/
		{ "Tx/Rx Disable",	0,		0x0			},
		{ "Tx Enable",		1,		_INT_TE		},
		{ "Rx Enable",		0,		_INT_RE		},
		{ "Tx/Rx Enable",	1,		_INT_TE_RE	}
	};

	unsigned long		flags;
	int					i;
	hpdi32_interrupt_t	int_data;
	int					own;
	pid_t				pid;
	int					status	= 0;

	label("Interrupt Notification");

	/* Enable interrupt notification. */
	signal(SIGIO, _handle_sigio);
	own		= fcntl(fd, F_GETOWN);
	pid		= getpid();
	fcntl(fd, F_SETOWN, pid);
	flags	= fcntl(fd, F_GETFL);
	flags	|= FASYNC;
	fcntl(fd, F_SETFL, flags);
	int_data.notify.gsc	= _INT_TE_RE;
	int_data.notify.other	= 0;
	hpdi32_int_notify(fd, &int_data, 0);

	for (i = 0; (status == 0) && (i < ARRAY_ELEMENTS(tests)); i++)
	{
		/* Configure the interrupt. */
		hpdi32_reg_write(fd, HPDI32_GSC_ICR, 0, 0);
		hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0, 0);
		hpdi32_reg_write(fd, HPDI32_GSC_IELR, _INT_TE_RE, 0);
		hpdi32_reg_write(fd, HPDI32_GSC_IHLR, _INT_TE_RE, 0);
		hpdi32_reg_write(fd, HPDI32_GSC_ICR, tests[i].icr, 0);

		/* Cause the interrupt and wait a moment. */
		_sigio	= 0;
		flags	= HPDI32_GSC_BCR_TX_ENABLE;
		hpdi32_reg_write(fd, HPDI32_GSC_BCR, flags, 0);
		usleep(10);

		/* Cleanup afterwards. */
		hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0, 0);
		hpdi32_reg_write(fd, HPDI32_GSC_ICR, 0, 0);

		/* Check the results. */

		if (_sigio != tests[i].sigio)
		{
			printf("(%s %d) ", tests[i].name, _sigio);
			status	= -1;
			break;
		}
	}

	/* Disable interrupt notification. */
	int_data.notify.gsc	= 0;
	hpdi32_int_notify(fd, &int_data, 0);
	signal(SIGIO, SIG_IGN);
	fcntl(fd, F_SETOWN, own);
	flags	= fcntl(fd, F_GETFL);
	flags	&= ~FASYNC;
	fcntl(fd, F_SETFL, flags);

	printf("%s\n", (status == -1) ? "FAIL" : "PASS");
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	return(status);
}



/*****************************************************************************
*
*	Function:	_int_notify_test
*
*	Purpose:
*
*		Verify that the Interrupt Notification IOCTL service is functional.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _int_notify_test(
	int					fd,
	hpdi32_interrupt_t*	irq,
	__u32*				set,
	const __u32*		list,
	size_t				size)
{
	int	errs;
	int	i;
	int	status	= 0;

	for (i = 0; i < (int) size; i++)
	{
		memset(irq, 0, sizeof(irq[0]));
		set[0]	= list[i];
		status	|= hpdi32_ioctl(fd, HPDI32_IOCTL_INT_NOTIFY, irq, 0);
	}

	errs	= status ? 1 : 0;
	return(errs);
}



/*****************************************************************************
*
*	Function:	int_notify_ioctl_test
*
*	Purpose:
*
*		Verify that the Interrupt Notification IOCTL service is functional.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int int_notify_ioctl_test(int fd)
{
	static const __u32	other[]	=
	{
		HPDI32_INT_PCI,
		HPDI32_INT_ANY_OTHER,
		HPDI32_INT_DMA_0_DONE,
		HPDI32_INT_DMA_1_DONE
	};

	static const __u32	gsc[]	=
	{
		HPDI32_INT_GSC_FRAME_VALID_S,
		HPDI32_INT_GSC_FRAME_VALID_E,
		HPDI32_INT_GSC_LV_GPIO0,
		HPDI32_INT_GSC_SV_GPIO1,
		HPDI32_INT_GSC_RR_GPIO2,
		HPDI32_INT_GSC_TR_GPIO3,
		HPDI32_INT_GSC_TE_GPIO4,
		HPDI32_INT_GSC_RE_GPIO5,
		HPDI32_INT_GSC_TX_FIFO_EMPTY,
		HPDI32_INT_GSC_TX_FIFO_AE,
		HPDI32_INT_GSC_TX_FIFO_AF,
		HPDI32_INT_GSC_TX_FIFO_FULL,
		HPDI32_INT_GSC_RX_FIFO_EMPTY,
		HPDI32_INT_GSC_RX_FIFO_AE,
		HPDI32_INT_GSC_RX_FIFO_AF,
		HPDI32_INT_GSC_RX_FIFO_FULL
	};

	int					errs	= 0;
	hpdi32_interrupt_t	irq;

	label("HPDI32_IOCTL_INT_NOTIFY");
	errs	+= _int_notify_test(fd,
								&irq,
								&irq.notify.gsc,
								gsc,
								ARRAY_ELEMENTS(gsc));
	errs	+= _int_notify_test(fd,
								&irq,
								&irq.notify.other,
								other,
								ARRAY_ELEMENTS(other));
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/*****************************************************************************
*
*	Function:	int_status_ioctl_test
*
*	Purpose:
*
*		Verify that the Interrupt Status IOCTL service is functional.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int int_status_ioctl_test(int fd)
{
	static const __u32	gsc[]	=
	{
		HPDI32_INT_GSC_FRAME_VALID_S,
		HPDI32_INT_GSC_FRAME_VALID_E,
		HPDI32_INT_GSC_LV_GPIO0,
		HPDI32_INT_GSC_SV_GPIO1,
		HPDI32_INT_GSC_RR_GPIO2,
		HPDI32_INT_GSC_TR_GPIO3,
		HPDI32_INT_GSC_TE_GPIO4,
		HPDI32_INT_GSC_RE_GPIO5,
		HPDI32_INT_GSC_TX_FIFO_EMPTY,
		HPDI32_INT_GSC_TX_FIFO_AE,
		HPDI32_INT_GSC_TX_FIFO_AF,
		HPDI32_INT_GSC_TX_FIFO_FULL,
		HPDI32_INT_GSC_RX_FIFO_EMPTY,
		HPDI32_INT_GSC_RX_FIFO_AE,
		HPDI32_INT_GSC_RX_FIFO_AF,
		HPDI32_INT_GSC_RX_FIFO_FULL
	};

	int					errs	= 0;
	int					i;
	hpdi32_interrupt_t	irq;
	int					status	= 0;

	label("HPDI32_IOCTL_INT_STATUS");

	// Enable notification of all interrupts.
	memset(&irq, 0, sizeof(irq));

	for (i = 0; i < (int) ARRAY_ELEMENTS(gsc); i++)
		irq.notify.gsc	|= gsc[i];

	status	|= hpdi32_ioctl(fd, HPDI32_IOCTL_INT_NOTIFY, &irq, 0);

	// Generate a Frame Valid Begin interrupt.
	status	|= hpdi32_reg_write(fd, HPDI32_GSC_ICR, 0x001, 0);// Fram Val Strt
	status	|= hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x002, 0);// Tx FIFO Reset
	status	|= hpdi32_reg_write(fd, HPDI32_GSC_FDR, 0x000, 0);// FIFO Data
	status	|= hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x010, 0);// Tx Enable
	status	|= hpdi32_reg_write(fd, HPDI32_GSC_BCR, 0x110, 0);// Tx Start
	usleep(100);

	// Get the Interrupt Status.
	status	|= hpdi32_ioctl(fd, HPDI32_IOCTL_INT_STATUS, &irq, 0);

	if (irq.status.gsc != HPDI32_INT_GSC_FRAME_VALID_S)
		status	|= -1;

	// Reset things.
	memset(&irq, 0, sizeof(irq));
	status	|= hpdi32_ioctl(fd, HPDI32_IOCTL_INT_NOTIFY, &irq, 0);
	status	|= hpdi32_reg_write(fd, HPDI32_GSC_BCR, 1, 0);// Board Reset

	// Finish
	errs	= status ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



