/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 2 $ */
/* $Date: 9/28/06 5:34p $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.17.0-GSC-DN/test/mmap.c $
 *
 * 2     9/28/06 5:34p Dnewbold
 * Updated per changes to the 1.18 driver.
 *
 * 1     2/27/06 10:40a Dnewbold
 * Modularized the test code.
*/

#include <stdio.h>
#include <sys/mman.h>

#include "main.h"
#include "HPDI32DocSrcLib.h"
#include "util_label.h"



/*****************************************************************************
*
*	Function:	_mmap_gsc_regs_test
*
*	Purpose:
*
*		Verify mmap() of the GSC registers.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _mmap_gsc_regs_test(int fd)
{
	int				errs	= 0;
	int				i;
	hpdi32_mmap_t	map;
	int				status;
	__u32*			ptr;
	__u32			v;

	for (;;)
	{
		label("GSC Registers");

		status	= hpdi32_mmap_info(fd, &map, 0);

		if (status)
		{
			errs	= 1;
			printf("FAIL <---  (hpdi32_mmap_info)\n");
			break;
		}

		if (map.regs.size == 0)
		{
			printf("unavailable\n");
			break;
		}

		ptr	= mmap(	0,
					map.regs.size,
					PROT_READ | PROT_WRITE,
					MAP_SHARED,
					fd,
					map.regs.offset);

		if (ptr == MAP_FAILED)
		{
			errs	= 1;
			printf("FAIL <---  (mmap)\n");
			break;
		}

		// Verify that the region can be read.

		for (i = 0; i < (map.regs.size / 4); i++)
			v	= ptr[i];

		// Verify that the region can be written.

		for (i = 0; i < (map.regs.size / 4); i++)
			ptr[i]	= 0;

		// We're done.
		munmap(ptr, map.regs.size);
		printf("PASS\n");
		break;
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_mmap_access_test
*
*	Purpose:
*
*		Verify mmap() using a buffer of the specified characteristics.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		offset	The offset to pass to mmap().
*
*		size	The size to pass to mmap();
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _mmap_access_test(int fd, long offset, long size)
{
	int		errs	= 0;
	int		i;
	long	l;
	__u32*	ptr		= NULL;
	long	samples;
	__u32	v;

	for (;;)
	{
		ptr	= mmap(0, size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, offset);

		if (ptr == MAP_FAILED)
		{
			errs	= 1;
			printf("FAIL <---  (mmap)\n");
			break;
		}

		// Verify write/read access to the memory region.
		samples	= size / 4;

		for (l = 0; l < samples; l++)
			ptr[l]	= 0x80000000 + l;

		for (l = 0; l < samples; l++)
		{
			v	= ptr[l];

			if (v != (0x80000000 + l))
				errs	= 1;
		}

		if (errs)
		{
			printf("FAIL <---  (wrote x, read y)\n");
			break;
		}

		// We're done.
		i	= munmap(ptr, size);

		if (i)
		{
			errs++;
			printf("FAIL <---  (munmap)\n");
			break;
		}

		printf("PASS\n");
		break;
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_mmap_io_rx_size_test
*
*	Purpose:
*
*		Verify mmap() using a Rx I/O buffer of the requested size.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		samples	The requested size of the buffer to use.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _mmap_io_rx_size_test(int fd, long samples)
{
	char				buf[64];
	int					errs	= 0;
	hpdi32_io_config_t	io;
	int					k	= 1024;
	hpdi32_mmap_t		map;
	const char*			psz;
	int					status;

	for (;;)
	{
		io.read.mode			= HPDI32_IO_MODE_NO_CHANGE;
		io.read.timeout			= HPDI32_IO_TIMEOUT_NO_CHANGE;
		io.read.samples			= samples;
		io.read.sample_bits		= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
		io.read.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
		io.write.mode			= HPDI32_IO_MODE_NO_CHANGE;
		io.write.timeout		= HPDI32_IO_TIMEOUT_NO_CHANGE;
		io.write.samples		= HPDI32_IO_SAMPLES_MIN;
		io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
		io.write.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
		status					= hpdi32_io_config(fd, &io, 0);

		psz	= label_format_k(io.read.samples, k);
		sprintf(buf, "%s Samples", psz);
		label(buf);

		if (status)
		{
			errs	= 1;
			printf("FAIL <---  (hpdi32_io_config)\n");
			break;
		}

		status	= hpdi32_mmap_info(fd, &map, 0);

		if (status)
		{
			errs	= 1;
			printf("FAIL <---  (hpdi32_mmap_info)\n");
			break;
		}

		errs	= _mmap_access_test(fd, map.read.offset, map.read.size);
		break;
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_mmap_io_tx_size_test
*
*	Purpose:
*
*		Verify mmap() using a Tx I/O buffer of the requested size.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		samples	The requested size of the buffer to use.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _mmap_io_tx_size_test(int fd, long samples)
{
	char				buf[64];
	int					errs	= 0;
	hpdi32_io_config_t	io;
	int					k	= 1024;
	hpdi32_mmap_t		map;
	const char*			psz;
	int					status;

	for (;;)
	{
		io.read.mode			= HPDI32_IO_MODE_NO_CHANGE;
		io.read.timeout			= HPDI32_IO_TIMEOUT_NO_CHANGE;
		io.read.samples			= HPDI32_IO_SAMPLES_MIN;
		io.read.sample_bits		= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
		io.read.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
		io.write.mode			= HPDI32_IO_MODE_NO_CHANGE;
		io.write.timeout		= HPDI32_IO_TIMEOUT_NO_CHANGE;
		io.write.samples		= samples;
		io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
		io.write.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
		status					= hpdi32_io_config(fd, &io, 0);

		psz	= label_format_k(io.write.samples, k);
		sprintf(buf, "%s Samples", psz);
		label(buf);

		if (status)
		{
			errs	= 1;
			printf("FAIL <---  (hpdi32_io_config)\n");
			break;
		}

		status	= hpdi32_mmap_info(fd, &map, 0);

		if (status)
		{
			errs	= 1;
			printf("FAIL <---  (hpdi32_mmap_info)\n");
			break;
		}

		errs	= _mmap_access_test(fd, map.write.offset, map.write.size);
		break;
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_mmap_io_rx_test
*
*	Purpose:
*
*		Verify mmap() using the Rx I/O buffer.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _mmap_io_rx_test(int fd)
{
	int	errs	= 0;

	label("Rx I/O Buffer");
	printf("\n");
	label_level_inc();

	errs	+= _mmap_io_rx_size_test(fd, HPDI32_IO_SAMPLES_MIN + 4096);

	label_level_dec();
	return(errs);
}



/*****************************************************************************
*
*	Function:	_mmap_io_tx_test
*
*	Purpose:
*
*		Verify mmap() using the Tx I/O buffer.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

static int _mmap_io_tx_test(int fd)
{
	int	errs	= 0;

	label("Tx I/O Buffer");
	printf("\n");
	label_level_inc();

	errs	+= _mmap_io_tx_size_test(fd, HPDI32_IO_SAMPLES_MIN + 4096);

	label_level_dec();
	return(errs);
}



/*****************************************************************************
*
*	Function:	mmap_test
*
*	Purpose:
*
*		Verify the mmap() implementation.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

int mmap_test(int fd)
{
	int	errs	= 0;

	label("mmap()");
	printf("\n");
	label_level_inc();

	errs	+= _mmap_gsc_regs_test(fd);
	errs	+= _mmap_io_tx_test(fd);
	errs	+= _mmap_io_rx_test(fd);

	label_level_dec();
	return(errs);
}



