/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 2 $ */
/* $Date: 9/28/06 5:34p $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.17.0-GSC-DN/test/reg.c $
 *
 * 2     9/28/06 5:34p Dnewbold
 * Modified the test titles, and when the test functions are called.
 *
 * 1     2/27/06 10:41a Dnewbold
 * Modularized the test code.
*/

#include <stdio.h>

#include "main.h"
#include "HPDI32DocSrcLib.h"
#include "util_label.h"



/* variables	*************************************************************/

static const struct
{
	unsigned long	reg;
	int		writeable;
} _reg_list[]	=
{
	/* reg					writeable	*/
	/* These are common to both the PCI9080 and the PCI9656. */
	{ HPDI32_PCI_IDR,		0		},
	{ HPDI32_PCI_CR,		0		},
	{ HPDI32_PCI_SR,		0		},
	{ HPDI32_PCI_REV,		0		},
	{ HPDI32_PCI_CCR,		0		},
	{ HPDI32_PCI_CLSR,		0		},
	{ HPDI32_PCI_LTR,		0		},
	{ HPDI32_PCI_HTR,		0		},
	{ HPDI32_PCI_BISTR,		0		},
	{ HPDI32_PCI_BAR0,		0		},
	{ HPDI32_PCI_BAR1,		0		},
	{ HPDI32_PCI_BAR2,		0		},
	{ HPDI32_PCI_BAR3,		0		},
	{ HPDI32_PCI_BAR4,		0		},
	{ HPDI32_PCI_BAR5,		0		},
	{ HPDI32_PCI_CIS,		0		},
	{ HPDI32_PCI_SVID,		0		},
	{ HPDI32_PCI_SID,		0		},
	{ HPDI32_PCI_ERBAR,		0		},
	{ HPDI32_PCI_ILR,		0		},
	{ HPDI32_PCI_IPR,		0		},
	{ HPDI32_PCI_MGR,		0		},
	{ HPDI32_PCI_MLR,		0		},

	/* These are specific to the PCI9656. */
	{ HPDI32_PCI_CAP_PTR,	0		},
	{ HPDI32_PCI_PMCAPID,	0		},
	{ HPDI32_PCI_PMNEXT,	0		},
	{ HPDI32_PCI_PMC,		0		},
	{ HPDI32_PCI_PMCSR,		0		},
	{ HPDI32_PCI_PMCSR_BSE,	0		},
	{ HPDI32_PCI_PMDATA,	0		},
	{ HPDI32_PCI_HS_CNTL,	0		},
	{ HPDI32_PCI_HS_NEXT,	0		},
	{ HPDI32_PCI_HS_CSR,	0		},
	{ HPDI32_PCI_VPDCNTL,	0		},
	{ HPDI32_PCI_VPD_NEXT,	0		},
	{ HPDI32_PCI_VPDAD,		0		},
	{ HPDI32_PCI_VPDATA,	0		},

	/* These are common to both the PCI9080 and the PCI9656. */
	{ HPDI32_PLX_LAS0RR,	0		},
	{ HPDI32_PLX_LAS0BA,	0		},
	{ HPDI32_PLX_MARBR,		0		},
	{ HPDI32_PLX_BIGEND,	0		},
	{ HPDI32_PLX_EROMRR,	0		},
	{ HPDI32_PLX_EROMBA,	0		},
	{ HPDI32_PLX_LBRD0,		0		},
	{ HPDI32_PLX_DMRR,		0		},
	{ HPDI32_PLX_DMLBAM,	0		},
	{ HPDI32_PLX_DMLBAI,	0		},
	{ HPDI32_PLX_DMPBAM,	0		},
	{ HPDI32_PLX_DMCFGA,	0		},
	{ HPDI32_PLX_LAS1RR,	0		},
	{ HPDI32_PLX_LAS1BA,	0		},
	{ HPDI32_PLX_LBRD1,		0		},

	{ HPDI32_PLX_MBOX0,		0		},
	{ HPDI32_PLX_MBOX1,		0		},
	{ HPDI32_PLX_MBOX2,		0		},
	{ HPDI32_PLX_MBOX3,		0		},
	{ HPDI32_PLX_MBOX4,		0		},
	{ HPDI32_PLX_MBOX5,		0		},
	{ HPDI32_PLX_MBOX6,		0		},
	{ HPDI32_PLX_MBOX7,		0		},
	{ HPDI32_PLX_P2LDBELL,	0		},
	{ HPDI32_PLX_L2PDBELL,	0		},
	{ HPDI32_PLX_INTCSR,	0		},
	{ HPDI32_PLX_CNTRL,		0		},
	{ HPDI32_PLX_PCIHIDR,	0		},
	{ HPDI32_PLX_PCIHREV,	0		},

	{ HPDI32_PLX_DMAMODE0,	0		},
	{ HPDI32_PLX_DMAPADR0,	0		},
	{ HPDI32_PLX_DMALADR0,	0		},
	{ HPDI32_PLX_DMASIZ0,	0		},
	{ HPDI32_PLX_DMADPR0,	0		},
	{ HPDI32_PLX_DMAMODE1,	0		},
	{ HPDI32_PLX_DMAPADR1,	0		},
	{ HPDI32_PLX_DMALADR1,	0		},
	{ HPDI32_PLX_DMASIZ1,	0		},
	{ HPDI32_PLX_DMADPR1,	0		},
	{ HPDI32_PLX_DMACSR0,	0		},
	{ HPDI32_PLX_DMACSR1,	0		},
	{ HPDI32_PLX_DMAARB,	0		},
	{ HPDI32_PLX_DMATHR,	0		},

	{ HPDI32_PLX_OPLFIS,	0		},
	{ HPDI32_PLX_OPLFIM,	0		},
	{ HPDI32_PLX_IQP,		0		},
	{ HPDI32_PLX_OQP,		0		},
	{ HPDI32_PLX_MQCR,		0		},
	{ HPDI32_PLX_QBAR,		0		},
	{ HPDI32_PLX_IFHPR,		0		},
	{ HPDI32_PLX_IFTPR,		0		},
	{ HPDI32_PLX_IPHPR,		0		},
	{ HPDI32_PLX_IPTPR,		0		},
	{ HPDI32_PLX_OFHPR,		0		},
	{ HPDI32_PLX_OFTPR,		0		},
	{ HPDI32_PLX_OPHPR,		0		},
	{ HPDI32_PLX_OPTPR,		0		},
	{ HPDI32_PLX_QSR,		0		},

	/* These are specific to the PCI9656. */
	{ HPDI32_PLX_LMISC1,	0		},
	{ HPDI32_PLX_PROT_AREA,	0		},
	{ HPDI32_PLX_LMISC2,	0		},
	{ HPDI32_PLX_DMDAC,		0		},
	{ HPDI32_PLX_ARB,		0		},
	{ HPDI32_PLX_ABTADR,	0		},

	{ HPDI32_PLX_DMADAC0,	0		},
	{ HPDI32_PLX_DMADAC1,	0		},

	/* These are common to all boards, depending on firmware support. */
	{ HPDI32_GSC_FRR,		1		},
	{ HPDI32_GSC_BCR,		1		},
	{ HPDI32_GSC_BSR,		1		},
	{ HPDI32_GSC_TAR,		1		},
	{ HPDI32_GSC_RAR,		1		},
	{ HPDI32_GSC_FSR,		1		},
	{ HPDI32_GSC_FDR,		1		},
	{ HPDI32_GSC_TSVLCR,	1		},
	{ HPDI32_GSC_TLVLCR,	1		},
	{ HPDI32_GSC_TLILCR,	1		},
	{ HPDI32_GSC_RSCR,		1		},
	{ HPDI32_GSC_RLCR,		1		},
	{ HPDI32_GSC_ICR,		1		},
	{ HPDI32_GSC_ISR,		1		},
	{ HPDI32_GSC_TCDR,		1		},
	{ HPDI32_GSC_TFSR,		1		},
	{ HPDI32_GSC_RFSR,		1		},
	{ HPDI32_GSC_TFWR,		1		},
	{ HPDI32_GSC_RFWR,		1		},
	{ HPDI32_GSC_IELR,		1		},
	{ HPDI32_GSC_IHLR,		1		}

};



/*****************************************************************************
*
*	Function:	register_read_test
*
*	Purpose:
*
*		Verify that all readable registers can be read.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

int register_read_test(int fd)
{
	int		i;
	__u32	reg;
	int		status	= 0;

	label("HPDI32_IOCTL_REG_READ");

	for (i = 0; (status == 0) && (i < ARRAY_ELEMENTS(_reg_list)); i++)
		status	= hpdi32_reg_read(fd, _reg_list[i].reg, &reg, 0);

	printf("%s (%d)\n", (status == -1) ? "FAIL" : "PASS", i);
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	return(status);
}



/*****************************************************************************
*
*	Function:	register_write_test
*
*	Purpose:
*
*		Verify that all writeable registers can be written.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

int register_write_test(int fd)
{
	int		i;
	__u32	reg;
	int		ro		= 0;
	int		rw		= 0;
	int		status	= 0;

	label("HPDI32_IOCTL_REG_WRITE");

	for (i = 0; (status == 0) && (i < ARRAY_ELEMENTS(_reg_list)); i++)
	{
		hpdi32_reg_read(fd, _reg_list[i].reg, &reg, 0);
		status	= hpdi32_reg_write(fd, _reg_list[i].reg, reg, 0);

		if (((_reg_list[i].writeable) && (status)) ||
			((!_reg_list[i].writeable) && (!status)))
		{
			/* If its RW then there should be no error. */
			/* It its RO then there should be an error. */
			status	= 1;
			break;
		}
		else
		{
			status	= 0;

			if (_reg_list[i].writeable)
				rw++;
			else
				ro++;
		}
	}

	printf(	"%s (%d: rw %d, ro %d)\n",
			(status == -1) ? "FAIL" : "PASS",
			i,
			rw,
			ro);
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	return(status);
}



