/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 3 $ */
/* $Date: 9/28/06 5:33p $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.17.0-GSC-DN/tx/tx.c $
 *
 * 3     9/28/06 5:33p Dnewbold
 * Updated per changes to the 1.18 driver.
 *
 * 2     2/01/06 3:05p Dnewbold
 * Updated and modified to use the utility sources and the I/O buffers.
 *
 * 1     12/19/05 1:55p Dnewbold
*/

#include <stdio.h>
#include <string.h>
#include <sys/time.h>

#include "HPDI32DocSrcLib.h"
#include "main.h"
#include "util_label.h"
#include "util_scheduler.h"



/* variables	*************************************************************/

// All of the setting are made here.
#define	ALMOST_EMPTY		16
#define	ALMOST_FULL			16
#define	CABLE_COMMAND_0		HPDI32_GSC_BCR_D0_FV
#define	CABLE_COMMAND_1		HPDI32_GSC_BCR_D1_IN // LV
#define	CABLE_COMMAND_2		HPDI32_GSC_BCR_D2_IN // SV
#define	CABLE_COMMAND_3		HPDI32_GSC_BCR_D3_RR
#define	CABLE_COMMAND_4		HPDI32_GSC_BCR_D4_TR
#define	CABLE_COMMAND_5		HPDI32_GSC_BCR_D5_TE
#define	CLOCK_DIV			0
#define	LV_OFF				256	// 12.8us at 20MHz
#define	LV_ON				384	// 19.2us at 20MHz
#define	LV_SV				HPDI32_GSC_BCR_STATUS_VALID
#define	OSC_REF				20000000L
#define	REMOTE_THROTTLE		0	// HPDI32_GSC_BCR_TX_THROTTLE
#define	SV_ON				760	// 38us at 20MHz
#define	TRI_STATE			0	// HPDI32_GSC_BCR_TEST_MODE
#define	TX_ENABLE			HPDI32_GSC_BCR_TX_ENABLE
#define	TX_IO_MODE			HPDI32_IO_MODE_DMDMA
#define	TX_IO_SAMPLES		HPDI32_IO_SAMPLES_MAX
#define	TX_IO_TIMEOUT		30
#define	TX_START			HPDI32_GSC_BCR_TX_START
#define	TX_STOP_AUTO		HPDI32_GSC_BCR_TX_AUTO_STOP_NO
#define	USE_IO_BUFFERS		1



/* variables	*************************************************************/

static	__u32	_tx_buf[1L * 1000L * 1000L];
static	__u32*	_tx_ptr;
static	long	_tx_siz;



static int _tx_data_config_check(
	int		fd,
	__u32	enable,
	__u32	remote,
	__u32	start,
	__u32	auto_stop_no)
{
	int		errs	= 0;
	__u32	frr;
	__u32	fsr;

	label("Configuration");

	for (;;)	// A convenience loop.
	{
		errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FRR, &frr, 1);

		if (errs)
			break;

		errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FSR, &fsr, 1);

		if (errs)
			break;

		if (enable != HPDI32_GSC_BCR_TX_ENABLE)
		{
			errs++;
			printf("FAIL <---  (Tx Enable)\n");
			break;
		}

		if ((start) && (start != HPDI32_GSC_BCR_TX_START))
		{
			errs++;
			printf("FAIL <---  (Tx Start)\n");
			break;
		}

		if ((auto_stop_no) && (auto_stop_no != HPDI32_GSC_BCR_TX_AUTO_STOP_NO))
		{
			errs++;
			printf("FAIL <---  (Tx Auto-Stop Disable)\n");
			break;
		}

		if (	(((frr & HPDI32_GSC_FRR_FSR) == 0) ||
				 ((fsr & HPDI32_GSC_FSR_GPIO6_TXAS) == 0)) &&
				(auto_stop_no))
		{
			errs++;
			printf("FAIL <---  (Tx Auto-Stop Disable is not available)\n");
			break;
		}

		if ((remote) && (remote != HPDI32_GSC_BCR_TX_THROTTLE))
		{
			errs++;
			printf("FAIL <---  (Tx Throttle)\n");
			break;
		}

		// Tx Start vs Tx Remote Throttle

		if ((remote) && (start))
		{
			errs++;
			printf("FAIL <---  (remote and local throttling)\n");
			break;
		}
		else if ((remote) || (start))
		{
		}
		else
		{
			errs++;
			printf("FAIL <---  (neither remote nor local throttling)\n");
			break;
		}

		printf("PASS\n");
		break;
	}

	return(errs);
}



static int _tx_data_control_check(
	int		fd,
	__u32	remote,
	__u32	start,
	__u32	auto_stop_no,
	__u32*	block)
{
	int	errs	= 0;

	label("Control");

	if ((remote) && (start))
	{
		block[0]	= 0;
		errs++;
		printf("FAIL <---  (invalid)\n");
	}
	else if (remote)
	{
		block[0]	= _tx_siz;
		printf("Remote  (via Cable Command 3)\n");
	}
	else if (start == 0)
	{
		block[0]	= 0;
		errs++;
		printf("FAIL <---  (none)\n");
	}
	else if (auto_stop_no)
	{
		block[0]	= _tx_siz;
		printf("Local  (via Tx Start and Tx Auto-Stop Disable)\n");
	}
	else
	{
		block[0]	= 4096;
		printf("Local  (via repetative Tx Start)\n");
	}

	return(errs);
}



static int _tx_data(
	int		fd,
	int		errs,
	__u32	enable,
	__u32	remote,
	__u32	start,
	__u32	auto_stop_no,
	int		io_buffers)
{
	struct timeval	begin;
	__u32			block;
	struct timeval	end;
	int				err	= errs;
	int				i;
	int				init;
	int				k		= 1000;
	long			ms;
	const char*		psz;
	__u32*			ptr;
	float			rate;
	long			sent	= 0;
	long			total;

	errs	= 0;

	label("Data Transmission");
	label_level_inc();

	if (err)
	{
		printf("(skipped due to errors)\n");
	}
	else
	{
		printf("\n");

		errs	+= _tx_data_config_check(	fd,
											enable,
											remote,
											start,
											auto_stop_no);
		errs	+= _tx_data_control_check(	fd,
											remote,
											start,
											auto_stop_no,
											&block);

		label("Transmitting Data");
		fflush(stdout);
	}

	if (err)
	{
	}
	if ((errs) || (block == 0))
	{
		printf("aborted due to errors\n");
	}
	else
	{
		if (auto_stop_no)
			start	= 0;	// Tx Start should already be set.

		block	/= 4;		// Convert to samples.
		init	= 1;
		gettimeofday(&begin, NULL);

		for (;;)
		{
			if (init)
			{
				init	= 0;
				ptr		= (__u32*) _tx_ptr;
				total	= _tx_siz / 4;
			}

			if (io_buffers)
				i	= hpdi32_write(fd, NULL, total, 1);
			else
				i	= hpdi32_write(fd, ptr, total, 1);

			if (i < 0)
			{
				errs++;
				break;
			}

			if (start)
			{
				errs	+= hpdi32_reg_mod(fd, HPDI32_GSC_BCR, start, start, 1);

				if (errs)
					break;
			}

			ptr		+= i;
			sent	+= i;
			total	-= i;

			if (total > 0)
				continue;

			init	= 1;
			gettimeofday(&end, NULL);

			if (end.tv_sec > (begin.tv_sec + 4))
				break;
		}

		if (errs)
		{
			printf("FAIL\n");
		}
		else
		{
			printf("PASS\n");
			label_level_inc();

			// Total Data
			label("Total Sent");
			sent	*= 4;
			psz		= label_format_k(sent, k);
			printf("%s Bytes  (1K = %d)\n", psz, k);

			// Transmittion Time
			label("Period");
			ms	= 1000L * (end.tv_sec - begin.tv_sec);
			ms	-= (long) (begin.tv_usec / 1000);
			ms	+= (long) (end.tv_usec / 1000);
			printf("%lu.%03lus\n", ms / 1000, ms % 1000);

			// Throughput
			label("Throughput");
			rate	= (1000. * sent) / ms;
			psz		= label_format_f(rate, k);
			printf("%s bytes/second\n", psz);

			label_level_dec();
		}
	}

	label_level_dec();
	return(errs);
}



static int _tx_fifo_config(int fd, __u32 af, __u32 ae)
{
	int		errs	= 0;
	__u32	tar;

	label("Tx FIFO");
	printf("\n");
	label_level_inc();

	label("Almost Full");
	printf("%lu\n", (long) af);

	label("Almost Empty");
	printf("%lu\n", (long) ae);

	label("Configuration");
	tar		= ((af & 0xFFFF) << 16) | (ae & 0xFFFF);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_TAR, tar, 0);
	errs	+= hpdi32_reg_mod(	fd,
								HPDI32_GSC_BCR,
								HPDI32_GSC_BCR_TX_FIFO_RESET,
								HPDI32_GSC_BCR_TX_FIFO_RESET,
								0);
	printf("%s\n", errs ? "FAIL" : "PASS");

	label_level_dec();
	return(errs);
}




static int _tx_start(int fd, __u32 tx_start, __u32 tx_stop, int verbose)
{
	int		errs	= 0;
	__u32	frr;
	__u32	fsr;

	if (verbose)
	{
		label("Transmission");
		printf("\n");
	}

	label_level_inc();

	if (verbose)
	{
		// Tx Start
		label("Tx Start");

		if (tx_start == HPDI32_GSC_BCR_TX_START)
		{
			printf("Yes\n");
		}
		else if (tx_start == 0)
		{
			printf("No\n");
		}
		else
		{
			errs++;
			printf("FAIL <---\n");
		}

		// Tx Auto Stop
		label("Tx Auto-Stop");

		if (tx_stop == HPDI32_GSC_BCR_TX_AUTO_STOP_NO)
		{
			printf("Disable\n");
		}
		else if (tx_stop == 0)
		{
			printf("Yes\n");
		}
		else
		{
			errs++;
			printf("FAIL <---\n");
		}
	}

	// Configuration

	if (verbose)
		label("Configuration");

	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FRR, &frr, 0);
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FSR, &fsr, 0);
	errs	+= hpdi32_reg_mod(	fd,
								HPDI32_GSC_BCR,
								tx_start,
								HPDI32_GSC_BCR_TX_START,
								0);
	errs	+= hpdi32_reg_mod(	fd,
								HPDI32_GSC_BCR,
								tx_stop,
								HPDI32_GSC_BCR_TX_AUTO_STOP_NO,
								0);

	if (errs)
	{
		if (verbose)
			printf("FAIL <---\n");
	}
	else if (	(((frr & HPDI32_GSC_FRR_FSR) == 0) ||
				 ((fsr & HPDI32_GSC_FSR_GPIO6_TXAS) == 0)) &&
				(tx_stop == HPDI32_GSC_BCR_TX_AUTO_STOP_NO))
	{
		errs++;

		if (verbose)
			printf("FAIL <---  (Tx Auto-Stop Disable not available)\n");
	}
	else
	{
		if (verbose)
			printf("PASS\n");
	}

	label_level_dec();
	return(errs);
}



static int _tx_enable(int fd, __u32 enable)
{
	int	errs	= 0;

	label("Tx Enable");
	printf("\n");
	label_level_inc();

	// Enable?
	label("Enable");

	if (enable == HPDI32_GSC_BCR_TX_ENABLE)
	{
		printf("Yes\n");
	}
	else if (enable == 0)
	{
		printf("No\n");
	}
	else
	{
		errs++;
		printf("FAIL <---\n");
	}

	// Configure
	label("Configuration");
	errs	+= hpdi32_reg_mod(	fd,
								HPDI32_GSC_BCR,
								enable,
								HPDI32_GSC_BCR_TX_ENABLE,
								0);
	printf("%s\n", errs ? "FAIL" : "PASS");

	label_level_dec();
	return(errs);
}



static int _cbl_cmd_5(int fd, __u32 func, __u32 tri_state)
{
	int		errs	= 0;
	__u32	frr;
	__u32	fsr;

	label("Cable Command 5");
	printf("\n");
	label_level_inc();

	// Function
	label("Function");

	switch (func)
	{
		case HPDI32_GSC_BCR_D5_TE:		printf("Tx Enabled\n");			break;
		case HPDI32_GSC_BCR_D5_IN:		printf("GPIO 4 Input\n");		break;
		case HPDI32_GSC_BCR_D5_OUT_H:	printf("GPIO 4 Output Hi\n");	break;
		case HPDI32_GSC_BCR_D5_OUT_L:	printf("GPIO 4 Output Low\n");	break;
		default:						printf("FAIL <---\n");
										errs++;
	}

	// Tri-State When Tx is disabled?
	label("Tri-State when Low");

	if (tri_state == HPDI32_GSC_BCR_TEST_MODE)
	{
		printf("Yes\n");
	}
	else if (tri_state == 0)
	{
		printf("No\n");
	}
	else
	{
		errs++;
		printf("FAIL <---\n");
	}

	// Apply the setting.
	label("Configuration");
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FRR, &frr, 0);
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FSR, &fsr, 0);
	errs	+= hpdi32_reg_mod(	fd,
								HPDI32_GSC_BCR,
								func,
								HPDI32_GSC_BCR_D5_MASK,
								0);
	errs	+= hpdi32_reg_mod(	fd,
								HPDI32_GSC_BCR,
								tri_state,
								HPDI32_GSC_BCR_TEST_MODE,
								0);

	if (errs)
	{
		printf("FAIL <---\n");
	}
	else if (	(((frr & HPDI32_GSC_FRR_FSR) == 0) ||
				 ((fsr & HPDI32_GSC_FSR_GPIO_0_5) == 0)) &&
				(func != HPDI32_GSC_BCR_D5_TE))
	{
		errs++;
		printf("FAIL <---  (GPIO 4 not available)\n");
	}
	else
	{
		printf("PASS\n");
	}

	label_level_dec();
	return(errs);
}



static int _cbl_cmd_4(int fd, __u32 func)
{
	int		errs	= 0;
	__u32	frr;
	__u32	fsr;

	label("Cable Command 4");
	printf("\n");
	label_level_inc();

	// Function
	label("Function");

	switch (func)
	{
		case HPDI32_GSC_BCR_D4_TR:		printf("Tx Data Ready\n");		break;
		case HPDI32_GSC_BCR_D4_IN:		printf("GPIO 3 Input\n");		break;
		case HPDI32_GSC_BCR_D4_OUT_H:	printf("GPIO 3 Output Hi\n");	break;
		case HPDI32_GSC_BCR_D4_OUT_L:	printf("GPIO 3 Output Low\n");	break;
		default:						printf("FAIL <---\n");
										errs++;
	}

	// Apply the setting.
	label("Configuration");
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FRR, &frr, 0);
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FSR, &fsr, 0);
	errs	+= hpdi32_reg_mod(	fd,
								HPDI32_GSC_BCR,
								func,
								HPDI32_GSC_BCR_D4_MASK,
								0);

	if (errs)
	{
		printf("FAIL <---\n");
	}
	else if (	(((frr & HPDI32_GSC_FRR_FSR) == 0) ||
				 ((fsr & HPDI32_GSC_FSR_GPIO_0_5) == 0)) &&
				(func != HPDI32_GSC_BCR_D4_TR))
	{
		errs++;
		printf("FAIL <---  (GPIO 3 not available)\n");
	}
	else
	{
		printf("PASS\n");
	}

	label_level_dec();
	return(errs);
}



static int _cbl_cmd_3(int fd, __u32 func, __u32 remote)
{
	int		errs	= 0;
	__u32	frr;
	__u32	fsr;

	label("Cable Command 3");
	printf("\n");
	label_level_inc();

	// Function
	label("Function");

	switch (func)
	{
		case HPDI32_GSC_BCR_D3_RR:		printf("Rx Ready\n");			break;
		case HPDI32_GSC_BCR_D3_IN:		printf("GPIO 2 Input\n");		break;
		case HPDI32_GSC_BCR_D3_OUT_H:	printf("GPIO 2 Output Hi\n");	break;
		case HPDI32_GSC_BCR_D3_OUT_L:	printf("GPIO 2 Output Low\n");	break;
		default:						printf("FAIL <---\n");
										errs++;
	}

	// Remote Throttling?
	label("Remote Throttle");

	if (remote == HPDI32_GSC_BCR_TX_THROTTLE)
	{
		printf("Yes\n");
	}
	else if (remote == 0)
	{
		printf("No\n");
	}
	else
	{
		errs++;
		printf("FAIL <---\n");
	}

	// Apply the setting.
	label("Configuration");
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FRR, &frr, 0);
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FSR, &fsr, 0);
	errs	+= hpdi32_reg_mod(	fd,
								HPDI32_GSC_BCR,
								func,
								HPDI32_GSC_BCR_D3_MASK,
								0);
	errs	+= hpdi32_reg_mod(	fd,
								HPDI32_GSC_BCR,
								remote,
								HPDI32_GSC_BCR_TX_THROTTLE,
								0);

	if (errs)
	{
		printf("FAIL <---\n");
	}
	else if (	(((frr & HPDI32_GSC_FRR_FSR) == 0) ||
				 ((fsr & HPDI32_GSC_FSR_GPIO_0_5) == 0)) &&
				(func != HPDI32_GSC_BCR_D3_RR))
	{
		errs++;
		printf("FAIL <---  (GPIO 2 not available)\n");
	}
	else
	{
		printf("PASS\n");
	}

	label_level_dec();
	return(errs);
}



static int _cbl_cmd_2(int fd, __u32 func, __u32 on, __u32 tx_clock)
{
	int			errs	= 0;
	__u32		frr;
	__u32		fsr;
	float		period;
	const char*	psz;

	label("Cable Command 2");
	printf("\n");
	label_level_inc();

	// Function
	label("Function");

	switch (func)
	{
		case HPDI32_GSC_BCR_D2_SV:		printf("Status Valid\n");		break;
		case HPDI32_GSC_BCR_D2_IN:		printf("GPIO 1 Input\n");		break;
		case HPDI32_GSC_BCR_D2_OUT_H:	printf("GPIO 1 Output Hi\n");	break;
		case HPDI32_GSC_BCR_D2_OUT_L:	printf("GPIO 1 Output Low\n");	break;
		default:						printf("FAIL <---\n");
										errs++;
	}

	// On period

	if (func == HPDI32_GSC_BCR_D2_SV)
	{
		label("Status Valid On");
		printf("%lu cycles", (long) on);
		period	= ((float) on) / tx_clock;
		psz		= label_format_f(period, 1000);
		printf("  (%ss", psz);
		psz		= label_format_k(tx_clock, 1000);
		printf(" at %sHz)\n", psz);
	}

	// Apply the setting.
	label("Configuration");
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FRR, &frr, 0);
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FSR, &fsr, 0);
	errs	+= hpdi32_reg_mod(	fd,
								HPDI32_GSC_BCR,
								HPDI32_GSC_BCR_D2_MASK,
								func,
								0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_TSVLCR, on, 0);

	if (errs)
	{
		printf("FAIL <---\n");
	}
	else if (	(((frr & HPDI32_GSC_FRR_FSR) == 0) ||
				 ((fsr & HPDI32_GSC_FSR_GPIO_0_5) == 0)) &&
				(func != HPDI32_GSC_BCR_D2_SV))
	{
		errs++;
		printf("FAIL <---  (GPIO 1 not available)\n");
	}
	else
	{
		printf("PASS\n");
	}

	label_level_dec();
	return(errs);
}



static int _cbl_cmd_1(
	int		fd,
	__u32	func,
	__u32	on,
	__u32	off,
	__u32	tx_clock,
	__u32	sv)
{
	int			errs	= 0;
	__u32		frr;
	__u32		fsr;
	float		period;
	const char*	psz;

	label("Cable Command 1");
	printf("\n");
	label_level_inc();

	// Function
	label("Function");

	switch (func)
	{
		case HPDI32_GSC_BCR_D1_LV:		printf("Line Valid\n");			break;
		case HPDI32_GSC_BCR_D1_IN:		printf("GPIO 0 Input\n");		break;
		case HPDI32_GSC_BCR_D1_OUT_H:	printf("GPIO 0 Output Hi\n");	break;
		case HPDI32_GSC_BCR_D1_OUT_L:	printf("GPIO 0 Output Low\n");	break;
		default:						printf("FAIL <---\n");
										errs++;
	}

	// Line Valid On period

	if (func == HPDI32_GSC_BCR_D1_LV)
	{
		label("Line Valid On");
		printf("%lu cycles", (long) on);
		period	= ((float) on) / tx_clock;
		psz		= label_format_f(period, 1000);
		printf("  (%ss", psz);
		psz		= label_format_k(tx_clock, 1000);
		printf(" at %sHz)\n", psz);
	}

	// Line Valid Off period

	if (func == HPDI32_GSC_BCR_D1_LV)
	{
		label("Line Valid Off");
		printf("%lu cycles", (long) off);
		period	= ((float) off) / tx_clock;
		psz		= label_format_f(period, 1000);
		printf("  (%ss", psz);
		psz		= label_format_k(tx_clock, 1000);
		printf(" at %sHz)\n", psz);
	}

	// Mirror Status Valid High?

	if (func == HPDI32_GSC_BCR_D1_LV)
	{
		label("High w/ Status Valid");

		if (sv == HPDI32_GSC_BCR_STATUS_VALID)
		{
			printf("Yes\n");
		}
		else if (sv == 0)
		{
			printf("No\n");
		}
		else
		{
			errs++;
			printf("FAIL <---\n");
		}
	}

	// Apply the setting.
	label("Configuration");
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FRR, &frr, 0);
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FSR, &fsr, 0);
	errs	+= hpdi32_reg_mod(	fd,
								HPDI32_GSC_BCR,
								func,
								HPDI32_GSC_BCR_D1_MASK,
								0);
	errs	+= hpdi32_reg_mod(	fd,
								HPDI32_GSC_BCR,
								sv,
								HPDI32_GSC_BCR_STATUS_VALID,
								0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_TLVLCR, on, 0);
	errs	+= hpdi32_reg_write(fd, HPDI32_GSC_TLILCR, off, 0);

	if (errs)
	{
		printf("FAIL <---\n");
	}
	else if (	(((frr & HPDI32_GSC_FRR_FSR) == 0) ||
				 ((fsr & HPDI32_GSC_FSR_GPIO_0_5) == 0)) &&
				(func != HPDI32_GSC_BCR_D1_LV))
	{
		errs++;
		printf("FAIL <---  (GPIO 0 not available)\n");
	}
	else
	{
		printf("PASS\n");
	}

	label_level_dec();
	return(errs);
}



static int _cbl_cmd_0(int fd, __u32 func)
{
	int		errs	= 0;
	__u32	frr;
	__u32	fsr;

	label("Cable Command 0");
	printf("\n");
	label_level_inc();

	// Function
	label("Function");

	switch (func)
	{
		case HPDI32_GSC_BCR_D0_FV:		printf("Frame Valid\n");		break;
		case HPDI32_GSC_BCR_D0_IN:		printf("GPIO 6 Input\n");		break;
		case HPDI32_GSC_BCR_D0_OUT_H:	printf("GPIO 6 Output Hi\n");	break;
		case HPDI32_GSC_BCR_D0_OUT_L:	printf("GPIO 6 Output Low\n");	break;
		default:						printf("FAIL <---\n");
										errs++;
	}

	// Apply the setting.
	label("Configuration");
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FRR, &frr, 0);
	errs	+= hpdi32_reg_read(fd, HPDI32_GSC_FSR, &fsr, 0);
	errs	+= hpdi32_reg_mod(	fd,
								HPDI32_GSC_BCR,
								HPDI32_GSC_BCR_D0_MASK,
								func,
								0);

	if (errs)
	{
		printf("FAIL <---\n");
	}
	else if (	(((frr & HPDI32_GSC_FRR_FSR) == 0) ||
				 ((fsr & HPDI32_GSC_FSR_GPIO6_TXAS) == 0)) &&
				(func != HPDI32_GSC_BCR_D0_FV))
	{
		errs++;
		printf("FAIL <---  (GPIO 6 not available)\n");
	}
	else
	{
		printf("PASS\n");
	}

	label_level_dec();
	return(errs);
}



static int _tx_io_config(int fd, __u32 mode, __u32 timeout, __u32 samples)
{
	int					errs	= 0;
	hpdi32_io_config_t	io;
	const char*			psz;

	label("I/O Settings");
	printf("\n");
	label_level_inc();

	// Mode
	label("Mode");

	switch (mode)
	{
		case HPDI32_IO_MODE_DMDMA:	printf("DMDMA\n");	break;
		case HPDI32_IO_MODE_PIO:	printf("PIO\n");	break;
		default:					printf("FAIL <---\n");
									errs++;
	}

	// Timeout
	label("Timeout");
	printf("%lu Second%s\n", (long) timeout, (timeout == 1) ? "" : "s");

	// Buffer size
	label("Buffer Size");
	psz	= label_format_k(samples, 1024);
	printf("%s Samples\n", psz);

	// Apply the settings.
	label("Configuration");
	memset(&io, 0, sizeof(io));
	io.read.mode			= HPDI32_IO_MODE_NO_CHANGE;
	io.read.timeout			= HPDI32_IO_TIMEOUT_NO_CHANGE;
	io.read.samples			= HPDI32_IO_SAMPLES_NO_CHANGE;
	io.read.sample_bits		= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.read.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
	io.write.mode			= mode;
	io.write.timeout		= timeout;
	io.write.samples		= samples;
	io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.write.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
	errs					= hpdi32_io_config(fd, &io, 0);
	printf("%s\n", errs ? "FAIL" : "PASS");

	label_level_dec();
	return(errs);
}



static int _clocking(int fd, __u32 osc_ref, __u32 clock_div, __u32* tx_clock)
{
	__u32		clock;
	int			errs;
	float		period;
	const char*	psz;

	label("Clocking");
	printf("\n");
	label_level_inc();

	// The reference clock.
	label("Reference");
	psz		= label_format_k(osc_ref, 1000);
	printf("%sHz", psz);
	period	= 1.0 / osc_ref;
	psz		= label_format_f(period, 1000);
	printf("  (%ss, according to #define)\n", psz);

	// The clock divider.
	label("Clock Divider");
	printf(	"%lu  (/%lu)\n",
			(long) clock_div,
			(long) (clock_div ? (clock_div * 2) : 1));

	// Tx Clock
	label("Tx Clock");
	clock	= clock_div ? (osc_ref / (clock_div * 2)) : osc_ref;
	period	= 1.0 / clock;
	psz		= label_format_k(clock, 1000);
	printf("%sHz", psz);
	psz		= label_format_f((float) period, 1000);
	printf("  (%ss)\n", psz);
	tx_clock[0]	= clock;

	// Configure
	label("Configuration");
	errs	= hpdi32_reg_write(fd, HPDI32_GSC_TCDR, clock_div, 0);
	printf("%s\n", errs ? "FAIL" : "PASS");

	label_level_dec();
	return(errs);
}



static int _dev_reset(int fd)
{
	int	errs;

	label("Reseting Device");
	errs	= hpdi32_reg_mod(	fd,
								HPDI32_GSC_BCR,
								~0L,
								HPDI32_GSC_BCR_BOARD_RESET,
								0);
	printf("%s\n", errs ? "FAIL" : "PASS");
	return(errs);
}



/******************************************************************************
*
*	Function:	_init_buffer
*
*	Purpose:
*
*		Initialize the data buffer.
*
*	Arguments:
*
*		fd			The file descriptor for the device to access.
*
*		io_buffers	Use I/O buffers for data transfer?
*
*	Returned:
*
*		>= 0		The number of errors seen.
*
******************************************************************************/

static int _init_buffer(int fd, int io_buffers)
{
	int					errs	= 0;
	hpdi32_io_config_t	io;
	hpdi32_mmap_t		mem;
	const char*			str;
	unsigned long		ul;

	label("Tx Data Buffer");
	printf("\n");
	label_level_inc();

	label("Buffer Source");

	if (io_buffers)
		printf("I/O Buffer (mmap)\n");
	else
		printf("Application Buffer\n");


	label("Configuration");

	if (io_buffers)
	{
		io.read.mode			= HPDI32_IO_MODE_NO_CHANGE;
		io.read.timeout			= HPDI32_IO_TIMEOUT_NO_CHANGE;
		io.read.samples			= HPDI32_IO_SAMPLES_NO_CHANGE;
		io.read.sample_bits		= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
		io.read.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
		io.write.mode			= HPDI32_IO_MODE_NO_CHANGE;
		io.write.timeout		= HPDI32_IO_TIMEOUT_NO_CHANGE;
		io.write.samples		= HPDI32_IO_SAMPLES_NO_CHANGE;
		io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
		io.write.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
		errs	+= hpdi32_io_config(fd, &io, 0);
		errs	+= hpdi32_mmap_info(fd, &mem, 0);
		_tx_ptr	= hpdi32_mmap(fd, &mem.write, 0);
		_tx_siz	= io.write.samples * 4;
		errs	= _tx_ptr ? errs : 1;
	}
	else
	{
		_tx_ptr	= _tx_buf;
		_tx_siz	= sizeof(_tx_buf);
	}

	printf("%s\n", errs ? "FAIL <---" : "PASS");

	label("Buffer Size");
	str	= label_format_k(_tx_siz, 1024);
	printf("%s Bytes\n", str);

	label("Initializing");
	fflush(stdout);

	if (_tx_ptr)
	{
		for (ul = 0; ul < (_tx_siz / 4); ul++)
			_tx_ptr[ul]	= 0x80000000L + ul;
	}

	printf("Done\n");

	label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	_reset_buffer
*
*	Purpose:
*
*		Release the buffer is mmap has been used.
*
*	Arguments:
*
*		fd			The file descriptor for the device to access.
*
*		io_buffers	Use I/O buffers for data transfer?
*
*	Returned:
*
*		>= 0		The number of errors seen.
*
******************************************************************************/

static int _reset_buffer(int fd, int io_buffers)
{
	int				errs	= 0;
	hpdi32_mmap_t	mem;

	label("Data Buffer Reset");
	printf("\n");
	label_level_inc();

	label("Buffer Source");

	if (io_buffers)
		printf("I/O Buffer (mmap)\n");
	else
		printf("Application Buffer\n");


	label("Release");

	if (io_buffers)
	{
		errs	+= hpdi32_mmap_info(fd, &mem, 0);
		errs	+= hpdi32_munmap(_tx_ptr, &mem.write, 0);
		_tx_ptr	= _tx_buf;
		_tx_siz	= sizeof(_tx_buf);
	}

	printf("%s\n", errs ? "FAIL <---" : "PASS");

	label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	_config_board
*
*	Purpose:
*
*		Configure the board for the desired operation.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _config_board(int fd)
{
	int		errs	= 0;
	__u32	tx_clock;

	label("Configuring Device");
	printf("\n");
	label_level_inc();

	errs	+= _dev_reset(fd);
	errs	+= _tx_io_config(fd, TX_IO_MODE, TX_IO_TIMEOUT, TX_IO_SAMPLES);
	errs	+= _tx_fifo_config(fd, ALMOST_FULL, ALMOST_EMPTY);
	errs	+= _clocking(fd, OSC_REF, CLOCK_DIV, &tx_clock);
	errs	+= _cbl_cmd_0(fd, CABLE_COMMAND_0);
	errs	+= _cbl_cmd_1(fd, CABLE_COMMAND_1, LV_ON, LV_OFF, tx_clock, LV_SV);
	errs	+= _cbl_cmd_2(fd, CABLE_COMMAND_2, SV_ON, tx_clock);
	errs	+= _cbl_cmd_3(fd, CABLE_COMMAND_3, REMOTE_THROTTLE);
	errs	+= _cbl_cmd_4(fd, CABLE_COMMAND_4);
	errs	+= _cbl_cmd_5(fd, CABLE_COMMAND_5, TRI_STATE);
	errs	+= _tx_start(fd, TX_START, TX_STOP_AUTO, 1);
	errs	+= _tx_enable(fd, TX_ENABLE);

	label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	tx_test
*
*	Purpose:
*
*		Perform a data transmission test.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int tx_test(int fd)
{
	int	errs	= 0;

	label("Tx Test");
	printf("\n");
	label_level_inc();

	errs	+= _config_board(fd);
	errs	+= _init_buffer(fd, USE_IO_BUFFERS);
	errs	+= scheduler_adjust(1);
	errs	+= _tx_data(fd,
						errs,
						TX_ENABLE,
						REMOTE_THROTTLE,
						TX_START,
						TX_STOP_AUTO,
						USE_IO_BUFFERS);
	errs	+= scheduler_adjust(0);
	errs	+= _reset_buffer(fd, USE_IO_BUFFERS);

	label_level_dec();
	return(errs);
}



