/* $Revision: 1 $ */
/* $Date: 2/01/06 3:05p $ */

#include <sched.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>

#include "util_scheduler.h"
#include "util_label.h"



/*****************************************************************************
*
*	Function:	scheduler_adjust
*
*	Purpose:
*
*		Adjust the scheduler policy and priority to get the best performance.
*
*	Arguments:
*
*		rt		Enable read-time processing.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int scheduler_adjust(int rt)
{
	int					i;
	struct sched_param	param;
	pid_t				pid;
	int					policy;

	label("Scheduler");
	printf("\n");
	label_level_inc();

	pid						= getpid();
	policy					= rt ? SCHED_RR : SCHED_OTHER;
	param.sched_priority	= (sched_get_priority_min(policy)
							+  sched_get_priority_max(policy)) * 30L / 100;
	sched_setscheduler(pid, policy, &param);

	label("Policy");
	i	= sched_getscheduler(pid);

	switch (i)
	{
		default:			printf("OTHER (%d)\n", i);					break;
		case SCHED_FIFO:	printf("First In-First Out (real time)\n");	break;
		case SCHED_OTHER:	printf("Non real-time\n");					break;
		case SCHED_RR:		printf("Round Robin (real time)\n");		break;
	}

	label("Priority");
	i	= sched_getparam(pid, &param);
	printf("%d\n", (int) param.sched_priority);

	label_level_dec();
	return(0);
}



